// unused code from counterexample.c

/*
 get_loop2() performs a breadth-first search on the SCC, where it tries to find a cycle
 */
static void get_loop2(ltl_formula *f, found_scc_type *found, modelitem **result, int *resultlength, int *loop_length) {
  ltlatom *atom, *nextatom;
  unsigned int bf_index, bf_size, nextitem_index;
  in_loop_result in_loop;
  unsigned int transition_id;
  loopitem *item, *nextitem;
  int found_valid_loop, i;
  ltlarray bf_nodes;
  ltlatom **scc = found->scc;
  int atomnumber = found->number_of_scc_atoms;

  /* we store for each loopitem a bitfield of the encountered goals of UNTILs */
  const size_t loopitem_size = sizeof(loopitem) + 2*get_bitfield_memsize(f->number_of_untils);

  IFLTLDEBUG(ltlmsg("** trying to find loop of lasso in SCC\n");
             ltldeb_inc(););

  init_array(&bf_nodes, loopitem_size, NULL, "bf_nodes");

  /* put first atom into the buffer of the breadth-first search */
  bf_size = 1;
  bf_index = 0;
  item = save_item(f, &bf_nodes, bf_index, 0, 0, found->scc_entry);
  item->number_of_encounters = 0;
    
  /* breadth-first search (for finding the shortest loop in the SCC) */
  found_valid_loop = 0;
  while(!found_valid_loop && bf_index < bf_size) {
    IFLTLDEBUG(ltlmsg("bf_index: %u\n", bf_index););
    item = (loopitem*) get_array_element(&bf_nodes, bf_index);
    atom = item->atom;
    /* find all next atoms in the SCC */
    for (i=0; !found_valid_loop && i<atom->number_of_transitions; i++) {
      IFLTLDEBUG(ltlmsg("transition number: %u\n", i);ltldeb_inc(););
      transition_id = atom->first_transition_id + i;
      nextatom = get_atom_trans(f, transition_id);
      if (is_in_scc(nextatom, atomnumber, scc)) {
        /* store an item for the new atom,
           this item may be overwritten later */
        nextitem_index = bf_size; /* store the new item at the end of the buffer,
                                  * it may be overidden later if bf_size is not increased */
        nextitem = save_item(f, &bf_nodes, nextitem_index, transition_id, bf_index, nextatom);
//        print_out_buffer(f,&bf_nodes,bf_size);
        IFLTLDEBUG(ltlmsg("check if atom is in loop\n", i););
        in_loop = is_atom_in_loop(f, &bf_nodes, nextitem, nextitem_index, loop_length);
        IFLTLDEBUG(ltlmsg("in_loop=%i\n", in_loop););
        switch (in_loop) {
        case item_with_all_matched_untils_in_loop:
          /* we found a matching loop */
          rewind_loop(found, &bf_nodes, nextitem_index, result, resultlength);
          found_valid_loop = 1;
          break;
        case item_with_more_goals_in_loop:
          /* a similiar atom with more goals was found
             before, ignore this node */
          break;
        case no_similiar_item_in_loop:
          /* no similiar atom found before,
             append found atom to end of buffer */
          bf_size++;
          item = nextitem;
          break;
        default:
          fprintf(stderr, "unexpected result from is_atom_in_loop().\nExiting.\n");
          exit(1);
        }
        IFLTLDEBUG(ltldeb_dec(););
      }
    }
    bf_index++;
  }
  if (!found_valid_loop) {
    ltl_print_prolog_graph(f, found, &bf_nodes, bf_size);
    fprintf(stderr, 
            "no loop found in SCC, this is a bug in the model checker!\n");
    fprintf(stderr, 
            "A file containing a dump called 'ltl_graphdump.pl' has been created\n");
    exit(1);
  }
  delete_array(&bf_nodes);
  IFLTLDEBUG(ltlmsg("** found loop of lasso in SCC (length: %i atoms)\n", *loop_length);
             ltldeb_dec(););
}



/*
 * rewind_loop(found_scc, items, solution, result, resultlength):
 *    takes the stack (items and solution) of the breadth first search in the SCC 
 *    and generates a linear trace of atoms from it
 *  found_scc: the SCC, used for the path to the SCC
 *  items: the stack of the breadth-first search
 *  solution: the index of the item of the stack that represents the last node in the search
 *            the item's atom is a duplicate of the loop entry
 *  result: the array of the resulting trace (call by reference)
 *  resultlength: the size of the array      (call by reference)
 */
static void rewind_loop(found_scc_type *fscc, ltlarray *bf_items, unsigned int solution, modelitem **result, int *resultlength) {
  int i,size = 0;
  loopitem *item;
  unsigned int index = solution;
  IFLTLDEBUG(ltlmsg("entered rewind_loop\n");ltldeb_inc(););
  /* determine the size of the loop */
  do {
    size++;
    item = (loopitem*) get_array_element(bf_items, index);
    index = item->previous;
  } while(index != 0);
  IFLTLDEBUG(ltlmsg("size=%i\n",size));
  IFLTLDEBUG(ltlmsg("path length=%i\n",fscc->number_of_path_elements));
  printf("LTL model checking: found counter-example (lasso-form): intro length = %i, path in SCC of length = %i\n",
         fscc->number_of_path_elements, size);
  *resultlength = fscc->number_of_path_elements + size;
  *result = (modelitem*) ltl_malloc(sizeof(modelitem) * (*resultlength));
  /* copy the path into the SCC to the beginning of the counter-example */
  for (i=0; i<fscc->number_of_path_elements; i++) {
    (*result)[i] = create_model_item(fscc->path[i],0);
  }
  /* copy the found loop into the counter-example */
  item = (loopitem*) get_array_element(bf_items, solution);
  index = item->previous;
  for (i=0; i<size; i++) {
    item = (loopitem*) get_array_element(bf_items, index);
    (*result)[*resultlength-i-1] = create_model_item(item->atom,item->transition_id);
    index = item->previous;
  }
  IFLTLDEBUG(ltlmsg("rewind_loop: exit\n");ltldeb_dec(););
}


/*
 * is_atom_in_loop(f, bf_nodes, newitem, loop_length):
 *    checks if the newly added atom is already encountered in the search and
 *    if yes, if all necessary until goals are met in the resulting loop
 *  f: the LTL formula, needed for checking the untils 
 *  bf_nodes: the breadth-first search stack
 *  newitem: the stack item of the breadth-first search containing the newly added atom
 *  newitem_index: the index of the newly added item
 *  loop_length: the size of the found loop if one was found (call by reference)
 *  return value:
 *    no_similiar_item_in_loop:
 *      the atom is encountered for the first time or
 *      since the last time we encountered more goals for untils
 *    item_with_all_matched_untils_in_loop:
 *      a loop was found which is a valid counter-example
 *    item_with_more_goals_in_loop:
 *      the atom was encountered before during the search but since then we have not
 *      encountered more until goals - it makes no sense do visit this atom a second time
 */
static in_loop_result is_atom_in_loop(ltl_formula *f, ltlarray *bf_nodes, loopitem *newitem,
                                      unsigned int newitem_index, int *loop_length) {
  unsigned int first_atom_index, last_atom_index;
  in_loop_result result;
  int found_atom;
  found_atom = find_atom_in_loop(bf_nodes, newitem_index, newitem,
                                 loop_length, &first_atom_index, &last_atom_index);
  if (found_atom) {
    if (check_if_untils_are_matched(f, bf_nodes, newitem_index, first_atom_index)) {
      IFLTLDEBUG( debug_print_loop(f,bf_nodes, newitem_index, first_atom_index); );
      result = item_with_all_matched_untils_in_loop;
    } else {
      if (has_new_goals_encountered(f,bf_nodes,newitem_index,first_atom_index,last_atom_index)) {
        result = no_similiar_item_in_loop;
      } else {
        result = item_with_more_goals_in_loop;
      }
    }
  } else {
    result = no_similiar_item_in_loop;
  }
  return result;
}


/*
 * creates an item for a new atom in the breadth first search
 * returns a link to the item
 */
static loopitem *save_item(ltl_formula *f, ltlarray *bf_nodes, unsigned int pos, 
                           unsigned int transition_id, unsigned int previous_item_pos, 
                           ltlatom *atom) {
  assure_arraysize(bf_nodes, pos+1);
  loopitem *nextitem = (loopitem*) get_array_element(bf_nodes, pos);
  if (nextitem != NULL) {
    nextitem->atom = atom;
    nextitem->previous = previous_item_pos;
    nextitem->transition_id = transition_id;
    store_used_untils(f, nextitem);
  } else {
    fprintf(stderr, "save_item(): nextitem is NULL.\n");
  }
  return nextitem;
}



/*
 * Check if the found loop contains a goal for each used until in the loop.
 * Additionally, it checks if the loop contains a goal for each used until in the path
 * leading to the loop.
 * (TODO: This could be improved. It would be enough to check whether each of the used
 * untils in the path up to the loop have a corresponding goal later, not necessarily in
 * the loop itself.)
 */
static int check_if_untils_are_matched(ltl_formula *f, ltlarray *bf_nodes, int start_item, int circle_start) {
  int result;
  /* we can use the tmp_evaluation bitfield, because the number of 
     UNTILs is always <= the size of the closure */
  bitfield *used_untils = f->tmp_evaluation;
  bitfield *encountered_goals = f->tmp_evaluation2;
  clear_bitfield(used_untils, f->number_of_untils);
  clear_bitfield(encountered_goals, f->number_of_untils);
  
  /* compute which untils are used and which goals are found in the loop*/
  compute_untils_for_items(f, bf_nodes, start_item, circle_start, 1, used_untils, encountered_goals);
  if (is_sub_bitfield(used_untils, encountered_goals, f->number_of_untils)) {
    /* The goals in the loop are all encountered, we still have to check the
     * path leading to the loop */
    IFLTLDEBUG(printf("  Bitfield of used untils: ");
               show_bitfield(used_untils,f->number_of_untils);
               printf("\n  Bitfield of goals      : ");
               show_bitfield(encountered_goals,f->number_of_untils);
               printf("\n") );
    compute_untils_for_items(f, bf_nodes, circle_start, 0, 1, used_untils, encountered_goals);
    result = is_sub_bitfield(encountered_goals, used_untils, f->number_of_untils);
  } else {
    result = 0;
  }
  return result;
}


/* Check if there are new encountered goals between the "newitem" and "last" items that
 * have not been encountered before in the loop, i.e. between "first" and "last".
 */
static int has_new_goals_encountered(ltl_formula *f, ltlarray *bf_nodes, int newitem, int first, int last) {
  bitfield *last_goals = f->tmp_evaluation;
  bitfield *goals_before = f->tmp_evaluation2;
  clear_bitfield(last_goals, f->number_of_untils);
  clear_bitfield(goals_before, f->number_of_untils);
  /* compute which untils are used and which goals are found in the loop*/
  compute_untils_for_items(f, bf_nodes, newitem, last, 0, NULL, last_goals);
  /* and before */
  compute_untils_for_items(f, bf_nodes, last, first, 1, NULL, goals_before);
  return !is_sub_bitfield(last_goals, goals_before, f->number_of_untils);
}


/* checks if the same atom has already been encountered in the breadth-first-search.
 * If yes, it returns the first and last occurrence of the atom in the search and
 * the length of the loop when using the first occurrence as a re-entry point for the loop.
 */
static int find_atom_in_loop(ltlarray *bf_nodes,
                             unsigned int index, loopitem *newitem,
                             int *loop_length, unsigned int *first_index, unsigned int *last_index) {
  loopitem *current = newitem;
  int found_first_matching_atom = 0;
  int found_last_matching_atom = 0;
  *loop_length = 0;
  newitem->number_of_encounters = 0;
  while (!found_first_matching_atom && index>0) {
    *loop_length += 1;
    index = current->previous;
    current = (loopitem *) get_array_element(bf_nodes, index);
    if (current->atom == newitem->atom) {
      if (!found_last_matching_atom) {
        found_last_matching_atom = 1;
        *last_index = index;
        newitem->number_of_encounters = current->number_of_encounters+1;
      }
      /* We try to find the first encounter of this item, continue if the item was previously found */
      if (current->number_of_encounters==0) {
        found_first_matching_atom = 1;
        *first_index = index;
      }
    }
  }
  if (!found_first_matching_atom && found_last_matching_atom) {
    fprintf(stderr, "Last atom found in loop, but not first - this is an invalid internal state, e.g. a BUG!\n");
    exit(1);
  }
  return found_first_matching_atom;
}


/*
 * calculates which untils are true and which goals are encountered in the item
 * the result is stored in the loop item
 */
static void store_used_untils(ltl_formula *f, loopitem *item) {
  bitfield *goalbuffer, *usedbuffer, *evalbuffer;
  unsigned int i, until;
  goalbuffer = until_goals_of_loopitem(item);
  usedbuffer = used_untils_of_loopitem(f,item);
  clear_bitfield(goalbuffer, f->number_of_untils);
  clear_bitfield(usedbuffer, f->number_of_untils);
  evalbuffer = get_atom_evaluation(item->atom);   
  for (i=0; i<f->number_of_untils; i++) {
    until = f->untils[i];
    if (eval(evalbuffer, &(f->closure[until].args.case_until.goal))) {
      setbit(goalbuffer, i, 1);
    }
    if (getbit(evalbuffer,until)) {
      setbit(usedbuffer, i, 1);
    }
  }
}


/* This function computes which untils are used (the until-formula evaluates to true) or
 * whose goals have been fulfilled in the given atoms. The atoms are given by indeces of
 * breadth first search items. 
 *   start_item: the index of the last item in the breadth-first-search
 *   first: the index of the first item in the breadth-first search (first and last seem
 *     to be confused here - that's because the search goes backwards via the
 *     pointers of the bf-items to previous items in the breadth-first-search
 *   include_first: boolean flag if the first element should be included in the aggregation
 *   used_untils: a bitfield that will be filled with the untils that have been used
 *                (it will _not_ initialised to zeros first)
 *   encountered_goals: a bitfield that will be filled with the goals of the untils that have 
 *               been found (it will _not_ initialised to zeros first)
 * The method uses precomputed values for each loop entry (see until_goals_of_loopitem() and
 * used_untils_of_loopitem() above).
 */
static void compute_untils_for_items(ltl_formula *f, ltlarray *bf_nodes, int start_item, int first, int include_first, bitfield *used_untils, bitfield *encountered_goals) {
  unsigned int itemnr = start_item;
  int first_item_encountered = 0;
  loopitem *item;
  while (!first_item_encountered) {
    first_item_encountered = (itemnr == first);
    if (include_first || !first_item_encountered) {
      item = (loopitem *) get_array_element(bf_nodes, itemnr);
      if (used_untils != NULL) {
        add_set_bits(used_untils_of_loopitem(f,item), used_untils, f->number_of_untils);
      }
      if (encountered_goals != NULL) {
        add_set_bits(until_goals_of_loopitem(item), encountered_goals, f->number_of_untils);
      }
    }
    itemnr = item->previous;
  }
}
