:- module(plspec_logger, [log/3, log/2, set_loglevel/1, possible_loglevel/1]).

possible_loglevel(info).
possible_loglevel(warning).
possible_loglevel(error).
possible_loglevel(debug).
possible_loglevel(nothing).

:- dynamic loglevel/1.
loglevel(error).
loglevel(warning).

set_loglevel(X) :-
  retractall(loglevel(_)),
  print(set_loglevel(X)),nl,
  set_loglevel_aux(X).

set_loglevel_aux(debug) :-
  assert(loglevel(debug)),
  assert(loglevel(warning)),
  assert(loglevel(error)),
  assert(loglevel(info)).
set_loglevel_aux(info) :-
  assert(loglevel(info)),
  assert(loglevel(error)),
  assert(loglevel(warning)).
set_loglevel_aux(warning) :-
  assert(loglevel(error)),
  assert(loglevel(warning)).
set_loglevel_aux(error) :-
  assert(loglevel(error)).
set_loglevel_aux(nothing).

log_stream(error,R,Col) :- !,R=user_error,Col=[bold,red].
log_stream(_,user_output,[blue]).

:- load_files(library(system), [when(compile_time), imports([environ/2])]).
:- if(environ(prob_safe_mode,true)).
% avoid loading tools_printing; pathes may not yet be setup
%:- use_module(probsrc(tools_printing),[format_with_colour/4]).
format_with_colour(Stream,Col,Str,Args) :-
   on_exception(error(existence_error(_,_),_), % in case tools_printing not loaded yet
                 tools_printing:format_with_colour(Stream,Col,Str,Args),
                 format(Stream,Str,Args)).
:- else.
format_with_colour(Stream,_,Str,Args) :- format(Stream,Str,Args).
:- endif.

log(X, Format, Args) :-
  (loglevel(X)
    -> log_stream(X,Stream,Colour),
       format(Stream,'~w ~w~`.t~20|',[plspec, X]),
       format_with_colour(Stream,Colour,Format,Args), nl(Stream)
    ; possible_loglevel(X) -> true
    ; format(user_error,'plspec internal error: ~w is not a valid loglevel!', [X])
    ).

log(X, Format) :-
  log(X, Format, []).
