:- module(pltables, [
                     new_table/2, add_column/3,
                     pltable/2, pltable_header/3,
                     delete_tables/0,
                     pltable_row/3, add_row/3,
                     tables_to_html/2, tables_to_html_stream/2,
                     tables_to_latex/2, tables_to_latex_stream/2,
                     tables_to_xml/2, tables_to_xml_stream/2,
                     tables_to_csv/2, tables_to_csv_stream/2
                    ]).

:- use_module(pltables_export).
:- use_module(probsrc(self_check)).
:- use_module(probsrc(module_information)).

:- module_info(group,pltables).
:- module_info(description,'This module is used to generate, store and export coverage tables.').

:- volatile pltable/2, pltable_header/3, pltable_row/3.
:- dynamic pltable/2, pltable_header/3, pltable_row/3.

:- assert_must_succeed((pltables:new_table(a,[]),pltables:delete_tables,\+(pltables:pltable(a,[])))).
delete_tables :-
    retractall(pltable(_,_)),
    retractall(pltable_header(_,_,_)),
    retractall(pltable_row(_,_,_)).

:- assert_must_succeed((pltables:new_table(a,[]),pltables:delete_tables)).
:- assert_must_succeed((pltables:new_table(aa,[]),pltables:new_table(bb,[]),pltables:delete_tables)).
:- assert_must_fail((pltables:new_table(x,[]),pltables:new_table(x,[]),pltables:delete_tables)).
% As the above test fails, delete_tables is not called. The following test serves as a cleanup routine
:- assert_must_succeed(pltables:delete_tables).
new_table(Name, Attributes) :-
    pltable(Name,Attributes) -> false ; assert(pltable(Name, Attributes)).

add_column(TableName, ColumnText, Attributes) :-
    pltable(TableName, _),
    ( retract(pltable_header(TableName, OldHeadings, OldAttributes)) ->
        append(OldAttributes, [Attributes], NewAttributes),
        append(OldHeadings, [ColumnText], NewHeadings),
        assert(pltable_header(TableName, NewHeadings, NewAttributes))
    ; otherwise ->
        assert(pltable_header(TableName, [ColumnText], [Attributes]))
    ).

add_row(TableName, Entries, Attributes) :-
    pltable_header(TableName, Columns, _),
    length(Columns, NrOfColumns),
    length(Entries, NrOfColumns),
    assert(pltable_row(TableName, Entries, Attributes)).

