:- module(pltables_export, [
                            tables_to_html/2,
                            tables_to_html_stream/2,
                            tables_to_latex/2,
                            tables_to_latex_stream/2,
                            tables_to_xml/2,
                            tables_to_xml_stream/2,
                            tables_to_csv/2,
                            tables_to_csv_stream/2
                           ]).

:- use_module(probsrc(self_check)).
:- use_module(probsrc(module_information)).

:- module_info(group,pltables).
:- module_info(description,'This module provides html/latex and codespeed export functionality for the other table modules').

:- use_module(pltables_export_html).
:- use_module(pltables_export_latex).
:- use_module(pltables_export_xml).
:- use_module(pltables_export_csv).

% ----
% exported predicates
% ----
tables_to_html(Tables, File) :-
    open(File, write, Stream),
    html_header(Stream, TableName),
    tables_to_html2(Tables, Stream),
    html_footer(Stream, TableName),
    close(Stream).

tables_to_html_stream(Tables, Stream) :-
    html_header(Stream, TableName),
    tables_to_html2(Tables, Stream),
    html_footer(Stream, TableName).

tables_to_latex(Tables, File) :-
    open(File, write, Stream),
    latex_header(Stream, TableName),
    tables_to_latex2(Tables, Stream),
    latex_footer(Stream, TableName),
    close(Stream).

tables_to_latex_stream(Tables, Stream) :-
    latex_header(Stream, TableName),
    tables_to_latex2(Tables, Stream),
    latex_footer(Stream, TableName).

tables_to_xml(Tables, File) :-
    open(File, write, Stream),
    xml_header(Stream, TableName),
    tables_to_xml2(Tables, Stream),
    xml_footer(Stream, TableName),
    close(Stream).

tables_to_xml_stream(Tables, Stream) :-
    xml_header(Stream, TableName),
    tables_to_xml2(Tables, Stream),
    xml_footer(Stream, TableName).

tables_to_csv(Tables, File) :-
    open(File, write, Stream),
    csv_header(Stream, TableName),
    tables_to_csv2(Tables, Stream),
    csv_footer(Stream, TableName),
    close(Stream).

tables_to_csv_stream(Tables, Stream) :-
    csv_header(Stream, TableName),
    tables_to_csv2(Tables, Stream),
    csv_footer(Stream, TableName).

% ----
% split predicates
% ----
tables_to_html2([],_S).
tables_to_html2([Table|Tables],Stream) :-
    table_to_html(Table,Stream),
    tables_to_html2(Tables,Stream).

table_to_html(TableName, Stream) :-
    html_table_header(Stream,TableName),
    html_table_rows(Stream,TableName),
    html_table_footer(Stream,TableName).

tables_to_latex2([],_S).
tables_to_latex2([Table|Tables],Stream) :-
    table_to_latex(Table,Stream),
    tables_to_latex2(Tables,Stream).

table_to_latex(TableName, Stream) :-
    latex_table_header(Stream, TableName),
    latex_table_rows(Stream, TableName),
    latex_table_footer(Stream,TableName).

tables_to_xml2([],_S).
tables_to_xml2([Table|Tables],Stream) :-
    table_to_xml(Table,Stream),
    tables_to_xml2(Tables,Stream).

table_to_xml(TableName, Stream) :-
    xml_table_header(Stream, TableName),
    xml_table_rows(Stream, TableName),
    xml_table_footer(Stream,TableName).

tables_to_csv2([],_S).
tables_to_csv2([Table|Tables],Stream) :-
    table_to_csv(Table,Stream),
    tables_to_csv2(Tables,Stream).

table_to_csv(TableName, Stream) :-
    csv_table_header(Stream, TableName),
    csv_table_rows(Stream, TableName),
    csv_table_footer(Stream,TableName).
