% (c) 2011-2015 Lehrstuhl fuer Softwaretechnik und Programmiersprachen,
% Heinrich Heine Universitaet Duesseldorf
% This software is licenced under EPL 1.0 (http://www.eclipse.org/org/documents/epl-v10.html)

:- module(user_signal, [ user_interruptable_call_det/2
                       , protect_from_user_interrupt_det/1
                       , ignore_user_interrupt_det/1
                       , get_user_signal_ref/1]).

:- use_module('../../src/module_information.pl').

:- module_info(group,infrastructure).
:- module_info(description,'This is the interface to C code for interrupting Prolog runs with a UNIX signal.').

:- meta_predicate user_interruptable_call_det(0,-).
:- meta_predicate protect_from_user_interrupt_det(0).
:- meta_predicate ignore_user_interrupt_det(0).

:- load_files(library(system), [when(compile_time), imports([environ/2])]).
:- if(environ(no_interrupts,true)).
% turn off user interrupt handling: ctrl-c jumps into Prolog debugger

user_interruptable_call_det(Call,Result) :- call(Call), Result=ok.
ignore_user_interrupt_det(Call) :- call(Call).
protect_from_user_interrupt_det(Call) :- call(Call).
get_user_signal_ref(off).

:- else.
foreign_resource(user_signal, [set_user_signal_mode,get_user_signal_mode,user_signal_init,get_user_sig_reference]).
foreign(user_signal_init, user_signal_init).
foreign(set_user_signal_mode, set_user_signal_mode(+integer,+integer)).
foreign(get_user_signal_mode, get_user_signal_mode([-integer])).
foreign(get_user_sig_reference, get_user_sig_reference([-integer])).

:- dynamic loaded/0.

init_user_signal :- loadfr.

loadfr :-
    ( loaded -> true
    ; otherwise ->
        load_foreign_resource(library(user_signal)),
        user_signal_init,
        %write(user_signal_int),nl,
        assert(loaded)).

:- meta_predicate catch_interrupt_exception(0,-).
catch_interrupt_exception(Call,Result) :-
    catch( call(Call),
           E,
           ( E==user_interrupt_signal -> Result=interrupted
           ; otherwise -> throw(E))),
    ( Result == interrupted -> true
    ; otherwise             -> Result=ok).

%% user_interruptable_call_det(Call,Result) :- !,call(Call), Result=ok. % comment in to turn off CTRL-C
user_interruptable_call_det(Call,Result) :-
    loadfr,
    get_user_signal_mode(Mode),
    call_cleanup( (set_user_signal_mode(1,2),
                      catch_interrupt_exception(Call,Result),!),
                  set_user_signal_mode(1,Mode)).

%% protect_from_user_interrupt_det(Call) :- !. % comment in to turn off CTRL-C
protect_from_user_interrupt_det(Call) :-
    loadfr,
    get_user_signal_mode(Mode),
    call_cleanup( (set_user_signal_mode(0,1), call(Call), !),
                  set_user_signal_mode(1,Mode)).

ignore_user_interrupt_det(Call) :-
    loadfr,
    get_user_signal_mode(Mode),
    call_cleanup( (set_user_signal_mode(0,0), call(Call), !),
        set_user_signal_mode(0,Mode)).

get_user_signal_ref(Ref) :-
    loadfr,
    get_user_sig_reference(Ref).
:- endif.
