#include <assert.h>
#include <stdio.h>
#include <stdlib.h>

#include <zmq.h>
#include <czmq.h>

#include "../common/queue.h"
#include "../common/msg_headers.h"
#include "../worker/wcallbacks.h"
#include "../common/shm_common.h"
#include "../common/shm_srv.h"
#include "../common/hashtrie.h"
#include "test_helpers.h"

wQueue *g_queue = NULL;
int g_worker_id = 0;
hashtrie g_trie;

// just to make it link
zsock_t *g_zocket_push = NULL;

static void test_results_message(void) {
    zmsg_t *rmsg = new_results_message();
    msg_put_str(rmsg, "invariant_violated");
    msg_put_str(rmsg, "deadlock");

    msg_put_str(rmsg, "this_is_my_state_that_is_totally_broken");

    assert(zmsg_size(rmsg) == 4);

    char *header_s = zmsg_popstr(rmsg);
    int header;
    int rc = sscanf(header_s, "%d", &header);
    assert(rc == 1);
    assert(header == IS_RESULT_MSG);
    free(header_s);

    char *result1 = zmsg_popstr(rmsg);
    char *result2 = zmsg_popstr(rmsg);
    char *result3 = zmsg_popstr(rmsg);

    assert(0 == strcmp(result1, "invariant_violated"));
    assert(0 == strcmp(result2, "deadlock"));
    assert(0 == strcmp(result3, "this_is_my_state_that_is_totally_broken"));

    free(result3);
    free(result2);
    free(result1);

    zmsg_destroy(&rmsg);

    printf(".");
}


static void test_put_succ(void) {
    g_queue = get_queue(0);
    assert(q_size(g_queue) == 0);

    zmsg_t *msg = zmsg_new();
    char *term = "DAawesome_term";
    put_succ(msg, term, strlen(term)+1);

    assert(1 == zmsg_size(msg));
    assert(q_size(g_queue) == 1);

    tCell *cell = dequeue(g_queue);
    assert(0 == strcmp(cell->term, term));
    assert(cell->term_length == strlen(term)+1);

    
    char *hash = zmsg_popstr(msg);
    assert(0 == memcmp(hash, cell->digest, HASH_LEN));
    int i;
    for (i = 0; i < INFO_LEN; i++) {
        assert(0 == hash[HASH_LEN + i]);
    }

    free(hash);

    zmsg_destroy(&msg);
    teardown_cell(&cell);
    teardown_queue(&g_queue);

    printf(".");
}


static void test_add_stats(void) {
    g_queue = get_queue(2);
    assert(q_size(g_queue) == 2);
    g_worker_id = 5;

    zmsg_t *msg = zmsg_new();
    zmsg_addstr(msg, "very readable digest");

    start_timer();
    stop_timer();
    add_stats(msg);

    assert(zmsg_size(msg) == 7);

    char *header_s = zmsg_popstr(msg);
    int header;
    int rc = sscanf(header_s, "%d", &header);
    free(header_s);
    assert(rc == 1);
    assert(header == IS_HASH_MSG);

    char *id = zmsg_popstr(msg);
    assert(0 == strcmp(id, "5"));
    free(id);

    char *qs_s = zmsg_popstr(msg);
    size_t qs = *((size_t*) qs_s);
    assert(qs == 2);
    free(qs_s);

    // it might be any number and if it's way off we will see in the logs
    // also, it is information not needed to function correctly
    char *tProlog = zmsg_popstr(msg);
    long int t;
    rc = sscanf(tProlog, "%lu", &t);
    assert(rc == 1);
    free(tProlog);

    // see above
    char *timestamp = zmsg_popstr(msg);
    rc = sscanf(timestamp, "%lu", &t);
    assert(rc == 1);
    free(timestamp);

    char *statenmbr = zmsg_popstr(msg);
    assert(0 == strcmp(statenmbr, "0"));
    free(statenmbr);

    zmsg_destroy(&msg);

    g_worker_id = 0;
    teardown_queue(&g_queue);
    printf(".");
}


int main(int argc, char *argv[]) {
    struct semap_env senv = create_semaphores(0);
    g_trie = get_trie(senv, 10000);

    test_results_message();
    test_put_succ();
    test_add_stats();


    teardown_hashtrie(&g_trie, 1, 0);
    puts("");
    return 0;
}
