#include <assert.h>
#include <stdio.h>

#include <zmq.h>
#include <czmq.h>

#include "../common/msg_headers.h"
#include "../worker/wpush.h"

#include "test_helpers.h"


static void test_signal_transfer_done(zsock_t * const push, zsock_t * const pull) {
    char const * const worker_id = "666";
    wQueue *q = get_queue(3);
    signal_transfer_done(push, G_TRANSFER_DONE, atoi(worker_id), q);

    int header;
    char *id;
    size_t *qs;
    size_t size;
    int rc = zsock_recv(pull, "isb", &header, &id, (char**) &qs, &size);
    assert(rc == 0);

    assert(header == G_TRANSFER_DONE);
    assert(0 == strcmp(id, worker_id));
    assert(*qs == 3);
    assert(size == sizeof(size_t));

    free(qs);
    free(id);
    teardown_queue(&q);

    printf(".");
}

int main(int argc, char *argv[]) {
    zsock_t *pull = zsock_new_pull("inproc://foo-pull");
    zsock_t *push = zsock_new_push("inproc://foo-pull");
    assert(pull);

    test_signal_transfer_done(push, pull);

    zsock_destroy(&pull);
    zsock_destroy(&push);

    puts("");
    return 0;
}
