#ifndef WCTRL_H_INCLUDED
#define WCTRL_H_INCLUDED

struct wrecv_ctrl_arg {
    zsock_t *push;
    wQueue *q;
    int worker_id;
    int fd_logfile;
};

/* reader function; use this when setting up the reactor */
int wrecv_ctrl_reader(zloop_t *loop, zsock_t *reader, void *arg);


/* adds every other work-item from the queue to the message provided,
 * but no more than max;
 * the format will be
 *     <digest>
 *     <term>
 *     <digest>
 *     <term>
 *     ...
 */
void add_wps(zmsg_t * const msg, wQueue * const q, const size_t max, const int fd_logfile);


/* appends the entire queue to the message provided;
 * format is the same as described for add_wps.
 * if the queue is empty, only one frame is added 
 * (containing the string "retry") */
void add_entire_queue(zmsg_t * const msg, wQueue * const q, const int fd_logfile);


/* connects via a ZMQ_REQ zocket to endpoint and
 * sends up to max work items from the queue, if header is a TRANSFER_TO one
 * or sends all items from the queue, if header is P_TRANSFER_ALL */
void send_work(char const * const endpoint,
               wQueue * const q,
               const size_t max,
               const int header,
               const uint64_t timestamp,
               const int fd_logfile);



/* receives a control message on zocket_ctrl;
 * additional parameters are the zocket used for push-pull communication,
 * the worker's queue and its ID */
int wrecv_ctrl(zsock_t * const zocket_ctrl,
               zsock_t * const zocket_push,
               wQueue * const q,
               const int worker_id,
               const int fd_logfile);


zsock_t *init_wsub(const int proxynmbr);

#endif
