% (c) 2009-2019 Lehrstuhl fuer Softwaretechnik und Programmiersprachen,
% Heinrich Heine Universitaet Duesseldorf
% This software is licenced under EPL 1.0 (http://www.eclipse.org/org/documents/epl-v10.html)

% term_expansion/6 hook for the debugging_calls module
% this file should only be used by the debugging_calls module

:- use_module(module_information,[module_info/3]).

:- module_info(debugging_calls_te,group,infrastructure).
:- module_info(debugging_calls_te,description,'This sub-module contains the term-expansing code of the debugging call module, to remove ~~ calls').

:- dynamic current_context/2.
current_context(none,none).

assert_layout(File,Layout,NewPos) :-
	print('Layout:'),print(Layout),nl,
	retractall(current_context(_,_)),
	( Layout = [NewPos|_] -> true
	; otherwise -> NewPos = none),
	assert(current_context(File,NewPos)).

logte(_Term,Layout) :-
	prolog_load_context(file,File),
	current_context(Before,OldPos),
	assert_layout(File,Layout,NewPos),
	( File=Before ->
	    true
	; otherwise ->
	    print('### start: '),
	    print(File),print('/'),print(NewPos),
	    print(', was: '),
	    print(Before),print('/'),print(OldPos),nl).


expand((:- meta_predicate '~~'(_X)),_Layout1,_Term2,_Layout2) :- !,
  %print(not_removing_in_meta_predicate('~~'(_X))),nl,
  fail.
expand((:- use_module(library(Lib))),Layout1,Term2,Layout2) :-
  library_to_patch(Lib,PatchedLib),
  current_module(CurModule),
  \+ ignore_patches_in_module(CurModule),
  !,
  Layout2 = Layout1,
  format('Patching use_module(library(~w)) in ~w to ~w~n',[Lib,CurModule,PatchedLib]),
  Term2 = (:- use_module(PatchedLib)).
expand((:- use_module(library(Lib),Preds)),Layout1,Term2,Layout2) :-
  library_to_patch(Lib,PatchedLib),
  Preds \= [], % so that there is a way to bypass the term expander
  current_module(CurModule),
  \+ ignore_patches_in_module(CurModule),
  !,
  Layout2 = Layout1,
  format('Patching use_module(library(~w),~w) in ~w to ~w~n',[Lib,Preds,CurModule,PatchedLib]),
  Term2 = (:- use_module(PatchedLib,Preds)).
expand(Term1,Layout1,Term2,Layout2) :-
   % removes terms like ~~ pp_mnf,...
	debugging_calls:remove_debugging_calls(Layout1,Term1,Layout2,Term2).

% patch libraries for plspec
:- load_files(library(system), [when(compile_time), imports([environ/2])]).
:- if(environ(plspec_patch_libraries,true)).
library_to_patch(ordsets,library_plspec(ordsetsp)).
library_to_patch(avl,library_plspec(avlp)).
library_to_patch(lists,library_plspec(listsp)).
:- endif.
library_to_patch('~~'(X),library(X)).

ignore_patches_in_module(X) :- var(X),!.
ignore_patches_in_module(plspec).
ignore_patches_in_module(plspec_core).
ignore_patches_in_module(plspec_logger).
ignore_patches_in_module(prettyprinter).
ignore_patches_in_module(validator).

ignore_patches_in_module(debugging_calls).
ignore_patches_in_module(pathes).
ignore_patches_in_module(module_information).

ignore_patches_in_module('$restore').
ignore_patches_in_module('$save').
ignore_patches_in_module(alignments).
ignore_patches_in_module(avl).
ignore_patches_in_module(clpfd).
ignore_patches_in_module(chr).
ignore_patches_in_module(codesio).
ignore_patches_in_module(file_systems).
ignore_patches_in_module(ordsets).
ignore_patches_in_module(lists).
ignore_patches_in_module(prolog).
ignore_patches_in_module(structs).
ignore_patches_in_module(system).
ignore_patches_in_module(terms).
ignore_patches_in_module(types).
ignore_patches_in_module('SU_messages').

:- multifile user:term_expansion/6.
user:term_expansion(Term1, Layout1, Ids, Term2, Layout2, [rm_debug_calls|Ids]) :-
	nonvar(Term1), nonmember(rm_debug_calls,Ids),
	expand(Term1,Layout1,Term2,Layout2).



