% (c) 2009-2019 Lehrstuhl fuer Softwaretechnik und Programmiersprachen,
% Heinrich Heine Universitaet Duesseldorf
% This software is licenced under EPL 1.0 (http://www.eclipse.org/org/documents/epl-v10.html)

% file: gui_tcltk.pl
% loads the Tcl/Tk files and integrates them into the ProB Prolog source code

:- use_module(prob_startup, [startup_prob/0]).
%:- use_module(pathes,[set_search_pathes/0]).
:- use_module(tools,[read_string_from_file/2, get_options/4]).
:- use_module(value_persistance, [set_storage_directory/1]).

%%
%% required for Tcl/Tk code:
:- use_module(specfile).
:- use_module(preferences).
:- use_module(error_manager).
:- use_module(self_check).
:- use_module(translate).
:- use_module(state_space).
:- use_module(debug).
:- use_module(state_viewer_images).
:- use_module(user_interrupts).
:- use_module(extension('user_signal/user_signal')).
:- use_module(meta_interface).
:- use_module(model_checker).
:- use_module(bvisual).
:- use_module(bvisual2).
:- use_module(eval_strings).
:- use_module(b_global_sets).
:- use_module(coverage_statistics).
:- use_module(visualize_graph).
:- use_module(state_space_reduction).
:- use_module(reduce_graph_state_space).
:- use_module(graph_canon).
:- use_module(state_custom_dot_graph).
:- use_module(runtime_profiler).
:- use_module(b_machine_hierarchy).
:- use_module(prologTasks).
:- use_module(enabling_analysis).
:- use_module(b_read_write_info).
:- use_module(b_state_model_check).
:- use_module(b_trace_checking).
:- use_module(bmachine).
:- use_module(eclipse_interface).
:- use_module(probltlsrc(ltl)).
:- use_module(mcdc_coverage).
:- use_module(predicate_debugger).
:- use_module(predicate_evaluator).
:- use_module(refinement_checker).
:- use_module(source_profiler).
:- use_module(sap).

:- use_module(ast_inspector).
:- use_module(tcltk_tree_inspector).

:- use_module(value_persistance).
:- use_module(probcspsrc(haskell_csp)).
:- use_module(library(system)).
:- use_module(version).
:- use_module(tools_commands).
% Modules from por folder
:- use_module(probporsrc(dot_graphs_static_analysis)).
:- use_module(probporsrc(static_analysis)).
:- use_module(probporsrc(ample_sets)).
:- use_module(probpgesrc(pge_algo)).

:- use_module(symbolic_model_checker(cbc_ba)).
:- use_module(symbolic_model_checker(bmc)).
:- use_module(symbolic_model_checker(kinduction)).
:- use_module(symbolic_model_checker(ic3)).
:- use_module(symbolic_model_checker(ctigar)).
:- use_module(probsrc(unsat_cores)).
:- use_module(probsrc(latex_processor)).

   /* for tcltk: */
:- use_module(pref_definitions,[b_get_definition_string_list_from_spec/2, b_get_definition_string_from_spec/2]).

:- dynamic cur_tcl_process/1.

%%:- use_module(cspParserHs,'cia/haskell/java_api/cspParserHs.pl',
%%               [initCspParser/0]).

:- use_module(prob_startup, [startup_prob/0]).
:- use_module(tools,[host_platform/1, print_message/1]).
:- use_module(typechecker,[check_types/0]).
go :-
    startup_prob,
     (host_platform(Host) -> print_message(host_platform(Host))
                          ; print_message('### Could not get host platform !')),
    %%initCspParser,
    check_types,
    %init_preferences, % now done via event_listener
    get_cli_options(Options,RemArgs),
    %print(cli_options(Options,RemArgs)),nl,
    go2(Options,RemArgs).

 :- dynamic cli_prefs_file/1.

tcltk_load_commandline_prefs_file :- cli_prefs_file(File),
    preferences:load_preferences(File),
    print('Loaded preferences file: '),print(File),nl.

:- use_module(library(system),[sleep/1]).
go2(Opt,_RemArgs) :-
    member(prefs(File),Opt),
    assert(cli_prefs_file(File)),
    fail.
go2(Opt,_RemArgs) :-
    memberchk(cache_storage(Dir),Opt),
    set_storage_directory(Dir),
    fail.
go2(_,[A1,A2|T]) :-
    print('### Too many command-line arguments: '),nl,
    print('### '), print([A1,A2|T]),nl,
    print('###'),nl,
    print('### Usage [-pref PrefFile] [B/CSP_File]'),nl,
    fail.
go2(_,_) :-  % TclTk will call tcltk_get_command_line_file_to_open/1
    on_exception(E,tk_new([name('ProB')], X),
                 (%print(E),nl,
                  nl,
                  print('### Unable to start Tcl/Tk !'),nl,
                  print('### Ensure that you have installed the right version of Tcl/Tk'),nl,
                  print('### Consult the ProB website https://www3.hhu.de/stups/prob'),nl,
                  version:version_str(Str),
                  format('### ProB version: ~w~n',[Str]),
                  prolog_flag(version,PrologVersion),
                  format('### Prolog version: ~w~n',[PrologVersion]),
                  sleep(2), % sleep two seconds so that user can read message on Windows,...
                  throw(E))),
    assert(cur_tcl_process(X)),
    print('Starting Tcl/Tk'),nl, flush_output(user),
    eval_all_tcl_code(X),
    nl,
    (tcltk_get_command_line_option(batch_mode) -> true ; run_tk_main_loop),
    tcl_delete(X),
    retract(cur_tcl_process(_)),
    (\+ cli_error_occurred -> prob_finished
       ; nl,print('Abnormal Termination of ProB Tcl/Tk'),nl,
         print_cli_errors,
         halt(1)).

:- if(debug:global_debug_flag).
prob_finished :- print('Finished ProB Tcl/Tk'),nl, timer:timer_statistics.
:- else.
prob_finished :- print('Finished ProB Tcl/Tk'),nl.
:- endif.

eval_all_tcl_code(X) :-
    tcl_code(ACode),
    tcl_eval(X, ACode, _Result),
    fail.
eval_all_tcl_code(_).

run_tk_main_loop :- tk_main_loop,!.
run_tk_main_loop :-
    format(user_error,'Tcl/Tk GUI tk_main_loop has failed, restarting Tcl/Tk interpreter.~n',[]),
    % Bug in SICStus or Tcl/Tk ?!
    run_tk_main_loop.



gen_txt_files :- print_message('Generating ProBLicence.txt'),
                 tell('ProBLicence.txt'),prob_licence(Lic), write(Lic),fail.
gen_txt_files :- nl,told,
                 print_message('Generating ProBSummary.txt'),
                 tell('ProBSummary.txt'),prob_summary(Lic), write(Lic),fail.
gen_txt_files :- nl,told.

gen :- gen_txt_files.

%
% The .tcl code embedded in Prolog
% ' has to be encoded as \' and \ as \\
%
%

tcltk_get_command_line_file_to_open(FileNameToOpen) :-
    get_cli_options(_,AX),
    AX = [AF],
    (absolute_file_name(AF,FileNameToOpen)  % main purpose is to convert Windows \ into / and avoid encoding problems
      -> true
      ; error_manager:add_error(gui_tcltk,'Cannot determine full file name: ',AF),fail).

tcltk_get_command_line_option(Opt) :-  get_cli_options(O,_), member(Opt,O).

tcltk_get_command_line_tcltk_commands(list(Res)) :-
   findall(Cmd,tcltk_get_command_line_option(tcltk_command(Cmd)),Res).

get_cli_options(Options,RemArgs) :-
    prolog_flag(argv,ArgV),
    get_options(ArgV,recognised_option,Options,RemArgs).

recognised_option(X,Opt,Args,true) :- recognised_option(X,Opt,Args).

recognised_option('-prefs',prefs(F),[F]).
recognised_option('-regression',run_regression_tests,[]).
recognised_option('-bench',run_benchmarks,[]).
recognised_option('-laws',run_full_law_tests,[]).
recognised_option('-laws_quick',run_quick_law_tests,[]).
recognised_option('-csptests',run_cspm_tests,[]).
recognised_option('-selfcheck',run_selfcheck,[]).
recognised_option('-batch',batch_mode,[]).
recognised_option('-last',tcltk_open_last_file,[]).
recognised_option('-t',tcltk_command(procCheckTraceFile),[]).
recognised_option('-trace',tcltk_command(procCheckTraceFile),[]).
recognised_option('-c',tcltk_command(procComputeCoverage0),[]).
recognised_option('-mc',tcltk_command(procModelCheck),[]).
recognised_option('-tcl',tcltk_command(Cmd),[Cmd]).
recognised_option('-ref',tcltk_command(default_trace_refinement_search),[]).
recognised_option('-cov',trace_Module(F),[F]). % dealt with by prob_coverage_new_tcltk.pl
recognised_option('-cache',cache_storage(D),[D]).


:- dynamic cli_error_occurred/0, cli_error_occurred/1.
% should be asserted if the sicstus process should terminate with halt(1)
assert_cli_error_occurred(X) :-
   (cli_error_occurred -> true ; assert(cli_error_occurred)),
   assert(cli_error_occurred(X)).
print_cli_errors :- print('Error that occurred: '),nl,
   cli_error_occurred(X), print(' * '), print(X),nl,fail.
print_cli_errors :- nl.

%*******************************************************************************
% load text from external files

:- use_module(library(lists)).

add_tcl_code(Filename) :-
    absolute_file_name(Filename,AbsFilename),
    %format('Loading TCL file: ~w~n',[AbsFilename]),
    read_string_from_file(AbsFilename,String),
    split_string_by_keyword(String,Parts),
    assert_tcl_codes(Parts).
assert_tcl_codes([]).
assert_tcl_codes([String|Rest]) :-
    atom_codes(Atom,String),
    assert(tcl_code(Atom)),
    assert_tcl_codes(Rest).

% alternative version which stores File as Code-List, needs updating Tcl/Tk code to use procShowSourceCodeFromCodes
%assert_from_file(Predicate,Filename) :- !,
%    read_string_from_file(Filename,String),
%    Fact =.. [Predicate,list(String)], assert(Fact).
:- meta_predicate assert_from_file(1,-).
assert_from_file(Predicate,Filename) :-
    read_string_from_file(Filename,String),
    atom_codes(Atom,String),
    assert_fact(Predicate,Atom).

assert_fact(Module:Pred,Atom) :- !, Fact =.. [Pred,Atom], assert(Module:Fact).
assert_fact(Pred,Atom) :- Fact =.. [Pred,Atom], assert(Fact).

split_string_by_keyword(String,Parts) :-
    split_string_by_keyword(X,X,Parts,String,"").
split_string_by_keyword(Stackstart,"\n",[Stackstart|Parts]) -->
    "\nproc ",!,
    {append("proc ",Stackend,NextStackstart)},
    split_string_by_keyword(NextStackstart,Stackend,Parts).
split_string_by_keyword(Stackstart,[C|Stackend],Parts) -->
    [C],!,split_string_by_keyword(Stackstart,Stackend,Parts).
split_string_by_keyword(Stackstart,[],[Stackstart]) --> "".



:- dynamic tcl_code/1.
:- dynamic prob_licence/1, prob_summary/1, procsp_summary/1, mc_summary/1, proz_summary/1, protla_summary/1,
           symmetry_summary/1, ltl_summary/1, ctl_summary/1, alloy2b_summary/1.

:- load_files(library(system), [when(compile_time), imports([environ/2])]).

tcl_source_file(btracer_pure_procedures).
tcl_source_file(ctext).
tcl_source_file(evaluation_view).
% tcl_source_file(ast_inspector). % now replaced by more generic one:
tcl_source_file(tree_inspector).
:- if(\+ environ(prob_release,true)).
tcl_source_file(benchmarks).
:- endif.
tcl_source_file(main_prob_tcltk_gui).

full_tcl_name(FullName) :-
    tcl_source_file(Name),
    atom_concat(Name,'.tcl',File),
    absolute_file_name(tclsrc(File),FullName).

read_tcl_source_files :-
    findall(File, full_tcl_name(File), Files),
    maplist(add_tcl_code,Files).

:- read_tcl_source_files.

:- assert_from_file(prob_licence,'../tcl/licence.txt').
:- assert_from_file(prob_summary,'../tcl/prob_summary.txt').
:- assert_from_file(procsp_summary,'../tcl/procsp_summary.txt').
:- assert_from_file(mc_summary,'../tcl/mc_summary.txt').
:- assert_from_file(protla_summary,'../tcl/tla_summary.txt').
:- assert_from_file(proz_summary,'../tcl/proz_summary.txt').
:- assert_from_file(symmetry_summary,'../tcl/symmetry_summary.txt').
:- assert_from_file(ltl_summary,'../tcl/ltl_summary.txt').
:- assert_from_file(ctl_summary,'../tcl/ctl_summary.txt').
:- assert_from_file(alloy2b_summary,'../tcl/alloy_summary.txt').

:- dynamic enabling_legend/1.
:- assert_from_file(enabling_legend, '../tcl/enabling_legend.txt').

% for a faster code-test-(rewrite-retest)*- .... -giveup loop ;)
:- public reset_tcl/0. % to be typed in Prolog console
reset_tcl :-
    retractall(tcl_code(_)),
    read_tcl_source_files.

:- public print_tcl_summary/0.
% just for interactive debugging
print_tcl_summary :-
    tcl_code(Atom),
    atom_codes(Atom,Codes),
    length(Codes,N),
    ( N =< 50 -> NCodes = Codes
    ; N > 50 -> prefix_length(Codes,NCodes,50)),
    maplist(is_nl,NCodes,MCodes),
    format('Tcl: ~s~n',[MCodes]),
    fail.
print_tcl_summary.

is_nl(10,32) :- !.
is_nl(X,X).
