% (c) 2019-2019  Lehrstuhl fuer Softwaretechnik und Programmiersprachen,
% Heinrich Heine Universitaet Duesseldorf
% This software is licenced under EPL 1.0 (http://www.eclipse.org/org/documents/epl-v10.html)
:- module(ordsetsp,[
    is_ordset/1,        %  Set ->
    list_to_ord_set/2,  %  List -> Set
    ord_add_element/3,  %  Set x Elem -> Set
    ord_del_element/3,  %  Set x Elem -> Set
    ord_disjoint/2,     %  Set x Set ->
    ord_disjoint_union/3,   %  Set x Set -> Set
    ord_intersect/2,    %  Set x Set ->
    ord_intersection/3, %  Set x Set -> Set
    ord_intersection/4, %  Set x Set -> Set x Set
    ord_intersection/2, %  list(Set) -> Set
    ord_member/2,       %  Elem x Set ->
    ord_nonmember/2,    %  Elem x Set ->
    ord_seteq/2,        %  Set x Set ->
    ord_setproduct/3,   %  Set x Set -> SetOfPairs
    ord_subset/2,       %  Set x Set ->
    ord_subtract/3,     %  Set x Set -> Set
    ord_symdiff/3,      %  Set x Set -> Set
    ord_union/3,        %  Set x Set -> Set
    ord_union/2,        %  list(Set) -> Set
    ord_union/4,        %  Set x Set -> Set x Set
    ordset_order/3      %  Term x Term -> Relation
    ]).


:- use_module(probsrc(module_information)).
:- module_info(group,tools).
:- module_info(description,'Patched ordsets library with runtime checks.').


/* plspec setup */
/* TODO: fix path */
:- use_module(extension('plspec/plspec/plspec_core')).

/* TODO */
% toggle all runtime checks
 :- enable_all_spec_checks.
%:- plspec:enable_spec_check([ord_add_element/3, ord_intersection/3]).

/* TODO */
% register error handler; any predicate of arity one.
% error terms can be found as arguments in
% ../repos/plspec/plspec/prettyprinter.pl
% they should be fail(...)
%:- set_error_handler(ordsetsp:add_plspec_error/1). /* ? */

add_plspec_error(X) :- add_internal_error('PLSPEC: ',X).

:- defspec_pred(ordset, is_ordset).
/* TODO: maybe this works as well and may be useful */
%:- defspec(cool_ordset(X), and([ordset, list(X)])).


/* if you do not care about the third argument at all , you can also put any */
:- spec_pre(ord_add_element/3,  [ordset, any, one_of([var, ordset])]).
:- spec_pre(ord_del_element/3,  [ordset, any, one_of([var, ordset])]).

:- spec_pre(ord_intersection/3, [ordset, ordset, one_of([var, ordset])]).
:- spec_pre(ord_union/3, [ordset, ordset, one_of([var, ordset])]).
:- spec_pre(ord_disjoint_union/3, [ordset, ordset, one_of([var, ordset])]).
:- spec_pre(ord_intersection/3, [ordset, ordset, one_of([var, ordset])]).
:- spec_pre(ord_subtract/3, [ordset, ordset, one_of([var, ordset])]).
:- spec_pre(ord_symdiff/3, [ordset, ordset, one_of([var, ordset])]).
:- spec_pre(ord_setproduct/3, [ordset, ordset, any]).

:- spec_pre(ord_intersection/4, [ordset, ordset, one_of([var, ordset]), one_of([var, ordset])]).
:- spec_pre(ord_union/4, [ordset, ordset, one_of([var, ordset]), one_of([var, ordset])]).

:- spec_pre(ord_disjoint/2, [ordset, ordset]).
:- spec_pre(ord_intersect/2, [ordset, ordset]).
:- spec_pre(ord_subset/2, [ordset, ordset]).
:- spec_pre(ord_seteq/2, [ordset, ordset]).

:- spec_pre(ord_member/2, [nonvar, ordset]).
:- spec_pre(ord_nonmember/2, [nonvar, ordset]).

/* end of plspec setup */



:- use_module(library(~~(ordsets)),[is_ordset/1]). % the only predicate we copy as is

:- use_module(probsrc(error_manager)).
check_is_ord_set(_PP,X) :- ordsets:is_ordset(X),!.
check_is_ord_set(PP,X) :- add_internal_error(PP,'Argument is not ordered set:',X,unknown).

:- use_module(library(~~(lists)),[maplist/2]).
check_is_ord_set_list(_PP,X) :- maplist(ordsets:is_ordset,X),!.
check_is_ord_set_list(PP,X) :- add_internal_error(PP,'Argument is not list of ordered sets:',X,unknown).

check_is_ground(_PP,X) :- ground(X),!.
check_is_ground(PP,X) :- add_warning(PP,'Argument is not ground: ',X,unknown).

%   list_to_ord_set/2,  %  List -> Set
list_to_ord_set(List,Set) :- ordsets:list_to_ord_set(List,Set).

%ord_add_element/3, %  Set x Elem -> Set
ord_add_element(Set,Elem,Res) :- check_is_ground(ord_add_element,Elem),
   ordsets:ord_add_element(Set,Elem,Res).

%ord_del_element/3, %  Set x Elem -> Set
ord_del_element(Set,Elem,Res) :- check_is_ground(ord_del_element,Elem),
   ordsets:ord_del_element(Set,Elem,Res).

%ord_disjoint/2,        %  Set x Set ->
ord_disjoint(Set1,Set2) :- %check_is_ord_set(ord_disjoint1,Set1), check_is_ord_set(ord_disjoint2,Set2),
   ordsets:ord_disjoint(Set1,Set2).

%   ord_disjoint_union/3,   %  Set x Set -> Set
ord_disjoint_union(Set1,Set2,Res) :-
    %check_is_ord_set(ord_disjoint_union1,Set1), check_is_ord_set(ord_disjoint_union2,Set2),
    ordsets:ord_disjoint_union(Set1,Set2,Res).

%   ord_intersect/2,    %  Set x Set ->
ord_intersect(Set1,Set2) :- %check_is_ord_set(ord_intersect1,Set1), check_is_ord_set(ord_intersect2,Set2),
    ordsets:ord_intersect(Set1,Set2).

%   ord_intersection/3, %  Set x Set -> Set
ord_intersection(Set1,Set2,Res) :- %check_is_ord_set(ord_intersect1,Set1), check_is_ord_set(ord_intersect2,Set2),
     ordsets:ord_intersection(Set1,Set2,Res).

%   ord_intersection/4, %  Set x Set -> Set x Set
ord_intersection(Set1,Set2,Res1,Res2) :- %check_is_ord_set(ord_intersect1,Set1), check_is_ord_set(ord_intersect2,Set2),
     ordsets:ord_intersection(Set1,Set2,Res1,Res2).

%   ord_intersection/2, %  list(Set) -> Set
ord_intersection(SetList,Res) :- %check_is_ord_set_list(ord_intersection,SetList),
   ordsets:ord_intersection(SetList,Res).

%   ord_member/2,       %  Elem x Set ->
ord_member(Elem,Set) :- check_is_ground(ord_member,Elem),
  ordsets:ord_member(Elem,Set).

%   ord_nonmember/2,    %  Elem x Set ->
ord_nonmember(Elem,Set) :- check_is_ground(ord_nonmember,Elem),
   ordsets:ord_nonmember(Elem,Set).

%   ord_seteq/2,        %  Set x Set ->
ord_seteq(Set1,Set2) :- ordsets:ord_seteq(Set1,Set2).

%   ord_setproduct/3,   %  Set x Set -> SetOfPairs
ord_setproduct(Set1,Set2,Res) :- ordsets:ord_setproduct(Set1,Set2,Res).

%   ord_subset/2,       %  Set x Set ->
ord_subset(Set1,Set2) :- %check_is_ord_set(ord_subset1,Set1), check_is_ord_set(ord_subset2,Set2),
    ordsets:ord_subset(Set1,Set2).

%   ord_subtract/3,     %  Set x Set -> Set
ord_subtract(Set1,Set2,Res) :- %check_is_ord_set(ord_subtract1,Set1), check_is_ord_set(ord_subtract2,Set2),
     ordsets:ord_subtract(Set1,Set2,Res).

%   ord_symdiff/3,      %  Set x Set -> Set
ord_symdiff(Set1,Set2,Res) :- %check_is_ord_set(ord_symdiff1,Set1), check_is_ord_set(ord_symdiff2,Set2),
    ordsets:ord_symdiff(Set1,Set2,Res).

%   ord_union/3,        %  Set x Set -> Set
ord_union(Set1,Set2,Res) :- %check_is_ord_set(ord_union1,Set1), check_is_ord_set(ord_union2,Set2),
     ordsets:ord_union(Set1,Set2,Res).

%   ord_union/2,        %  list(Set) -> Set
ord_union(SetList,Res) :- %check_is_ord_set_list(ord_union,SetList),
   ordsets:ord_union(SetList,Res).

%   ord_union/4,        %  Set x Set -> Set x Set
ord_union(Set1,Set2,Res1,Res2) :-  %check_is_ord_set(ord_union1,Set1), check_is_ord_set(ord_union2,Set2),
   ordsets:ord_union(Set1,Set2,Res1,Res2).

%   ordset_order/3      %  Term x Term -> Relation
ordset_order(Set1,Set2,Res) :- ordsets:ordset_order(Set1,Set2,Res).
