% (c) 2009-2020 Lehrstuhl fuer Softwaretechnik und Programmiersprachen,
% Heinrich Heine Universitaet Duesseldorf
% This software is licenced under EPL 1.0 (http://www.eclipse.org/org/documents/epl-v10.html)

%:- prolog_flag(compiling,_,debugcode).
%:- prolog_flag(source_info,_,on).


% patch for SICStus 4.3.3 performance issue
% sprm_14972_patch.pl BEGIN
:- if((current_prolog_flag(dialect, sicstus),
      current_prolog_flag(version_data, sicstus(4,3,3,_,_)))).
prolog:wf_call_like_arg(A, B, C, D, E, F, G, H, I, _) :-
  prolog:wellformed_body_iso(A, B, C, D, E, F, G, H, I, quiet),
  !.
prolog:wf_call_like_arg(A, B, C, A, D, _, E, _, _, _) :-
  F=E:A,
  prolog:condense_layout(B, G),
  prolog:comp_layout2(B, G, B, H),
  C=call(F),
  prolog:comp_layout1(B, H, D).
:- endif.
% sprm_14972_patch.pl END

% -------------

%% sp_4_1_3_nested_exception_patch.pl BEGIN
%% Compile this to patch a bug in 4.1.3 affecting nested exception handlers.
%% This source code is a no-op on other versions of sicstus than 4.1.3.
%% The underlying problem was introduced in 4.1.3 and is fixed in the
%% next release of SICStus Prolog.

:- if(current_prolog_flag(dialect,sicstus)).
:- if(current_prolog_flag(version_data,sicstus(4,1,3,0,_))).

prolog:(matching_exception(internal, Ball, Templ) :- !,
        Templ = Ball).
prolog:(matching_exception(prolog, '$reserved'(_), _) :- !,
        fail).
prolog:(matching_exception(prolog, time_out, _) :- !,
        fail).
prolog:(matching_exception(_, Ball, Template) :-
       compound(Ball),
        sysball_iso_equiv(Ball,Iso1Exc), !,
        Template = error(Iso1Exc, Ball)).
prolog:(matching_exception(_, Ball, Ball)).

:- endif.                       % sicstus 4.1.3
:- endif.                       % sicstus
%% sp_4_1_3_nested_exception_patch.pl END


% -------------
portray_message(informational, _).


%:- prolog_flag(compiling,_,profiledcode), print(profiling),nl,nl. %%
% :- use_module(library(gauge)),view([ haskell_csp:_, translate:_, user:_]).
% :- use_module(library(gauge)),view([ model_checker:_, hashing:_, specfile:_, user:_, state_space:_, b_state_model_check:_, error_manager:_]).
% :- use_module(library(gauge)),view([ b_interpreter:_]).
% :- use_module(library(gauge)),view([ graph_iso_nauty:_, graphiso:_]).

% got :- go, print_timer_results. %%


:- use_module(library(tcltk)).
:- use_module(library(system)).


:- use_module(debugging_calls).
:- register_debugging_call pp_mnf(*), pp_cll(*), mnf(*), mnf(-,*), det_call(*).
:- disable_debugging_calls. %%

:- use_module(library(random),[random/3]).
do_one_gui_event :-
  (random(1,5,1) -> force_do_one_gui_event ; true).  % do_one_gui_event is quite expensive

force_do_one_gui_event :-
  (tk_do_one_event([tk_dont_wait,
                       tk_window_events,tk_idle_events ])
    -> true ; true
  ).

tk_do_all_events :-
        tk_do_one_event, !,
        tk_do_all_events.
tk_do_all_events.


:- use_module(pathes). % first to set_compile_time_search_pathes

:- use_module(preferences,[set_prob_application_type/1]).
:- set_prob_application_type(tcltk).

%:- print(compile(tcltk_interface)),nl.
:- use_module(tcltk_interface).
:- use_module(fdr_csp_generator).
%:- print(compile(gui_tcltk)),nl.

:- compile(gui_tcltk).

:- use_module(basic_unit_tests). % basic unit tests
:- use_module(compile_time_flags).

%:- use_module(b_normal_form).
:- use_module(bvisual2,[bv_include_variables/0]).
:- bv_include_variables.

:- use_module(eventhandling,[announce_event/1]).
:- announce_event(compile_prob).

%:- go.
save :- save_program('ProB.sav').

runtime_entry(start) :- go.


/* :- self_check:perform_self_check. */
:- nl,print('Loaded ProB'),nl,
      print('Type go.<return> to (re-)start ProB'),nl,nl.
