% (c) 2009 Lehrstuhl fuer Softwaretechnik und Programmiersprachen, 
% Heinrich Heine Universitaet Duesseldorf
% This software is licenced under EPL 1.0 (http://www.eclipse.org/org/documents/epl-v10.html) 

:- module(zparameters,[insert_parameter/4]).

:- use_module(library(lists)).

:- use_module(probsrc(tools)).

:- use_module(subexpressions).

:- use_module(probsrc(module_information)).
:- module_info(group,proz).
:- module_info(description,'This module provides functionality to replace a Z schema\'s parameter by expressions').

%********************************************************************************
% insert_parameter
insert_parameter(Exprs,[],[],Exprs) :- !.
insert_parameter(Exprs,Params,Values,Result) :- insert_parameter2(Exprs,Params,Values,Result).

insert_parameter2([],_,_,[]) :- !.
insert_parameter2([E|ERest],Params,Values,[P|PRest]) :-
    !,insert_parameter2(E,Params,Values,P),
    insert_parameter2(ERest,Params,Values,PRest).
insert_parameter2(ref(Name,P),Params,Values,ref(Value,P)) :-
    !,
    ( nth0(N,Params,Name) ->
        nth0(N,Values,Value)
    ; otherwise ->
        Value = Name).
insert_parameter2(Expr,Params,Values,Result) :-
    znamespace(Expr,Outer,Inner,Names,NOuter,NInner,Result),
    !,
    insert_parameter2(Outer,Params,Values,NOuter),
    remove_params(Params,Values,Names,NParams,NValues),
    insert_parameter(Inner,NParams,NValues,NInner).
insert_parameter2(Expr,Params,Values,Result) :-
    zexpr_conversion(Expr,Subs,PSubs,Result),
    insert_parameter2(Subs,Params,Values,PSubs).

remove_params([],[],_,[],[]).
remove_params([P|PRest],[V|VRest],Names,Params,Values) :-
    (member(P,Names) ->
        (Params = OParams, Values = OValues);
        (Params = [P|OParams], Values = [V|OValues])),
    remove_params(PRest,VRest,Names,OParams,OValues).
