% (c) 2009-2019 Lehrstuhl fuer Softwaretechnik und Programmiersprachen,
% Heinrich Heine Universitaet Duesseldorf
% This software is licenced under EPL 1.0 (http://www.eclipse.org/org/documents/epl-v10.html)


:- module(tools_files,
                 [write_lines_to_file/2,
                  write_to_file/2,
                  write_to_utf8_file/2, write_to_utf8_file_or_user_output/2,
                  put_codes/2]).


:- use_module(module_information).

:- module_info(group,infrastructure).
:- module_info(description,'A few utilities on files seperated out from tools.pl to avoid cyclic module dependencies.').

write_lines_to_file(FILE,Lines) :-
   lines_to_code_list(Lines,CodeList),
   write_to_file(FILE,CodeList).

lines_to_code_list([],[]).
lines_to_code_list([Line|Lines],CodeList) :-
   (number(Line) -> number_codes(Line,L); atom_codes(Line,L)),
   append(L,[10],L1),
   append(L1,RL,CodeList),
   lines_to_code_list(Lines,RL).

write_to_utf8_file(FILE,CodesList) :-
   open(FILE,write,S,[encoding('UTF-8')]),
   put_codes(CodesList,S),
   close(S).

write_to_utf8_file_or_user_output(user_output,CodesList) :- !,
   put_codes(CodesList,user_output).
write_to_utf8_file_or_user_output(FILE,CodesList) :- write_to_utf8_file(FILE,CodesList).

write_to_file(FILE,CodesList) :-
   open(FILE,write,S),
   put_codes(CodesList,S),
   close(S).

put_codes([],_).
put_codes([H|T],S) :- put_code(S,H), put_codes(T,S).
