% (c) 2009-2020 Lehrstuhl fuer Softwaretechnik und Programmiersprachen,
% Heinrich Heine Universitaet Duesseldorf
% This software is licenced under EPL 1.0 (http://www.eclipse.org/org/documents/epl-v10.html)


:- module(tools_lists,
                 [count_occurences/2,
                  ord_member_nonvar_chk/2,
                  ord_select_nonvar_chk/3,
                  delete_first/3,
                  length_greater/2,
                  member_nonvar_list/2
                  ]).

:- use_module(module_information).

:- module_info(group,infrastructure).
:- module_info(description,'A few utilities on lists seperated out from tools.pl to avoid cyclic module dependencies.').


% self-check in tools to avoid dependency on self_check module
%:- assert_must_succeed((tools:count_occurences([a,b,a,a,b],R),R == [a-3,b-2])).
:- use_module(library(avl), [avl_fetch/3, avl_to_list/2, avl_store/4,empty_avl/1]).

% count number of occurences inside a list
count_occurences(L,R) :- empty_avl(E), count_occ_aux(L,E,R).
count_occ_aux([Term|T],A,Res) :- (avl_fetch(Term,A,Count) -> C is Count+1 ; C = 1),
   avl_store(Term,A,C,A2),
   count_occ_aux(T,A2,Res).
count_occ_aux([],A,L) :- avl_to_list(A,L).

% like ord_member but also allows nonvar lookup terms
% note that this test fails for ord_member:
%:- assert_must_succeed((tools:ord_member_nonvar_chk(p(b,X),[p(a,c),p(b,d)]),R == d)).
ord_member_nonvar_chk(X, [H|T]) :-
        (X=H -> true
         ;  X @>H -> ord_member_nonvar_chk(X,T)).

:- use_module(library(lists), [select/3]).

% in contrast to delete/3 we delete only first occurence
delete_first(List,X,NewList) :- select(X,List,Del),!,NewList=Del.
delete_first(L,_,L).



%:- assert_must_succeed((tools_lists:ord_select_nonvar_chk(b,[a,b,c,d],R),R == [a,c,d])).
ord_select_nonvar_chk(X, [H|T], Res) :-
        (X=H -> Res=T
         ;  X @>H -> Res=[H|TR], ord_select_nonvar_chk(X,T,TR)).

% length_greater([1,2,3],1)
length_greater(_,X) :- X<0,!.
length_greater([_|T],X) :- X1 is X-1, length_greater(T,X1).


% a version of member that stops at a variable list tail
member_nonvar_list(X,List) :- nonvar(List), List=[H|T],
   (X=H ; member_nonvar_list(X,T)).
