:- use_module([library(terms),zlib]).

:- multifile user:term_expansion/2.

:- op(1200,xfx,::-).

:- dynamic unit_test_id/1.
user:term_expansion((unit_test(Id) ::- Body),_Lay1,Tokens1,(unit_test(Id) :- Body),_Lay2,[unit_test|Tokens1]) :-
    nonmember(unit_test, Tokens1),
    print(unit_test_id(Id)),nl,
    assert(unit_test_id(Id)).

test :-
    findall(X,unit_test_id(X),Tests),
    run_tests(Tests).

run_tests([]).
run_tests([Test|Tail]) :-
    (  unit_test(Test)
    -> format("Test '~s': success~n", [Test])
    ;  format("Test '~s': failed!~n", [Test])
    ),
    run_tests(Tail),
    halt.

unit_test(byte_array) ::-
    atom_codes(abbaabba,U),
    length(U,ULen),
    zlib_compress(U,C),
    zlib_uncompress(C,ULen,U).

:- use_module(library(terms),[subsumeschk/2]).

unit_test(simple_term) ::-
    T = a(abcdef),
    zlib_compress_term(T,C,TLen),
    zlib_uncompress_term(C,TLen,T2),
    subsumeschk(T,T2),
    subsumeschk(T2,T).

unit_test(complex_term) ::-
    T = a(X),
    when(nonvar(X),write(X)),
    zlib_compress_term(T,C,TLen),
    zlib_uncompress_term(C,TLen,T2),
    subsumeschk(T,T2),
    subsumeschk(T2,T).

unit_test(prob_library4) ::-
    T = const_and_vars(0,[bind(loan,[]),bind(book,[]),bind(member,avl_set(node(fd(1,'MEMBERID'),true,1,empty,node(fd(2,'MEMBERID'),true,0,empty,empty)))),bind(reservation,[])]),
    print(T),
    zlib_compress_term(T,C,TLen),
    print(len(TLen)),nl,
    print(C), length(C,LC), print(len(LC)),nl,
    zlib_uncompress_term(C,TLen,T2),
    T==T2.


get_memory_used([M,PU]) :- garbage_collect_atoms,statistics(program,[PU,_]),statistics(memory_used,M).
print_memory_used([M,PU]) :- print_mb(M), print(' ('), print_mb(PU), print(' program) ').
print_memory_used_difference([M1,_PU1],[NewM2,_PU2]) :- Diff is (M1)-(NewM2),
   (Diff >= 0 -> print(' freed: '), print_bytes(Diff)
    ; print(' allocated: '), D2 is -(Diff), print_bytes(D2)),nl.
   %print(' / '),Diff2 is (PU1)-(PU2), print_mb(Diff2).
print_bytes(X) :- (X<50000 -> print_kb(X) ; print_mb(X)).
print_mb(X) :- XMB is X / 1048576,
  format(' ~3f MB',[XMB]).
print_kb(X) :- XKB is X / 1024,
  format(' ~3f KB',[XKB]).

 :- dynamic p/1, p/2.
unit_test(memory) ::-
    T = const_and_vars(0,[bind(loan,[]),bind(book,[]),
     bind(member,avl_set(node(fd(1,'MEMBERID'),true,1,empty,node(fd(2,'MEMBERID'),true,0,empty,empty)))),
     bind(reservation,[])]),
    zlib_compress_term(T,C,TLen),
    get_memory_used(M1),
    print(asserting_uncompressed_term),nl,
    assertn(100000,p(T)),
    get_memory_used(M2),
    print_memory_used_difference(M1,M2),
    print(asserting_compressed_term),nl,
    assertn(100000,p(TLen,C)),
    get_memory_used(M3),
    print_memory_used_difference(M2,M3).

assertn(0,_) :- !.
assertn(N,X) :- N>0, assert(X), N1 is N-1, assertn(N1,X).
