
:- load_files(library(system), [when(compile_time), imports([environ/2])]).
:- if(environ(partially_evaluate, true)).
:- use_module(partial_evaluator(pe)).
:- unfold(normalise_value(V1,V2),(nonvar(V1);nonvar(V2))).
:- unfold(l_expand_and_normalise_values(H,_),nonvar(H)).
:- unfold(l_expand_and_normalise_values(H,_,_),nonvar(H)).
:- unfold(store_value(_Var,_Value,_Old,_New)).
:- unfold(store_value2(Old,Var,_Value,_New),(nonvar(Old),nonvar(Var))). % TO DO: also check whether we can perform the comparison  with bind(V,_) in head of Old !
:- unfold(lookup_and_delete_value(_Var,_Value,_T,_UT,_ReportError,_WF,_Finished)).
:- unfold(lookup_and_delete_value1(T,Var,_Value,_UT,_ReportError,_WF,_Finished),(nonvar(T),nonvar(Var))).
:- unfold(lookup_and_delete_value2(Var2,_Value2,_T,_Var,_Value,_Res,_ReportError,_WF,_Finished), nonvar(Var2)).
:- unfold(delete_variables_from_state(A,_,_),nonvar(A)).
:- unfold(delete_variable(InState, _, _),nonvar(InState)).
:- unfold(store_intermediate_updates(_InStore,_Update,_OutStore,_)).
:- unfold(store_intermediate_updates2(A,B,_C,_),(nonvar(A),nonvar(B))).
:- unfold(store_intermediate_updates3(Updates,_),nonvar(Updates)).
:- unfold(search_and_delete_value(Var,_OldVal,_NewVal,_Updates,_NUpdates),nonvar(Var)).
:- unfold(search_and_delete_value2(InStore,_Var,_OldVal,_Value,_OutStore),nonvar(InStore)).
:- unfold(search_and_delete_value3(Var2,_Value2,_T,_Var,_OldVal,_Value,_Res),nonvar(Var2)).
:- execute(normalise_value_for_var(Var,EXPAND,Val,_Res),(ground(Var),ground(EXPAND),simple_value_for_normalise(Val))).
simple_value_for_normalise(V) :- var(V),!,fail.
simple_value_for_normalise(fd(_,_)).
simple_value_for_normalise(int(_)).
simple_value_for_normalise([]).
simple_value_for_normalise(string(_)).
simple_value_for_normalise(pred_true). simple_value_for_normalise(pred_false).
simple_value_for_normalise((A,B)) :- simple_value_for_normalise(A), simsimple_value_for_normaliseple_value(B).

:- forward_block(lookup_and_delete_value1(?,?,?,?,?,?,?)) .
:- forward_block(lookup_and_delete_value2(?,?,?,?,?,?,?,?,?)) .
:- forward_block(search_and_delete_value(?,?,?,?,?)) .
:- forward_block(search_and_delete_value2(?,?,?,?,?)) .
:- forward_block(search_and_delete_value3(?,?,?,?,?,?,?)) .
:- forward_block(store_intermediate_updates2(?,?,?,?)) .
:- forward_block(store_intermediate_updates3(?,?)) .
:- endif.





% a version of storing updates for ground updates and previous states
% tries to keep sharing, important in particular for -execute
store_ground_updates_and_normalise(Updates,InState,OutState) :-
   preferences:get_computed_preference(convert_closures_into_explicit_form_for_store,EXPAND),
   sort(Updates,SUpdates), % to be able to use ord_select_nonvar_chk
   store_ground_updates_and_normalise1(InState,SUpdates,EXPAND,OutState).

:- use_module(tools_lists,[ord_select_nonvar_chk/3]).
store_ground_updates_and_normalise1([],SUpdates,_,Res) :- !,
   (SUpdates=[] -> true ; add_internal_error('Unknown variables in update:',SUpdates)),
   Res=SUpdates.
store_ground_updates_and_normalise1([bind(Var,_)|T],SUpdates,EXPAND,Res) :-
   ord_select_nonvar_chk(bind(Var,NewVal),SUpdates,RestUpdates),
   !,
   normalise_value_for_var(Var,EXPAND,NewVal,NormNewVal),
   Res = [bind(Var,NormNewVal)|TR],
   (RestUpdates=[] -> TR=T
    ; store_ground_updates_and_normalise1(T,RestUpdates,EXPAND,TR)).
store_ground_updates_and_normalise1([BIND|T],SUpdates,EXPAND,[BIND|TR]) :-
   store_ground_updates_and_normalise1(T,SUpdates,EXPAND,TR).




% a blocking version of store_updates_and_normalise; to be used by CBC

lazy_store_updates_and_normalise(Updates,InState,OutState) :-
   preferences:get_computed_preference(convert_closures_into_explicit_form_for_store,EXPAND),
   lazy_store_updates_and_normalise1(Updates,EXPAND,[],InState,OutState).

:- block lazy_store_updates_and_normalise1(-,?,?,?,?).
lazy_store_updates_and_normalise1([],_,_,InState,InState).
lazy_store_updates_and_normalise1([bind(Var,NewVal)|RestUpdates],EXPAND,UpdSoFar,InState,OutState) :-
   lazy_store_updates_and_normalise2(Var,EXPAND,NewVal,RestUpdates,UpdSoFar,InState,OutState).
:- block lazy_store_updates_and_normalise2(-,?,?,?,?,?,?).
lazy_store_updates_and_normalise2(Var,EXPAND,NewVal,RestUpdates,UpdSoFar,InState,OutState) :-
   normalise_value_for_var(Var,EXPAND,NewVal,NormalisedNewVal),
   %print(store(Var,NewVal,NormalisedNewVal)),nl,
   store_value_for_existing_id(InState,Var,NormalisedNewVal,InState2),
   (member(bind(Var,OtherValue),UpdSoFar) % safe_member(bind(Var,OtherValue),UpdSoFar,store_updates_and_normalise)
      ->
      add_error(store_updates_and_normalise,'Variable has been assigned twice in parallel: ',Var),
      add_error(store_updates_and_normalise,'Values: ',(NewVal,OtherValue))
    ;  (b_is_constant(Var)
        -> add_error(store_updates_and_normalise,'Assigned value to constant: ',Var) ; true)
   ),
   lazy_store_updates_and_normalise1(RestUpdates,EXPAND,[bind(Var,NewVal)|UpdSoFar],InState2,OutState).
