
%% Run this file from the root of the prob_prolog directory like 'sics -l extensions/prolog_fuzzer/fuzzer_runner.pl'.
% Jenkins: https://alfred.cs.uni-duesseldorf.de/view/ProB%20Tests/job/ProB_Fuzz_Tests/

:- ensure_loaded('../../src/prob_cli').

:- use_module(fuzzing).

:- use_module(probsrc('kodkod/kodkod_annotator')).
:- use_module(probsrc(b_interpreter)).
:- use_module(probsrc(kernel_equality)).
%:- use_module(probsrc(kernel_objects)).
:- use_module(probsrc(solver_interface)).
:- use_module(smt_solvers_interface(smt_solvers_interface), [smt_solve_predicate/4]).

:- use_module(library(random),[random/3,setrand/1]).

run_tests :- run_fuzz_tests, do_halt.

run_fuzz_tests :-
    init_pref ,
    % generate fresh state at each start by using system time
    now(R),
    setrand(R),
    run_all_tests.

run_all_tests :-
     test2 ,  test3 ,  test4 ,  test5 ,  test6 ,  test7 ,  test8 ,  test9 ,
    test10 , test11 , test12 , test13 , test14 , test15 , test16 , test17 ,
    test18 , test19 , test20 , test21 , test22 , test23 , test24 , test25 ,
    %test26 , test27 ,
    test29 , test30,
    test_automated_kodkod_annotation, test_automated_kodkod_annotation_solving.

do_halt :-
    (on_exception(_,error_detected(true),fail) ->
        halt(1)
    ;   halt(0)).

init_pref :-
    pathes:set_search_pathes,
    error_manager:init_error_manager ,
    init_eclipse_preferences,
    external_functions:reset_argv,
    set_prefs.

%%%%%%%%%%% b_interpreter tests

% every generated predicate has to evaluate to true or false
% or there is a well-definedness error
test1_aux(Predicate) :-
    error_manager:reset_errors,
    % generate bindlist
    generate(prob_state_bindlist(Predicate),BindList:_) ,
    (b_test_boolean_expression_wf(Predicate,BindList,[]) ;
     b_not_test_boolean_expression_wf(Predicate,BindList,[]) ;
     get_error(well_definedness_error,_)).
test1 :-
    init_pref,
    fuzz(test1_aux/1,10000,prob_ast_pred([size:3])).

% every generated expression can be computed
% unless there is a well-definedness error
test2_aux(Expression) :-
    error_manager:reset_errors,
    % generate bindlist
    generate(prob_state_bindlist(Expression),BindList:_) ,
    (b_compute_expression_nowf(Expression,BindList,[],_R) ;
     get_error(well_definedness_error,_)).
% enable generation of identifier by option "id"
test2 :-
    init_pref,
    fuzz(test2_aux/1,20000,prob_ast_expr([id])).

% mutation of integer expressions
test3 :-
    init_pref,
    fuzz(test2_aux/1,10000,
        mutation([
            b(add(b(unary_minus(b(integer(81),integer,[])),integer,[]),b(integer(-8),integer,[])),integer,[]),
            b(max(b(value([int(-57),int(-18),int(-88),int(119),int(98)]),set(integer),[])),integer,[]),
            b(integer(144),integer,[]),
            b(add(b(integer(25),integer,[]),b(integer(95),integer,[])),integer,[]),
            b(modulo(b(integer(7),integer,[]),b(integer(121),integer,[])),integer,[]),
            b(integer(1024),integer,[])
            ]:prob_ast_int_expr)).
% mutation of predicates
test4 :-
    init_pref,
    fuzz(test1_aux/1,10000,
        mutation([
            b(negation(b(member(b(boolean_true,boolean,[]),b(set_extension([b(boolean_false,boolean,[]),b(boolean_false,boolean,[]),b(boolean_true,boolean,[]),b(boolean_false,boolean,[]),b(boolean_true,boolean,[]),b(boolean_false,boolean,[])]),set(boolean),[])),pred,[])),pred,[]),
            b(not_subset(b(set_extension([b(string('jjk'),string,[])]),set(string),[]),b(value([string('U)=$HN!u'),string('KyR'),string('S5ZsD4?KD'),string('U'),string(';}XG:Z=EnU\\.Bt'),string(':U3nc>RhxwiQNM6p_')]),set(string),[])),pred,[]),
            b(less_equal(b(unary_minus(b(max(b(value(avl_set(node(int(-29),true,0,node(int(-37),true,0,node(int(-61),true,0,empty,empty),node(int(-31),true,0,empty,empty)),node(int(23),true,-1,node(int(20),true,0,empty,empty),empty)))),set(integer),[])),integer,[])),integer,[]),b(multiplication(b(minus(b(b(power_of(b(integer(2),integer,[]),b(integer(1),integer,[])),integer,[])div b(integer(24),integer,[]),integer,[]),b(integer(93),integer,[])),integer,[]),b(b(max(b(set_extension([b(integer(6),integer,[]),b(integer(-20),integer,[]),b(integer(3),integer,[])]),set(integer),[])),integer,[])div b(integer(2),integer,[]),integer,[])),integer,[])),pred,[]),
            b(less(b(integer(7),integer,[]),b(add(b(integer(6),integer,[]),b(integer(45),integer,[])),integer,[])),pred,[]) ,
            b(equal(b(integer(7),integer,[]),b(integer(456),integer,[])),pred,[]),
            b(less(b(minus(b(add(b(modulo(b(integer(99),integer,[]),b(integer(2211),integer,[])),integer,[]),b(b(power_of(b(integer(23),integer,[]),b(integer(3),integer,[])),integer,[])div b(integer(109),integer,[]),integer,[])),integer,[]),b(b(minus(b(b(integer(-35),integer,[])div b(integer(68),integer,[]),integer,[]),b(b(add(b(add(b(integer(59),integer,[]),b(min(b(set_extension([b(integer(-30),integer,[]),b(integer(79),integer,[]),b(integer(-81),integer,[]),b(integer(1),integer,[]),b(integer(-111),integer,[])]),set(integer),[])),integer,[])),integer,[]),b(max(b(value([int(103),int(0),int(-113),int(-82),int(-122),int(-25),int(2)]),set(integer),[])),integer,[])),integer,[])div b(integer(73),integer,[]),integer,[])),integer,[])div b(integer(87),integer,[]),integer,[])),integer,[]),b(add(b(integer(25),integer,[]),b(integer(95),integer,[])),integer,[])),pred,[])
            ]:prob_ast_pred)).
% mutation of set expressions
test5 :-
    init_pref,
    fuzz(test2_aux/1,10000,
        mutation([
            b(interval(b(integer(7),integer,[]),b(integer(19),integer,[])),set(integer),[]),
            b(intersection(b(intersection(b(value(avl_set(node(pred_true,true,-1,node(pred_false,true,0,empty,empty),empty))),set(boolean),[]),b(value(avl_set(node(pred_true,true,-1,node(pred_false,true,0,empty,empty),empty))),set(boolean),[])),set(boolean),[]),b(value(avl_set(node(pred_false,true,1,empty,node(pred_true,true,0,empty,empty)))),set(boolean),[])),set(boolean),[]),
            b(general_union(b(set_extension([b(value([pred_false,pred_true,pred_true,pred_false,pred_false,pred_true,pred_true,pred_true]),set(boolean),[])]),set(set(boolean)),[])),set(boolean),[]),
            b(intersection(b(set_extension([b(integer(-6303),integer,[]),b(integer(18362),integer,[]),b(integer(16442),integer,[]),b(integer(-15665),integer,[]),b(integer(18289),integer,[])]),set(integer),[]),b(value([int(-16422),int(-16366),int(-15490),int(-14410),int(-12186),int(-11100)]),set(integer),[])),set(integer),[]),
            b(set_extension([]),set(integer),[])]:prob_ast_set_expr)).
% mutation of sequence_expressions
test6 :-
    init_pref ,
    fuzz(test2_aux/1,10000,
        mutation([
            b(concat(b(front(b(sequence_extension([b(string('oY@OBd#Ri!>Ds\\Us|'),string,[]),b(string('#%o5MP$b0q_InWD'),string,[]),b(string('*N?:zU4\\78n[vV'),string,[]),b(string('4.`tA6c;|'),string,[]),b(string('vWer7i)lO@~Q!1]'),string,[]),b(string('0'),string,[]),b(string('Kd'),string,[]),b(string('zcMyHHnc<gT]\\^[dHT'),string,[])]),seq(string),[])),seq(string),[]),b(sequence_extension([b(string(u),string,[]),b(string('};V(9Fuzh^2q`oY'),string,[]),b(string('zl*WD'),string,[]),b(string(oalUaD),string,[]),b(string('KAFeT8:IT\\S'),string,[])]),seq(string),[])),seq(string),[]),
            b(tail(b(sequence_extension([b(integer(-18837),integer,[]),b(integer(-7726),integer,[]),b(integer(15128),integer,[]),b(integer(-1800),integer,[]),b(integer(-16838),integer,[]),b(integer(-11056),integer,[]),b(integer(-9539),integer,[]),b(integer(6246),integer,[]),b(integer(18344),integer,[]),b(integer(11487),integer,[]),b(integer(-6982),integer,[]),b(integer(2270),integer,[]),b(integer(2702),integer,[]),b(integer(-14619),integer,[]),b(integer(-10634),integer,[])]),seq(integer),[])),seq(integer),[])
            ]:prob_ast_seq_expr)).
%%%%%%%%%%%

%%%%%%%%%%% kernel_equality tests
% for every generated value integers equality_objects/3 will return
% either pred_true or pred_false
test7 :-
    fuzz(kernel_equality:equality_objects/3,10000,
         prob_value_integer([]):prob_value_integer([]):var).

% always fail for strings or value sets of different size or type
test8_aux(String1,String2) :-
    kernel_equality:equality_objects(String1,String2,Res) , Res = pred_false.

test8 :-
    fuzz(test8_aux/2,10000,
         prob_value_string([size:111]):
         prob_value_string([size:110])).
test9 :-
    fuzz(test8_aux/2,10000,
         prob_value_set(integer([]),[size:53]):
         prob_value_set(integer([]),[size:102])).

test10 :-
    fuzz(kernel_equality:and_equality/3,10000,
        prob_value_boolean([]):prob_value_boolean([]):var).

test11 :-
    fuzz(kernel_equality:or_eq_obj/3, 10000,
        prob_value_boolean([]):prob_value_boolean([]):var).

test12 :-
    fuzz(kernel_equality:equality_objects/3,10000,
         prob_value_set(set(integer([]),[size:4]),[]):
         prob_value_set(set(integer([]),[size:4]),[]):
         var).

% predicate should terminate for any set of string
test13 :-
    init_pref,
    fuzz(kernel_equality:subset_test/4,10000,
         prob_value_set(string([]),[]):
         prob_value_set(string([]),[]):var:var).

test14 :-
    fuzz(kernel_equality:subset_test/4,10000,
         prob_value_set(set(integer([]),[size:2]),[list]):
         prob_value_set(set(integer([]),[size:3]),[list]):var:var).

test15 :-
    init_pref ,
    fuzz(kernel_equality:subset_strict_test/4,10000,
         prob_value_set(integer([]),[avl]):
         prob_value_set(integer([]),[avl]):var:var).

test16 :-
    init_pref ,
    fuzz(kernel_equality:subset_strict_test/4,10000,
         prob_value_set(set(string([]),[size:2]),[avl,size:2]):
         prob_value_set(set(string([]),[size:2]),[list,size:4]):var:var).

test17 :-
    init_pref,
    fuzz(kernel_equality:membership_test_wf/4,10000,
         prob_value_set(set(string([]),[size:5]),[]):
         prob_value_set(string([]),[list,size:5]):var:var).

test18 :-
    init_pref,
    fuzz(kernel_equality:membership_test_wf/4,10000,
         prob_value_set(integer([]),[]):
         prob_value_integer([]):var:var).

test19 :-
    init_pref,
    fuzz(kernel_equality:membership_test_wf/4,10000,
         prob_value_set(set(integer([]),[size:2]),[avl,size:6]):
         prob_value_set(integer([]),[list,size:2]):var:var).

test20_aux(AnyValue) :-
    error_manager:reset_errors,
    kernel_equality:equality_objects(AnyValue,AnyValue,Res), Res == pred_true.

test20 :-
    init_pref,
    fuzz(test20_aux/1,10000,prob_value_any([small])).

% integer node and its mutation should be evaluated to the same value
test21_aux(IntegerNode) :-
    generate(mutation(IntegerNode:prob_ast_int_expr),Mutation) ,
    b_compute_expression_nowf(IntegerNode,[],[],Value) ,
    b_compute_expression_nowf(Mutation,[],[],MutationValue) , ! ,
    kernel_objects:equal_object(Value,MutationValue).
test21 :-
    init_pref ,
    fuzz(test21_aux/1,10000,prob_ast_integer([])).

% generate equality nodes with two records of the same type using the same field names
test22 :-
    init_pref ,
    fuzz(test1_aux/1,10000,prob_ast_pred(equal,[record])).
%%%%%%%%%%%

%%%%%%%%%%% kernel_objects tests
test23 :-
    fuzz(kernel_objects:add_element/3,10000,
        prob_value_integer([]):prob_value_set(integer([]),[]):var).
test24 :-
    fuzz(kernel_objects:add_element/3,10000,
        prob_value_set(string([]),[size:2]):prob_value_set(set(string([]),[size:5]),[size:3]):var).

test25_aux([Value1,Value2]) :-
    ( kernel_objects:equal_object(Value1,Value2)
    ; kernel_objects:not_equal_object(Value1,Value2)).
test25 :-
    init_pref ,
    fuzz(test25_aux/1,10000,
        % generate two elements of any but the same type, therefore generate a list
        prob_value_set(any([noset]),[list,size:2,not-well-defined-values])).
%%%%%%%%%%%

%%%%%%%%%%% smt-solver tests

% there shouldn't be any conflicting solutions when
% comparing ProB, Z3 and KodKod
%test26_aux(AST) :-
%    error_manager:reset_errors ,
%    (  solver_interface:solve_predicate(AST,_,ResultProB) ,
%       solver_interface:solve_predicate(AST,_,1,[try_kodkod_on_load/true],ResultKodKod) ,
%       smt_solvers_interface:smt_solve_predicate(z3,AST,_,ResultZ3) ,
%        (\+conflicting_results([ResultProB,ResultZ3,ResultKodKod]) ;
%         ResultZ3 = no_solution_found(unsupported_type_or_expression(_))))
%    ;  get_error(well_definedness_error,_)).
%test26 :-
%    init_pref ,
%    smt_solvers_interface:init ,
%    fuzz(test26_aux/1,1000,prob_ast_sat([maxID:3,size:3,posInterval,maxInterval:10])).

%conflicting_results(L) :-
%    member(solution(_),L) ,
%    %Contradiction =.. [contradiction_found|_] ,
%    member(contradiction_found(_),L).

% if ProB finds a solutions, Z3 and KodKod should accept this solution
% else Z3 and KodKod shouldn't find any solution too
%test27_aux(AST) :-
%    error_manager:reset_errors ,
%    solver_interface:solve_predicate(AST,_,ResultProB) ,
%    ( ResultProB = solution(Solution) ->
%        % add the found solution to the ast and call Z3 and KodKod
%        add_solution_to_ast(AST,Solution,NewAST) ,         % Todo: create add_solution_to_ast/3
%        solver_interface:solve_predicate(NewAST,_,1,[try_kodkod_on_load/true],ResultKodKod) ,
%        smt_solvers_interface:smt_solve_predicate(z3,NewAST,_,ResultZ3) ,
%        \+member(contradiction_found,[ResultZ3,ResultKodKod])
%    ;   % or all solver will find a contradiction
%        solver_interface:solve_predicate(AST,_,1,[try_kodkod_on_load/true],ResultKodKod) ,
%        smt_solvers_interface:smt_solve_predicate(z3,AST,_,ResultZ3) ,
%        \+member(solution(_),[ResultZ3,ResultKodKod])).
%test27 :-
%    init_pref ,
%    smt_solvers_interface:init ,
%    fuzz(test27_aux/1,1000,prob_ast_sat([maxID:3,size:3,posInterval,maxInterval:10])).

% find all solutions using ProB, Z3 (and KodKod) and compare the amount of solutions
% Todo: don't use findall for ProB (and KodKod)
%test28_aux(AST) :-
%    error_manager:reset_errors ,
%    ((findall(ResultProB,solver_interface:solve_predicate(AST,_,ResultProB),TempListProB) ,
%      findall(ResultZ3,smt_solvers_interface:smt_solve_predicate(z3,AST,_,ResultZ3),TempListZ3)) ;
%      % skip unsupported expressions
%      ( member(no_solution_found(unsupported_type_or_expression(_)),[ResultZ3])
%      ; get_error(well_definedness_error,_) )) ,

%    maplist(remove_dups,[TempListProB,TempListZ3],[ResultListProB,ResultListZ3]) ,
%    maplist(length,[ResultListProB,ResultListZ3],[AmountProB,AmountZ3]) ,
%    (AmountProB = AmountZ3 ->
%        % ProB and Z3 found the same amount of solutions
%        true
%    ;   nl , nl ,
%        format('Solvers found different amount of solutions:',[]) , nl ,
%        format('ProB: ~w',[AmountProB]) , nl ,
%        format('Z3: ~w',[AmountZ3]) , nl , ! ,
%        fail).
%test28 :-
%    init_pref ,
%    smt_solvers_interface:init ,
%    fuzz(test28_aux/1,1000,prob_ast_sat([maxID:3,size:3,posInterval,maxInterval:10])).

%%%%%%%%%%% typechecker tests
test29_aux(AST) :-
    generate(detype(AST),DetypedAST) ,
    btypechecker:btype(fuzzing,DetypedAST,env_empty,_NType,TypedAST,[],_TRout) ,
    (( b_compute_expression_nowf(AST,[],[],Res1) ,
       b_compute_expression_nowf(TypedAST,[],[],Res2) ,
       kernel_objects:equal_object(Res1,Res2) )
    ;  get_error(well_definedness_error,_)).
test29 :-
    init_pref ,
    fuzz(test29_aux/1,10000,prob_ast_int_expr([])).

test30_aux(AST) :-
    % remove types and re-run typechecker and check that either both ASTs succeed or both fail
    %nl,nl,print('Test 30 : '),translate:print_bexpr(AST),nl,
    generate(detype(AST),DetypedAST) ,
    % TO DO: detype sometimes removes types for empty set or empty sequence or sequence_extension([]) ...
    % TO DO: find a better entry point predicate in btypechecker or provide it there
    btypechecker:typecheck_result_init(_ErrorsIn,_Delayed,_Identifiers,TRStart),
    btypechecker:btype(fuzzing,DetypedAST,openenv(_,env_empty),_Type,TypedAST,TRStart,TRout) ,
    btypechecker:typecheck_result_get_errors(TRout,Errors),
    (var(Errors) -> true ; format('TYPE ERRORS: ~w~n',[Errors]),translate:print_bexpr(AST),nl,fail),
    %print('Re-typed: '),translate:print_bexpr(TypedAST),nl,
    (  b_test_boolean_expression_wf(AST,[],[]) ,
       b_test_boolean_expression_wf(TypedAST,[],[])
    ;  b_not_test_boolean_expression_wf(AST,[],[]) ,
       b_not_test_boolean_expression_wf(TypedAST,[],[])
    ;  get_error(well_definedness_error,_)
    ).
test30 :-
    init_pref ,
    fuzz(test30_aux/1,1000,prob_ast_pred([alph,noQuantifier])).
%%%%%%%%%%%

%%%%%%%%%%% automated Kodkod annotation
conflicting_solutions(contradiction_found, solution(_)).
conflicting_solutions(solution(_), contradiction_found).

no_conflict_due_to_annotation(Pred) :-
  get_preference(sat_supported_interpreter, Pref),
  temporary_set_preference(sat_supported_interpreter, false),
  solve_predicate(Pred, _, Sol),
  temporary_set_preference(sat_supported_interpreter, true),
  solve_predicate(Pred, _, AnnotatedSol),
  set_preference(sat_supported_interpreter, Pref),
  \+conflicting_solutions(Sol, AnnotatedSol).

test_automated_kodkod_annotation :-
  fuzz(kodkod_annotator:annotate_kodkod_constraints/3, 100000, 500, fixed(greedy):prob_ast_pred([]):var),
  !,
  fuzz(kodkod_annotator:annotate_kodkod_constraints/3, 100000, 500, fixed(greedy):prob_ast_expr([]):var),
  !,
  fuzz(kodkod_annotator:annotate_kodkod_constraints/3, 100000, 500, fixed(preferred):prob_ast_pred([]):var),
  !,
  fuzz(kodkod_annotator:annotate_kodkod_constraints/3, 100000, 500, fixed(preferred):prob_ast_expr([]):var),
  !.

test_automated_kodkod_annotation_solving :-
  get_preference(time_out, Timeout),
  fuzz(user:no_conflict_due_to_annotation/1, 1000, Timeout, prob_ast_pred([])),
  !.
%%%%%%%%%%%
