% (c) 2009-2019 Lehrstuhl fuer Softwaretechnik und Programmiersprachen,
% Heinrich Heine Universitaet Duesseldorf
% This software is licenced under EPL 1.0 (http://www.eclipse.org/org/documents/epl-v10.html)

:- module(translate,
          [print_bexpr_or_subst/1, l_print_bexpr_or_subst/1,
           print_bexpr/1, debug_print_bexpr/1, nested_print_bexpr/1, nested_print_bexpr_as_classicalb/1,
           print_bexpr_stream/2,
           print_components/1,
           print_bexpr_with_limit/2,
           print_unwrapped_bexpr_with_limit/1,print_bvalue/1, l_print_bvalue/1, print_bvalue_stream/2,
           translate_params_for_dot/2,
           print_machine/1,
           translate_machine/3,
           set_unicode_mode/0, unset_unicode_mode/0, unicode_mode/0,
           set_latex_mode/0, unset_latex_mode/0, latex_mode/0,
           set_force_eventb_mode/0, unset_force_eventb_mode/0,
           set_translation_mode/1, unset_translation_mode/1,
           translate_bexpression_to_unicode/2,
           translate_bexpression/2, translate_subst_or_bexpr_in_mode/3,
           translate_bexpression_with_limit/3, translate_bexpression_with_limit/2,
           translate_bexpression_to_codes/2,
           translate_predicate_into_machine/2,
           get_bexpression_column_template/4,
           translate_subst_or_bexpr/2, translate_subst_or_bexpr_with_limit/3,
           translate_substitution/2, print_subst/1,
           translate_bvalue/2, translate_bvalue_to_codes/2,
           translate_bvalue_with_limit/3,
           translate_bvalue_with_type/3, translate_bvalue_with_type_and_limit/4,
           translate_bvalue_for_expression/3, translate_bvalue_for_expression_with_limit/4,
           translate_bvalue_with_tlatype/3,
           print_state/1,
           translate_bstate/2, translate_bstate_limited/2,
           print_bstate/1, translate_context/2, print_context/1,
           translate_any_state/2,  %translate_b_state_to_comma_list_codes/3,
           print_value_variable/1,
           print_cspm_state/1, translate_cspm_state/2,
           print_csp_value/1, translate_csp_value/2,
           translate_cspm_expression/2,
           translate_properties_with_limit/2,
           translate_event/2,translate_events/2,
           translate_event_with_target_id/4,
           translate_event_with_src_and_target_id/4, translate_event_with_src_and_target_id/5,
           get_non_det_modified_vars_in_target_id/3,
           translate_event_with_limit/3,
           translate_state_errors/2,translate_state_error/2,
           translate_event_error/2,
           translate_prolog_constructor/2, translate_prolog_constructor_in_mode/2,
           pretty_type/2,
           explain_state_error/3,
           explain_event_trace/2,
           generate_typing_predicates/2,

           print_raw_machine_terms/1,
           print_raw_bexpr/1,

           print_span/1, translate_span/2,
           translate_span_with_filename/2,

           set_type_to_maximal_texpr/2, type_set/2, % should probably be moved to other module

           translate_error_term/2,

           set_translation_constants/1, set_translation_context/1,
           clear_translation_constants/0,

           set_print_type_infos/1,
           suppress_rodin_positions/0,

           install_b_portray_hook/0,remove_b_portray_hook/0,

           translate_eventb_to_classicalb/3,
           return_csp_closure_value/2

           ]).

:- use_module(tools).

:- use_module(module_information).
:- module_info(group,tools).
:- module_info(description,'This module is responsible for pretty-printing B and CSP, source spans, ...').

:- use_module(library(lists)).
:- use_module(library(codesio)).
:- use_module(library(terms)).
:- use_module(library(avl)).

:- use_module(debug).
:- use_module(error_manager).
:- use_module(self_check).
:- use_module(b_global_sets).
:- use_module(specfile,[csp_with_bz_mode/0,process_algebra_mode/0,
                        animation_minor_mode/1,set_animation_minor_mode/1,
                        remove_animation_minor_mode/0,
                        animation_mode/1,set_animation_mode/1, csp_mode/0,
                        translate_operation_name/2]).
:- use_module(bsyntaxtree).
%:- use_module('smv/smv_trans',[smv_print_initialisation/2]).
:- use_module(preferences,[get_preference/2, set_preference/2]).
:- use_module(bmachine_structure).
:- use_module(avl_tools,[check_is_non_empty_avl/1]).

% print a list of expressions or substitutions
l_print_bexpr_or_subst([]).
l_print_bexpr_or_subst([H|T]) :-
   print_bexpr_or_subst(H),
   (T=[] -> true
    ; (get_texpr_type(H,Type),is_subst_type(Type) -> print('; ') ; print(', '),
       l_print_bexpr_or_subst(T))
    ).

is_subst_type(T) :- var(T),!,fail.
is_subst_type(subst).
is_subst_type(op(_,_)).

print_bexpr_or_subst(E) :- get_texpr_type(E,T),is_subst_type(T),!, print_subst(E).
print_bexpr_or_subst(precondition(A,B)) :- !, print_subst(precondition(A,B)).
print_bexpr_or_subst(any(A,B,C)) :- !, print_subst(any(A,B,C)).
print_bexpr_or_subst(select(A)) :- !, print_subst(select(A)). % TO DO: add more cases ?
print_bexpr_or_subst(E) :- print_bexpr(E).

print_unwrapped_bexpr_with_limit(Expr) :- print_unwrapped_bexpr_with_limit(Expr,200).
print_unwrapped_bexpr_with_limit(Expr,Limit) :-
   translate:print_bexpr_with_limit(b(Expr,pred,[]),Limit),nl.
debug_print_bexpr(E) :- debug:debug_mode(off) -> true ; print_bexpr(E).
print_bexpr(Expr) :- translate_bexpression(Expr,R), write(R).
print_bexpr_with_limit(Expr,Limit) :- translate_bexpression_with_limit(Expr,Limit,R), write(R).
print_bvalue(Val) :- translate_bvalue(Val,TV), write(TV).
print_bexpr_stream(S,Expr) :- translate_bexpression(Expr,R), write(S,R).
print_bvalue_stream(S,Val) :- translate_bvalue(Val,R), write(S,R).

print_components(C) :- print_components(C,0).
print_components([],Nr) :- write('Nr of components: '),write(Nr),nl.
print_components([component(Pred,Vars)|T],Nr) :- N1 is Nr+1,
   write('Component: '), write(N1), write(' over '), write(Vars),nl,
   print_bexpr(Pred),nl,
   print_components(T,N1).

l_print_bvalue([]).
l_print_bvalue([H|T]) :- print_bvalue(H), write(' : '),l_print_bvalue(T).

nested_print_bexpr_as_classicalb(E) :-
   (animation_minor_mode(X)
    -> remove_animation_minor_mode,
       call_cleanup(nested_print_bexpr(E), set_animation_minor_mode(X))
     ; nested_print_bexpr(E)).

nested_print_bexpr([]) :- !.
nested_print_bexpr([H|T]) :- !,nested_print_bexpr(H),nl,nested_print_bexpr(T).
nested_print_bexpr(Expr) :- nbp(Expr,'&',0).

nbp(b(E,_,Info),Type,Indent) :- nbp2(E,Type,Info,Indent).
nbp2(E,Type,_Info,Indent) :- get_binary_op(E,NewType,LHS,RHS),!,
  inc_indent(NewType,Type,Indent,NIndent),
  print_bracket(Indent,NIndent,'('),
  nbp(LHS,NewType,NIndent),
  print_indent(NIndent), print(NewType),nl,
  nbp(RHS,NewType,NIndent),
  print_bracket(Indent,NIndent,')').
nbp2(lazy_let_pred(TID,LHS,RHS),Type,_Info,Indent) :-
  def_get_texpr_id(TID,ID),!,
  NewType=lazy_let_pred(TID),
  inc_indent(NewType,Type,Indent,NIndent),
  print_indent(Indent), format('LET ~w = (~n',[ID]),
  nbp(LHS,NewType,NIndent),
  print_indent(NIndent), print(') IN ('),nl,
  nbp(RHS,NewType,NIndent),
  print_indent(NIndent),print(')'),nl.
nbp2(negation(LHS),_Type,_Info,Indent) :- !,
  inc_indent(negation,false,Indent,NIndent),
  print_indent(Indent), print('not('),nl,
  nbp(LHS,negation,NIndent),
  print_indent(Indent), print(')'),nl.
nbp2(let_predicate(Ids,Exprs,Pred),_Type,_Info,Indent) :- !,
  inc_indent(let_predicate,false,Indent,NIndent),
  pp_expr_ids_in_mode(Ids,_LR,Codes,[]),
  print_indent(Indent),format('#~s.( /* LET */~n',[Codes]),
  pp_expr_let_pred_exprs(Ids,Exprs,_LimitReached,Codes2,[]),
  print_indent(Indent), format('~s~n',[Codes2]),
  print_indent(NIndent), print('&'),nl,
  nbp(Pred,let_predicate,NIndent),
  print_indent(Indent), print(')'),nl.
nbp2(exists(Ids,Pred),_Type,_Info,Indent) :- !,
  inc_indent(exists,false,Indent,NIndent),
  pp_expr_ids_in_mode(Ids,_LR,Codes,[]),
  print_indent(Indent),format('#~s.(~n',[Codes]),
  nbp(Pred,exists,NIndent),
  print_indent(Indent), print(')'),nl.
nbp2(forall(Ids,LHS,RHS),_Type,_Info,Indent) :- !,
  inc_indent(forall,false,Indent,NIndent),
  pp_expr_ids_in_mode(Ids,_LR,Codes,[]),
  print_indent(Indent),format('!~s.(~n',[Codes]),
  nbp(LHS,forall,NIndent),
  print_indent(NIndent), print('=>'),nl,
  nbp(RHS,forall,NIndent),
  print_indent(Indent), print(')'),nl.
nbp2(E,_,Info,Indent) :- print_indent(Indent), print_bexpr(b(E,pred,Info)),nl.

get_binary_op(conjunct(LHS,RHS),'&',LHS,RHS).
get_binary_op(disjunct(LHS,RHS),'or',LHS,RHS).
get_binary_op(implication(LHS,RHS),'=>',LHS,RHS).
get_binary_op(equivalence(LHS,RHS),'<=>',LHS,RHS).

inc_indent(Type,CurType,I,NewI) :- (Type=CurType -> NewI=I ; NewI=s(I)).
print_bracket(I,I,_) :- !.
print_bracket(I,_NewI,Bracket) :-
   print_indent(I), print(Bracket),nl.

print_indent(s(X)):- !,
    print('  '),
    print_indent(X).
print_indent(_).

/* =============================================== */
/* Translating expressions and values into strings */
/* =============================================== */





translate_params_for_dot(List,TransList) :- translate_params_for_dot(List,TransList,-3).
translate_params_for_dot([],'',_).
translate_params_for_dot([H|T],Res,Nr) :-
  translate_property_with_limit(H,100,TH),
  (Nr>2 -> N1=1 ; N1 is Nr+1),
  translate_params_for_dot(T,TT,N1),
  string_concatenate(TH,TT,Res1),
  (N1=1 -> string_concatenate(',\n',Res1,Res)
    ; (Nr>(-3) -> string_concatenate(',',Res1,Res) ; Res=Res1)).

translate_channel_values(X,['_'|T],T) :- var(X),!.
translate_channel_values([],S,S) :- !.
translate_channel_values([tuple([])|T],S,R) :- !,
  translate_channel_values(T,S,R).
translate_channel_values([in(tuple([]))|T],S,R) :- !,
  translate_channel_values(T,S,R).
translate_channel_values([H|T],['.'|S],R) :-  !,
  ((nonvar(H),H=in(X))
    -> Y=X
    ;  Y=H
    ),
  pp_csp_value(Y,S,S2),
  translate_channel_values(T,S2,R).
translate_channel_values(tail_in(X),S,T) :-
      (X=[] ; X=[_|_]), !, translate_channel_values(X,S,T).
translate_channel_values(_X,['??'|S],S).



pp_single_csp_value(V,'_') :- var(V),!.
pp_single_csp_value(X,'_cyclic_') :- cyclic_term(X),!.
pp_single_csp_value(int(X),A) :- atomic(X),!,number_chars(X,Chars),atom_chars(A,Chars).

:- assert_must_succeed((translate_cspm_expression(listExp(rangeOpen(2)),R), R == '<2..>')).
:- assert_must_succeed((translate_cspm_expression(listFrom(2),R), R == '<2..>')).
:- assert_must_succeed((translate_cspm_expression(listFromTo(2,6),R), R == '<2..6>')).
:- assert_must_succeed((translate_cspm_expression(setFromTo(2,6),R), R == '{2..6}')).
:- assert_must_succeed((translate_cspm_expression('#'(listFromTo(2,6)),R), R == '#<2..6>')).
:- assert_must_succeed((translate_cspm_expression(inGuard(x,setFromTo(1,5)),R), R == '?x:{1..5}')).
:- assert_must_succeed((translate_cspm_expression(builtin_call(int(3)),R), R == '3')).
:- assert_must_succeed((translate_cspm_expression(set_to_seq(setValue([int(1),int(2)])),R), R == 'seq({1,2})')).
:- assert_must_succeed((translate_cspm_expression(diff(setValue([int(1)]),setValue([])),R), R == 'diff({1},{})')).
:- assert_must_succeed((translate_cspm_expression(inter(setValue([int(1)]),setValue([])),R), R == 'inter({1},{})')).
:- assert_must_succeed((translate_cspm_expression(lambda([x,y],'*'(x,y)),R), R == '\\ x,y @ (x*y)')).
:- assert_must_succeed((translate_cspm_expression(lambda([x,y],'/'(x,y)),R), R == '\\ x,y @ (x/y)')).
:- assert_must_succeed((translate_cspm_expression(lambda([x,y],'%'(x,y)),R), R == '\\ x,y @ (x%y)')).
:- assert_must_succeed((translate_cspm_expression(rename(x,y),R), R == 'x <- y')).
:- assert_must_succeed((translate_cspm_expression(link(x,y),R), R == 'x <-> y')).
:- assert_must_succeed((translate_cspm_expression(agent_call_curry(f,[[a,b],[c]]),R), R == 'f(a,b)(c)')).

translate_cspm_expression(Expr, Text) :-
    (pp_csp_value(Expr,Atoms,[]) -> ajoin(Atoms,Text)
    ; print('Pretty printing expression failed: '),print(Expr),nl).

pp_csp_value(X,[A|S],S) :- pp_single_csp_value(X,A),!.
pp_csp_value(setValue(L),['{'|S],T) :- !,pp_csp_value_l(L,',',S,['}'|T],inf).
pp_csp_value(setExp(rangeEnum(L)),['{'|S],T) :- !,pp_csp_value_l(L,',',S,['}'|T],inf).
pp_csp_value(setExp(rangeEnum(L),Gen),['{'|S],T) :- !,
      copy_term((L,Gen),(L2,Gen2)), numbervars((L2,Gen2),1,_),
      pp_csp_value_l(L2,',',S,['|'|S2],inf),
      pp_csp_value_l(Gen2,',',S2,['}'|T],inf).
pp_csp_value(avl_set(A),['{'|S],T) :- !, check_is_non_empty_avl(A),
      avl_domain(A,L),pp_csp_value_l(L,',',S,['}'|T],inf).
pp_csp_value(setExp(rangeClosed(L,U)),['{'|S],T) :- !, pp_csp_value(L,S,['..'|S2]),pp_csp_value(U,S2,['}'|T]).
pp_csp_value(setExp(rangeOpen(L)),['{'|S],T) :- !, pp_csp_value(L,S,['..}'|T]).
% TO DO: pretty print comprehensionGuard; see prints in coz-example.csp ; test 1846
pp_csp_value(comprehensionGenerator(Var,Body),S,T) :- !, pp_csp_value(Var,S,['<-'|S1]),
    pp_csp_value(Body,S1,T).
pp_csp_value(listExp(rangeEnum(L)),['<'|S],T) :- !,pp_csp_value_l(L,',',S,['>'|T],inf).
pp_csp_value(listExp(rangeClosed(L,U)),['<'|S],T) :- !, pp_csp_value(L,S,['..'|S2]),pp_csp_value(U,S2,['>'|T]).
pp_csp_value(listExp(rangeOpen(L)),['<'|S],T) :- !, pp_csp_value(L,S,['..>'|T]).
pp_csp_value(setFromTo(L,U),['{'|S],T) :- !,
   pp_csp_value(L,S,['..'|S2]),pp_csp_value(U,S2,['}'|T]).
pp_csp_value(setFrom(L),['{'|S],T) :- !,
   pp_csp_value(L,S,['..}'|T]).
pp_csp_value(closure(L), ['{|'|S],T) :- !,pp_csp_value_l(L,',',S,['|}'|T],inf).
pp_csp_value(list(L),['<'|S],T) :- !,pp_csp_value_l(L,',',S,['>'|T],inf).
pp_csp_value(listFromTo(L,U),['<'|S],T) :- !,
   pp_csp_value(L,S,['..'|S2]),pp_csp_value(U,S2,['>'|T]).
pp_csp_value(listFrom(L),['<'|S],T) :- !,
   pp_csp_value(L,S,['..>'|T]).
pp_csp_value('#'(L),['#'|S],T) :- !,pp_csp_value(L,S,T).
pp_csp_value('^'(X,Y),S,T) :- !,pp_csp_value(X,S,['^'|S1]), pp_csp_value(Y,S1,T).
pp_csp_value(linkList(L),S,T) :- !,pp_csp_value_l(L,',',S,T,inf).
pp_csp_value(in(X),['?'|S],T) :- !,pp_csp_value(X,S,T).
pp_csp_value(inGuard(X,Set),['?'|S],T) :- !,pp_csp_value(X,S,[':'|S1]),
  pp_csp_value(Set,S1,T).
pp_csp_value(out(X),['!'|S],T) :- !,pp_csp_value(X,S,T).
pp_csp_value(alsoPat(X,_Y),S,T) :- !,pp_csp_value(X,S,T).
pp_csp_value(appendPat(X,_Fun),S,T) :- !,pp_csp_value(X,S,T).
pp_csp_value(tuple(vclosure),S,T) :- !, S=T.
pp_csp_value(tuple([X]),S,T) :- !,pp_csp_value_in(X,S,T).
pp_csp_value(tuple([X|vclosure]),S,T) :- !,pp_csp_value_in(X,S,T).
pp_csp_value(tuple([H|TT]),S,T) :- !,pp_csp_value_in(H,S,['.'|S1]),pp_csp_value(tuple(TT),S1,T).
pp_csp_value(dotTuple([]),['unit_channel'|S],S) :- ! .
pp_csp_value(dotTuple([H]),S,T) :- !, pp_csp_value_in(H,S,T).
pp_csp_value(dotTuple([H|TT]),S,T) :- !, pp_csp_value_in(H,S,['.'|S1]),
  pp_csp_value(dotTuple(TT),S1,T).
pp_csp_value(tupleExp(Args),S,T) :- !,pp_csp_args(Args,S,T,'(',')').
pp_csp_value(na_tuple(Args),S,T) :- !,pp_csp_args(Args,S,T,'(',')').
pp_csp_value(record(Name,Args),['('|S],T) :- !,pp_csp_value(tuple([Name|Args]),S,[')'|T]).
pp_csp_value(val_of(Name,_Span),S,T) :- !, pp_csp_value(Name,S,T).
pp_csp_value(builtin_call(X),S,T) :- !,pp_csp_value(X,S,T).
pp_csp_value(seq_to_set(X),['set('|S],T) :- !,pp_csp_value(X,S,[')'|T]).
pp_csp_value(set_to_seq(X),['seq('|S],T) :- !,pp_csp_value(X,S,[')'|T]).
%pp_csp_value('\\'(B,C,S),S1,T) :- !, pp_csp_process(ehide(B,C,S),S1,T).
pp_csp_value(agent_call(_Span,Agent,Parameters),['('|S],T) :- !,
  pp_csp_value(Agent,S,S1),
  pp_csp_args(Parameters,S1,[')'|T],'(',')').
pp_csp_value(agent_call_curry(Agent,Parameters),S,T) :- !,
  pp_csp_value(Agent,S,S1),
  pp_csp_curry_args(Parameters,S1,T).
pp_csp_value(lambda(Parameters,Body),['\\ '|S],T) :- !,
  pp_csp_args(Parameters,S,[' @ '|S1],'',''),
  pp_csp_value(Body,S1,T).
pp_csp_value(rename(X,Y),S,T) :- !,pp_csp_value(X,S,[' <- '|S1]),
  pp_csp_value(Y,S1,T).
pp_csp_value(link(X,Y),S,T) :- !,pp_csp_value(X,S,[' <-> '|S1]),
  pp_csp_value(Y,S1,T).
% binary operators:
pp_csp_value(Expr,['('|S],T) :- bynary_numeric_operation(Expr,E1,E2,OP),!,
    pp_csp_value(E1,S,[OP|S2]),
    pp_csp_value(E2,S2,[')'|T]).
% built-in functions for sets
pp_csp_value(empty(A),[empty,'('|S],T) :- !, pp_csp_value(A,S,[')'|T]).
pp_csp_value(card(A),[card,'('|S],T) :- !, pp_csp_value(A,S,[')'|T]).
pp_csp_value('Set'(A),['Set','('|S],T) :- !, pp_csp_value(A,S,[')'|T]).
pp_csp_value('Inter'(A1),['Inter','('|S],T) :- !,pp_csp_value(A1,S,[')'|T]).
pp_csp_value('Union'(A1),['Union','('|S],T) :- !,pp_csp_value(A1,S,[')'|T]).
pp_csp_value(diff(A1,A2),[diff,'('|S],T) :- !,pp_csp_args([A1,A2],S,[')'|T],'','').
pp_csp_value(inter(A1,A2),[inter,'('|S],T) :- !,pp_csp_args([A1,A2],S,[')'|T],'','').
pp_csp_value(union(A1,A2),[union,'('|S],T) :- !,pp_csp_args([A1,A2],S,[')'|T],'','').
pp_csp_value(member(A1,A2),[member,'('|S],T) :- !,pp_csp_args([A1,A2],S,[')'|T],'','').
% built-in functions for sequences
pp_csp_value(null(A),[null,'('|S],T) :- !, pp_csp_value(A,S,[')'|T]).
pp_csp_value(length(A),[length,'('|S],T) :- !, pp_csp_value(A,S,[')'|T]).
pp_csp_value(head(A),[head,'('|S],T) :- !, pp_csp_value(A,S,[')'|T]).
pp_csp_value(tail(A),[tail,'('|S],T) :- !, pp_csp_value(A,S,[')'|T]).
pp_csp_value(elem(A1,A2),[elem,'('|S],T) :- !,pp_csp_args([A1,A2],S,[')'|T],'','').
pp_csp_value(concat(A1,A2),[concat,'('|S],T) :- !,pp_csp_args([A1,A2],S,[')'|T],'','').
pp_csp_value('Seq'(A),['Seq','('|S],T) :- !, pp_csp_value(A,S,[')'|T]).
% vclosure
pp_csp_value(Expr,S,T) :- is_list(Expr),!,pp_csp_value(closure(Expr),S,T).
% Type expressions
pp_csp_value(dotTupleType([H]),S,T) :- !, pp_csp_value_in(H,S,T).
pp_csp_value(dotTupleType([H|TT]),S,T) :- !,pp_csp_value_in(H,S,['.'|S1]), pp_csp_value(dotTupleType(TT),S1,T).
pp_csp_value(typeTuple(Args),S,T) :- !, pp_csp_args(Args,S,T,'(',')').
pp_csp_value(dataType(T),[T|S],S) :- ! .
pp_csp_value(boolType,['Bool'|S],S) :- ! .
pp_csp_value(intType,['Int'|S],S) :- ! .
pp_csp_value(dataTypeDef([H]),S,T) :- !, pp_csp_value(H,S,T).
pp_csp_value(dataTypeDef([H|TT]),S,T) :- !, pp_csp_value(H,S,['|'|S1]),
  pp_csp_value(dataTypeDef(TT),S1,T).
pp_csp_value(constructor(Name),[Name|S],S) :- ! .
pp_csp_value(constructorC(C,Type),[C,'('|S],T) :- !, pp_csp_value(Type,S,[')'|T]).
% Argument of function can be process

pp_csp_value(Expr,S,T) :- pp_csp_process(Expr,S,T),!. % pp_csp_process has a catch-all !!! TO DO: look at this
pp_csp_value(Expr,S,T) :- csp_with_bz_mode,!,pp_value(Expr,S,T).
pp_csp_value(X, [A|S], S) :- % ['<< ',A,' >>'|S],S) :-  % the << >> pose problems when checking against FDR
    write_to_codes(X,Codes),atom_codes_with_limit(A,Codes).

pp_csp_value_in(H,S,T) :- nonvar(H),H=in(X),!, pp_csp_value(X,S,T).
pp_csp_value_in(H,S,T) :- pp_csp_value(H,S,T).

print_csp_value(Val) :- pp_csp_value(Val,Atoms,[]), ajoin(Atoms,Text),
  print(Text).

translate_csp_value(Val,Text) :- pp_csp_value(Val,Atoms,[]), ajoin(Atoms,Text).

return_csp_closure_value(closure(S),List) :- pp_csp_value_l1(S,List).
return_csp_closure_value(setValue(S),List) :- pp_csp_value_l1(S,List).

pp_csp_value_l1([Expr|Rest],List) :-
    ( nonvar(Rest),Rest=[] ->
       pp_csp_value(Expr,T,[]),ajoin(T,Value),List=[Value]
    ;  pp_csp_value_l1(Rest,R),pp_csp_value(Expr,T,[]),ajoin(T,Value),List=[Value|R]
    ).

pp_csp_args([],T,T,_LPar,_RPar).
pp_csp_args([H|TT],[LPar|S],T,LPar,RPar) :- pp_csp_value(H,S,S1), pp_csp_args2(TT,S1,T,RPar).
pp_csp_args2([],[RPar|T],T,RPar).
pp_csp_args2([H|TT],[','|S],T,RPar) :- pp_csp_value(H,S,S1), pp_csp_args2(TT,S1,T,RPar).

pp_csp_curry_args([],T,T).
pp_csp_curry_args([H|TT],S,T) :- is_list(H), pp_csp_args(H,S,S1,'(',')'), pp_csp_curry_args(TT,S1,T).

pp_csp_value_l(V,_Sep,['...'|S],S,N) :- (var(V) ; (N \= inf -> N<1;fail)), !.
pp_csp_value_l([],_Sep,S,S,_).
pp_csp_value_l([Expr|Rest],Sep,S,T,Nr) :-
    ( nonvar(Rest),Rest=[] ->
        pp_csp_value(Expr,S,T)
    ; otherwise ->
        (Nr=inf -> N1 = Nr ; N1 is Nr-1),
        pp_csp_value(Expr,S,[Sep|S1]),pp_csp_value_l(Rest,Sep,S1,T,N1)).

:- assert_must_succeed((translate:convert_set_into_sequence([(int(1),int(5))],Seq),
                        check_eqeq(Seq,[int(5)]))).
:- assert_must_succeed((translate:convert_set_into_sequence([(int(2),X),(int(1),int(5))],Seq),
                        check_eq(Seq,[int(5),X]))).

convert_set_into_sequence(Set,Seq) :-
    nonvar(Set), \+ eventb_translation_mode,
    convert_set_into_sequence1(Set,Seq).
convert_set_into_sequence1(avl_set(A),Seq) :- !, check_is_non_empty_avl(A),
     avl_size(A,Sz),size_is_in_set_limit(Sz),convert_avlset_into_sequence(A,Seq).
convert_set_into_sequence1([],Seq) :- !, Seq=[].
convert_set_into_sequence1(Set,Seq) :-
    convert_set_into_sequence2(Set,0,_,SetElems,Seq),ground(SetElems).
convert_set_into_sequence2([],_Max,([],[]),_,_Seq).
convert_set_into_sequence2([Pair|T],Max,Last,SetElems,Seq) :-
    nonvar(Pair),nonvar(T),Pair=(Index,H),ground(Index),
    Index=int(Nr),
    insert_el_into_seq(Nr,H,Seq,SetElems,L),
    (Nr>Max -> NMax=Nr,NLast=L ; NMax=Max,NLast=Last),
    convert_set_into_sequence2(T,NMax,NLast,SetElems,Seq).
insert_el_into_seq(1,H,[H|L],[set|L2],(L,L2)) :- !.
insert_el_into_seq(N,H,[_|T],[_|T2],Last) :- N>1, N1 is N-1, insert_el_into_seq(N1,H,T,T2,Last).

convert_avlset_into_sequence(Avl,Sequence) :-
    \+ eventb_translation_mode,
    convert_avlset_into_sequence2(Avl,1,Sequence).
convert_avlset_into_sequence2(Avl,_Index,[]) :-
    empty_avl(Avl),!.
convert_avlset_into_sequence2(Avl,Index,[Head|Tail]) :-
    avl_del_min(Avl, Min, _ ,NewAvl),
    nonvar(Min), Min=(L,Head),
    ground(L), L=int(Index),
    Index2 is Index + 1,
    convert_avlset_into_sequence2(NewAvl,Index2,Tail).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% translate new syntax tree -- work in progress
:- assert_must_succeed((translate_cspm_state(lambda([x,y],'|~|'(prefix(_,[],x,skip(_),_),prefix(_,[],y,skip(_),_),_)),R), R == 'CSP: \\ x,y @ (x->SKIP) |~| (y->SKIP)')).
:- assert_must_succeed((translate_cspm_state(agent_call_curry('F',[[a,b],[c]]),R), R == 'CSP: F(a,b)(c)')).
:- assert_must_succeed((translate_cspm_state(ifte(bool_not('<'(x,3)),';'(esharing([a],'/\\'('P1','P2',span),procRenaming([rename(r,s)],'Q',span),span),lParallel([link(b,c)],'R','S',span),span),'[>'(elinkParallel([link(h1,h2)],'G1','G2',span),exception([a],'H1','H2',span),span),span1,span2,span3),R), R == 'CSP:  if not((x<3)) then ((P1) /\\ (P2) [|{|a|}|] Q[[r <- s]]) ; (R [{|b <-> c|}] S) else (G1 [{|h1 <-> h2|}] G2) [> (H1 [|{|a|}|> H2)')).
:- assert_must_succeed((translate_cspm_state(aParallel([a,b],'P',[b,c],'Q',span),R), R == 'CSP: P [{|a,b|} || {|b,c|}] Q')).
:- assert_must_succeed((translate_cspm_state(eaParallel([a,b],'P',[b,c],'Q',span),R), R == 'CSP: P [{|a,b|} || {|b,c|}] Q')).
:- assert_must_succeed((translate_cspm_state(eexception([a,b],'P','Q',span),R), R == 'CSP: P [|{|a,b|}|> Q')).

print_cspm_state(State) :- translate_cspm_state(State,T), print(T).

translate_cspm_state(State,Text) :-
   ( pp_csp_process(State,Atoms,[]) -> true
     ; print(pp_csp_process_failed(State)),nl,Atoms=State),
   ajoin(['CSP: '|Atoms],Text).

pp_csp_process(skip(_Span),S,T) :- !, S=['SKIP'|T].
pp_csp_process(stop(_Span),S,T) :- !, S=['STOP'|T].
pp_csp_process('CHAOS'(_Span,Set),['CHAOS('|S],T) :- !,
  pp_csp_value(Set,S,[')'|T]).
pp_csp_process(val_of(Agent,_Span),S,T) :- !,
  pp_csp_value(Agent,S,T).
pp_csp_process(builtin_call(X),S,T) :- !,pp_csp_process(X,S,T).
pp_csp_process(agent(F,Body,_Span),S,T) :- !,
  F =.. [Agent|Parameters],
  pp_csp_value(Agent,S,S1),
  pp_csp_args(Parameters,S1,[' = '|S2],'(',')'),
  pp_csp_value(Body,S2,T).
pp_csp_process(agent_call(_Span,Agent,Parameters),S,T) :- !,
  pp_csp_value(Agent,S,S1),
  pp_csp_args(Parameters,S1,T,'(',')').
pp_csp_process(agent_call_curry(Agent,Parameters),S,T) :- !,
  pp_csp_value(Agent,S,S1),
  pp_csp_curry_args(Parameters,S1,T).
pp_csp_process(lambda(Parameters,Body),['\\ '|S],T) :- !,
  pp_csp_args(Parameters,S,[' @ '|S1],'',''),
  pp_csp_value(Body,S1,T).
pp_csp_process('\\'(B,C,S),S1,T) :- !, pp_csp_process(ehide(B,C,S),S1,T).
pp_csp_process(ehide(Body,ChList,_Span),['('|S],T) :- !,
  pp_csp_process(Body,S,[')\\('|S1]),
  pp_csp_value(ChList,S1,[')'|T]).
pp_csp_process(let(Decls,P),['let '| S],T) :- !,
  maplist(translate_cspm_state,Decls,Texts),
  ajoin_with_sep(Texts,' ',Text),
  S=[Text,' within '|S1],
  pp_csp_process(P,S1,T).
pp_csp_process(Expr,['('|S],T) :- binary_csp_op(Expr,X,Y,Op),!,
  pp_csp_process(X,S,[') ',Op,' ('|S1]),
  pp_csp_process(Y,S1,[')'|T]).
pp_csp_process(Expr,S,T) :- sharing_csp_op(Expr,X,Middle,Y,Op1,Op2),!,
  pp_csp_process(X,S,[Op1|S1]),
  pp_csp_value(Middle,S1,[Op2|S2]),
  pp_csp_process(Y,S2,T).
pp_csp_process(Expr,S,T) :- asharing_csp_op(Expr,X,MiddleX,MiddleY,Y,Op1,MOp,Op2),!,
  pp_csp_process(X,S,[Op1|S1]),
  pp_csp_value(MiddleX,S1,[MOp|S2]),
  pp_csp_value(MiddleY,S2,[Op2|S3]),
  pp_csp_process(Y,S3,T).
pp_csp_process(Expr,S,T) :- renaming_csp_op(Expr,X,RList,Op1,Op2),!,
  pp_csp_process(X,S,[Op1|S1]),
  pp_csp_value_l(RList,',',S1,[Op2|T],10).
pp_csp_process(prefix(_SPAN1,Values,ChannelExpr,CSP,_SPAN2),S,T) :- !,
  pp_csp_value_l([ChannelExpr|Values],'',S,['->'|S2],20),
  pp_csp_process(CSP,S2,T).
pp_csp_process('&'(Test,Then),S,T) :- !,
  pp_csp_bool_expr(Test,S,['&'|S2]),
  pp_csp_process(Then,S2,T).
pp_csp_process(ifte(Test,Then,Else,_SPAN1,_SPAN2,_SPAN3),[' if '|S],T) :- !,
  pp_csp_bool_expr(Test,S,[' then '|S2]),
  pp_csp_process(Then,S2,[' else '|S3]),
  pp_csp_process(Else,S3,T).
pp_csp_process(head(A),[head,'('|S],T) :- !, pp_csp_process(A,S,[')'|T]).
pp_csp_process(X,[X|T],T).

pp_csp_bool_expr(bool_not(BE),['not('|S],T) :- !, pp_csp_bool_expr(BE,S,[')'|T]).
pp_csp_bool_expr(BE,['('|S],T) :- binary_bool_op(BE,BE1,BE2,OP), !,
    pp_csp_bool_expr(BE1,S,[OP|S2]),
    pp_csp_bool_expr(BE2,S2,[')'|T]).
pp_csp_bool_expr(BE,[OP,'('|S],T) :- binary_pred(BE,BE1,BE2,OP), !,
    pp_csp_value(BE1,S,[','|S2]),
    pp_csp_value(BE2,S2,[')'|T]).
pp_csp_bool_expr(BE,S,T) :- pp_csp_value(BE,S,T).

bynary_numeric_operation('+'(X,Y),X,Y,'+').
bynary_numeric_operation('-'(X,Y),X,Y,'-').
bynary_numeric_operation('*'(X,Y),X,Y,'*').
bynary_numeric_operation('/'(X,Y),X,Y,'/').
bynary_numeric_operation('%'(X,Y),X,Y,'%').

binary_pred('member'(X,Y),X,Y,member).
binary_pred('<'(X,Y),X,Y,'<').
binary_pred('>'(X,Y),X,Y,'>').
binary_pred('>='(X,Y),X,Y,'>=').
binary_pred('<='(X,Y),X,Y,'=<').
binary_pred('elem'(X,Y),X,Y,is_elem_list).
binary_pred('=='(X,Y),X,Y,equal_element).
binary_pred('!='(X,Y),X,Y,not_equal_element).


binary_bool_op('<'(X,Y),X,Y,'<').
binary_bool_op('>'(X,Y),X,Y,'>').
binary_bool_op('>='(X,Y),X,Y,'>=').
binary_bool_op('<='(X,Y),X,Y,'=<').
binary_bool_op('=='(X,Y),X,Y,'==').
binary_bool_op('!='(X,Y),X,Y,'!=').
binary_bool_op(bool_and(X,Y),X,Y,'&&').
binary_bool_op(bool_or(X,Y),X,Y,'||').

binary_csp_op('|||'(X,Y,_Span),X,Y,'|||').
binary_csp_op('[]'(X,Y,_Span),X,Y,'[]').
binary_csp_op('|~|'(X,Y,_Span),X,Y,'|~|').
binary_csp_op(';'(X,Y,_Span),X,Y,';').
binary_csp_op('[>'(P,Q,_SrcSpan),P,Q,'[>').
binary_csp_op('/\\'(P,Q,_SrcSpan),P,Q,'/\\').

sharing_csp_op(esharing(CList,X,Y,_SrcSpan),X,CList,Y,' [|','|] ').
sharing_csp_op(sharing(CList,X,Y,_SrcSpan),X,CList,Y,' [|','|] ').
sharing_csp_op(lParallel(LinkList,X,Y,_Span),X,LinkList,Y,' [','] ').
sharing_csp_op(elinkParallel(LinkList,X,Y,_Span),X,LinkList,Y,' [','] ').
sharing_csp_op(exception(CList,X,Y,_SrcSpan),X,CList,Y,' [|','|> ').
sharing_csp_op(eexception(CList,X,Y,_SrcSpan),X,CList,Y,' [|','|> ').

asharing_csp_op(aParallel(CListX,X,CListY,Y,_SrcSpan),X,CListX,CListY,Y,' [',' || ','] ').
asharing_csp_op(eaParallel(CListX,X,CListY,Y,_SrcSpan),X,CListX,CListY,Y,' [',' || ','] ').

renaming_csp_op(procRenaming(RenameList,X,_SrcSpan),X,RenameList,'[[',']]').
renaming_csp_op(eprocRenaming(RenameList,X,_SrcSpan),X,RenameList,'[[',']]').

:- use_module(bmachine,[b_get_machine_operation_parameter_types/2, b_is_operation_name/1]).

translate_events([],[]).
translate_events([E|Erest],[Out|Orest]) :-
    translate_event(E,Out),
    translate_events(Erest,Orest).


% a version of translate_event which has access to the target state id:
% this allows to translate setup_constants, intialise by inserting target constants or values

translate_event_with_target_id(Term,Dst,Limit,Str) :-
   translate_event_with_src_and_target_id(Term,unknown,Dst,Limit,Str).
translate_event_with_src_and_target_id(Term,Src,Dst,Str) :-
   translate_event_with_src_and_target_id(Term,Src,Dst,5000,Str).

translate_event_with_src_and_target_id(Term,Src,Dst,Limit,Str) :-
    get_preference(expand_avl_upto,CurLim),
    SetLim is Limit/2,% at least two symbols per element
    (CurLim<0 ; SetLim < CurLim),!,
    temporary_set_preference(expand_avl_upto,SetLim,CHNG),
    translate_event_with_target_id2(Term,Src,Dst,Limit,Str),
    reset_temporary_preference(expand_avl_upto,CHNG).
translate_event_with_src_and_target_id(Term,Src,Dst,Limit,Str) :-
    translate_event_with_target_id2(Term,Src,Dst,Limit,Str).

translate_event_with_target_id2(Term,_,Dst,Limit,Str) :-
   functor(Term,'$setup_constants',_),
   get_preference(show_initialisation_arguments,true),
   state_space:visited_expression(Dst,concrete_constants(State)),
   get_non_det_constant(State,NonDetState),
   !,
   translate_b_state_to_comma_list_codes("SETUP_CONSTANTS",NonDetState,Limit,Codes),
   atom_codes_with_limit(Str,Limit,Codes).
translate_event_with_target_id2(Term,_,Dst,Limit,Str) :-
   functor(Term,'$initialise_machine',_),
   get_preference(show_initialisation_arguments,true),
   bmachine:b_get_operation_non_det_modifies('$initialise_machine',NDModVars),
   state_space:visited_expression(Dst,State), get_variables(State,VarsState),
   (NDModVars \= []
    ->
       include(non_det_modified_var(NDModVars),VarsState,ModVarsState) % first show non-det variables
       % we could add a preference for whether to show the deterministicly assigned variables at all
       %exclude(non_det_modified_var(NDModVars),VarsState,ModVarsState2),
       %append(ModVarsState1,ModVarsState2,ModVarsState)
    ; ModVarsState = VarsState),
   !,
   translate_b_state_to_comma_list_codes("INITIALISATION",ModVarsState,Limit,Codes),
   atom_codes_with_limit(Str,Limit,Codes).
translate_event_with_target_id2(Term,Src,Dst,Limit,Str) :-
   atomic(Term), % only applied to operations without parameters
   specfile:b_mode,
   get_non_det_modified_vars_in_target_id(Term,Dst,ModVarsState0), % only show non-det variables
   (Src \= unknown,
    state_space:visited_expression(Src,SrcState), get_variables(SrcState,PriorVarsState)
     -> exclude(var_not_really_modified(PriorVarsState),ModVarsState0,ModVarsState)
        % we could also optionally filter out vars which have the same value for all outgoing transitions of Src
      ; ModVarsState = ModVarsState0
    ),
   !,
   atom_codes(Term,TermCodes),
   translate_b_state_to_comma_list_codes(TermCodes,ModVarsState,Limit,Codes),
   atom_codes_with_limit(Str,Limit,Codes).
translate_event_with_target_id2(Term,_,_,Limit,Str) :- translate_event_with_limit(Term,Limit,Str).


get_non_det_modified_vars_in_target_id(OpName,DstId,ModVarsState0) :-
   bmachine:b_get_operation_non_det_modifies(OpName,NDModVars),
   NDModVars \= [], % a variable is non-deterministically written
   state_space:visited_expression(DstId,State), % TO DO: unpack only NModVars
   get_variables(State,VarsState),
   include(non_det_modified_var(NDModVars),VarsState,ModVarsState0).

:- use_module(library(ordsets)).
non_det_modified_var(NDModVars,bind(Var,_)) :- ord_member(Var,NDModVars).

var_not_really_modified(PriorState,bind(Var,Val)) :-
  (member(bind(Var,PVal),PriorState) -> PVal=Val).

get_variables(const_and_vars(_,VarsState),S) :- !, S=VarsState.
get_variables(S,S).

:- dynamic non_det_constants/2.

% compute which constants are non-deterministically assigned and which ones not
% TODO: maybe move to state space and invalidate info in case execute operation by predicate used
get_non_det_constant(Template,Result) :- non_det_constants(A,B),!, (A,B)=(Template,Result).
get_non_det_constant(Template,Result) :-
   state_space:transition(root,_,DstID),
   state_space:visited_expression(DstID,concrete_constants(State)), %print(get_non_det_constant(DstID)),nl,
   !,
   findall(D,(state_space:transition(root,_,D),D \= DstID),OtherDst),
   compute_non_det_constants2(OtherDst,State),
   non_det_constants(Template,Result).
get_non_det_constant(A,A).

compute_non_det_constants2([],State) :- adapt_state(State,Template,Result),
   (Result = [] -> assert(non_det_constants(A,A)) % in case all variables are deterministic: just show them
     ; assert(non_det_constants(Template,Result))).
compute_non_det_constants2([Dst|T],State) :-
   state_space:visited_expression(Dst,concrete_constants(State2)),
   lub_state(State,State2,NewState),
   compute_non_det_constants2(T,NewState).

lub_state([],[],[]).
lub_state([bind(V,H1)|T1],[bind(V,H2)|T2],[bind(V,H3)|T3]) :-
   (H1==H2 -> H3=H1 ; H3='$NONDET'), lub_state(T1,T2,T3).

adapt_state([],[],[]).
adapt_state([bind(ID,Val)|T],[bind(ID,X)|TX],[bind(ID,X)|TY]) :- Val='$NONDET',!,
   adapt_state(T,TX,TY).
adapt_state([bind(ID,_)|T],[bind(ID,_)|TX],TY) :- % Value is deterministic: do not copy
   adapt_state(T,TX,TY).



% ------------------------------------

translate_event_with_limit(Event,Limit,Out) :-
    translate_event2(Event,Atoms,[]),!,
    ajoin_with_limit(Atoms,Limit,Out).
    %,print(done),debug:print_debug_stats,nl.% , print(Out),nl.
translate_event_with_limit(Event,_,Out) :- add_error(translate_event_with_limit,'Could not translate event: ', Event),
    Out = '???'.

translate_event(Event,Out) :- %print(translate),print_debug_stats,nl,
    translate_event2(Event,Atoms,[]),!,
    ajoin(Atoms,Out).
    %,print(done),debug:print_debug_stats,nl.% , print(Out),nl.
translate_event(Event,Out) :-
    add_error(translate_event,'Could not translate event: ', Event),
    Out = '???'.
/* BEGIN CSP */
translate_event2(start_cspm(Process),['start_cspm('|S],T) :- process_algebra_mode,!,pp_csp_value(Process,S,[')'|T]).
%% translate_event2(i(_Span),['i'|T],T) :- process_algebra_mode,!.   /* CSP */ %% deprecated
translate_event2(tick(_Span),['tick'|T],T) :- process_algebra_mode,!.   /* CSP */
translate_event2(tau(hide(Action)),['tau(hide('|S],T) :- process_algebra_mode,nonvar(Action), !,
     translate_event2(Action,S,['))'|T]).   /* CSP */
translate_event2(tau(link(Action1,Action2)),['tau(link('|S],T) :- /* CSP */
     nonvar(Action1), nonvar(Action2), process_algebra_mode, !,
     translate_event2(Action1,S,['<->'|S1]),
     translate_event2(Action2,S1,['))'|T]).
translate_event2(tau(Info),['tau(',Fun,')'|T],T) :-
     nonvar(Info), process_algebra_mode,!,   /* CSP */
     functor(Info,Fun,_). %(translate_event(Info,Fun) -> true ; functor(Info,Fun,_)),
translate_event2(io(V,Ch,_Span),S,T) :- process_algebra_mode,!, /* CSP */
    (specfile:csp_with_bz_mode ->
        S=['CSP:'|S1],
        translate_event2(io(V,Ch),S1,T)
    ; otherwise ->
        translate_event2(io(V,Ch),S,T)
    ).
translate_event2(io(X,Channel),S,T) :- process_algebra_mode,!,   /* CSP */
  (X=[] -> translate_event2(Channel,S,T)
   ; (translate_event2(Channel,S,S1),
      translate_channel_values(X,S1,T))
   ).
/* END CSP */
translate_event2('$JUMP'(Name),[A|T],T) :- write_to_codes(Name,Codes),
  atom_codes_with_limit(A,Codes).
translate_event2('-->'(Operation,ResultValues),S,T) :- nonvar(ResultValues),
  ResultValues=[_|_],!,
  translate_event2(Operation,S,['-->',ValuesStr|T]),
  translate_bvalues(ResultValues,ValuesStr).
translate_event2(Op,[TOpName|S],T) :-
  nonvar(Op), Op =.. [OpName|Args],
  translate_operation_name(OpName,TOpName),
  ( Args=[] -> S=T
  ; otherwise ->
      S=['(',ValuesStr,')'|T],
      ((get_preference(show_eventb_any_arguments,false), % otherwise we have additional ANY parameters !
        \+ is_init(OpName), % order of variables does not always correspond to Variable order used by b_get_machine_operation_parameter_types ! TO DO - Fix this in b_initialise_machine2  (see Interlocking.mch CSP||B model)
        specfile:b_mode,
        b_is_operation_name(OpName),
        b_get_machine_operation_parameter_types(OpName,ParTypes),
        ParTypes \= [])
       -> translate_bvalues_with_types(Args,ParTypes,ValuesStr)
       %; is_init(OpName) -> b_get_machine_operation_typed_parameters(OpName,TypedParas),
       ;  translate_bvalues(Args,ValuesStr))
  ),!.
translate_event2(Op,[A|T],T) :-
   %['<< ',A,' >>'|T],T) :-   % the << >> pose problems when checking against FDR
  write_to_codes(Op,Codes),
  atom_codes_with_limit(A,Codes).

is_init('$initialise_machine').
is_init('$setup_constants').

translate_bvalues_with_types(Values,Types,Output) :-
    %set_up_limit_reached(Codes,1000,LimitReached),
    pp_value_l_with_types(Values,',',Types,_LimitReached,Codes,[]),!,
    atom_codes_with_limit(Output,Codes).
translate_bvalues_with_types(Values,T,Output) :-
    add_internal_error('Call failed: ',translate_bvalues_with_types(Values,T,Output)),
    translate_bvalues(Values,Output).

pp_value_l_with_types([],_Sep,[],_) --> !.
pp_value_l_with_types([Expr|Rest],Sep,[TE|TT],LimitReached) -->
    ( {nonvar(Rest),Rest=[]} ->
        pp_value_with_type(Expr,TE,LimitReached)
    ; {otherwise} ->
        pp_value_with_type(Expr,TE,LimitReached),ppatom(Sep),
        pp_value_l_with_types(Rest,Sep,TT,LimitReached)).


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% pretty-print properties
translate_properties_with_limit([],[]).
translate_properties_with_limit([P|Prest],[Out|Orest]) :-
    translate_property_with_limit(P,320,Out), % reduced limit as we now have evaluation view + possibility to inspect all of value
    translate_properties_with_limit(Prest,Orest).

translate_property_with_limit(Prop,Limit,Output) :-
    (pp_property(Prop,Limit,Output) -> true ; (add_error(translate_property,'Could not translate property: ',Prop),Output='???')).
pp_property(Prop,Limit,Output) :-
    pp_property_without_plugin(Prop,Limit,Output).
pp_property_without_plugin(=(Key,Value),_,A) :-
    !,ajoin([Key,' = ',Value],A).
pp_property_without_plugin(':'(Key,Value),_,A) :-
    !,ajoin([Key,' : ',Value],A).
pp_property_without_plugin(info(I),_,I) :- !.
pp_property_without_plugin(Prop,Limit,A) :-
    write_to_codes(Prop,Codes),
    atom_codes_with_limit(A,Limit,Codes).


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
:- use_module(tools_meta,[translate_term_into_atom_with_max_depth/3]).

% pretty-print errors belonging to a certain state
translate_error_term(Var,S) :- var(Var),!,
   translate_term_into_atom_with_max_depth(Var,5,S).
translate_error_term('@fun'(X,F),S) :-
   translate:translate_bvalue(X,TX), translate:translate_bvalue(F,TF),!,
   % translate_term_into_atom_with_max_depth('@fun'(TX,TF),5,S).
   (get_error_span_for_value(F,Span)
     ->  ajoin(['Function argument: ',TX, ', function value: ',TF,' defined at ',Span],S)
      ;  ajoin(['Function argument: ',TX, ', function value: ',TF],S)
   ).
translate_error_term('@rel'(Arg,Res1,Res2),S) :-
   translate:translate_bvalue(Arg,TA), translate:translate_bvalue(Res1,R1),
   translate:translate_bvalue(Res2,R2),!,
   ajoin(['Function argument: ',TA, ', two possible values: ',R1,', ',R2],S).
translate_error_term([operation(Name,Env)],S) :-
    translate:translate_any_state(Env,TEnv), !,
    translate_term_into_atom_with_max_depth(operation(Name,TEnv),10,S).
translate_error_term(error(E1,E2),S) :- !, translate_prolog_error(E1,E2,S).
translate_error_term(b(B,T,I),S) :-
     translate_subst_or_bexpr_with_limit(b(B,T,I),1000,do_not_report_errors,S),!. % do not report errors, otherwise we end in an infinite loop of adding errors while adding errors
translate_error_term(T,S) :- translate_term_into_atom_with_max_depth(T,5,S).

% try and get error location for span:
get_error_span_for_value(Var,_) :- var(Var),!,fail.
get_error_span_for_value(closure(_,_,Body),Span) :- translate_span_with_filename(Body,Span), Span \= ''.

translate_prolog_error(existence_error(procedure,Pred),_,S) :- !,
    translate_term_into_atom_with_max_depth('Unknown Prolog predicate:'(Pred),8,S).
translate_prolog_error(existence_error(source_sink,File),_,S) :- !,
    translate_term_into_atom_with_max_depth('Could not access file:'(File),8,S).
translate_prolog_error(E1,_,S) :- translate_term_into_atom_with_max_depth(E1,8,S).

translate_state_errors([],[]).
translate_state_errors([E|ERest],[Out|ORest]) :-
    ( E = eventerror(Event,EError,_) ->
        translate_event_error(EError,Msg),
        ajoin([Event,': ',Msg],Out)
    ; translate_state_error(E,Out) -> true
    ; otherwise -> functor(E,Out,_) ),
    translate_state_errors(ERest,ORest).

translate_error_context(E,TE) :- translate_error_context2(E,Codes,[]),
    atom_codes_with_limit(TE,Codes).
translate_error_context2(span_context(Span,Context)) --> !,
   translate_error_context2(Context),
   translate_span(Span,only_subsidiary).
translate_error_context2([H]) --> !,translate_error_context2(H).
translate_error_context2(checking_invariant) --> !,
    {get_specification_description_codes(invariant,A)}, A. %"INVARIANT".
translate_error_context2(checking_assertions) --> !,
    {get_specification_description_codes(assertions,A)}, A. %"ASSERTIONS".
translate_error_context2(checking_negation_of_invariant(_State)) --> !,
    "not(INVARIANT)".
translate_error_context2(operation(OpName,_State)) --> !,
    {translate_operation_name(OpName,TOp)},
    ppterm(TOp).
translate_error_context2(checking_context(Check,Name)) --> !,
    ppterm(Check),ppterm(Name).
translate_error_context2(X) --> "???:", ppterm(X).

print_span(Span) :- translate_span(Span,Atom), !, print(Atom).
print_span(S) :- print(span(S)).

translate_span(Span,Atom) :- translate_span(Span,only_subsidiary,Codes,[]),
    atom_codes_with_limit(Atom,Codes).
translate_span_with_filename(Span,Atom) :-
    translate_span(Span,always_print_filename,Codes,[]),
    atom_codes_with_limit(Atom,Codes).

translate_span(Span,PrintFileNames) --> {extract_line_col(Span,Srow,Scol,_Erow,_Ecol)},!,
   "(Line:",ppterm(Srow)," Col:",ppterm(Scol),
     %"-",ppterm(Erow),":",ppterm(Ecol),
     translate_span_file_opt(Span,PrintFileNames),
     % TO DO: print short version of extract_additional_description ?
    ")".
translate_span(Span,PrintFileNames) -->
    % for Event-B, e.g., line-col fails but we can get a section/file name
    "(File:",translate_span_file(Span,PrintFileNames),!,")".
translate_span(_,_PrintFileNames) --> "".

translate_span_file(Span,always_print_filename) -->
     {extract_tail_file_name(Span,Filename)},!,
     %{bmachine:b_get_main_filenumber(MainFN), Nr \= MainFN},!,
     ":", ppterm(Filename).
translate_span_file(Span,_) -->
     {extract_subsidiary_tail_file_name(Span,Filename)},
     %{bmachine:b_get_main_filenumber(MainFN), Nr \= MainFN},!,
     !,
     ":", ppterm(Filename).
translate_span_file_opt(Span,Print) --> translate_span_file(Span,Print).
translate_span_file_opt(_,_) --> "".


explain_span_file(Span) -->
     {extract_subsidiary_tail_file_name(Span,Filename)},
     %{bmachine:b_get_main_filenumber(MainFN), Nr \= MainFN},!,
     "\n### File: ", ppterm(Filename).
explain_span_file(_) --> "".

explain_span(span_predicate(Pred,LS,S)) --> !, explain_span2(span_predicate(Pred,LS,S)),
    explain_local_state(LS). %, explain_global_state(S).
explain_span(Span) --> explain_span2(Span).
explain_span2(Span) --> {extract_line_col(Span,Srow,Scol,Erow,Ecol)},!,
   "\n### Line: ",ppterm(Srow),", Column: ",ppterm(Scol),
   " until Line: ",ppterm(Erow),", Column: ",ppterm(Ecol),
   explain_span_file(Span),
   explain_span_context(Span).
explain_span2(_) --> "".

explain_span_context(Span) --> {extract_additional_description(Span,Msg),!},
  "\n### within ", ppterm(Msg). % context of span, such as definition call hierarchy
explain_span_context(_) --> "".

explain_local_state([]) --> !, "".
explain_local_state(LS) --> "\n  Local State: ", pp_b_state(LS).
%explain_global_state([]) --> !, "".
%explain_global_state(LS) --> "\n  Global State: ", pp_b_state(LS).

translate_event_error(Error,Out) :-
    ( translate_event_error2(Error,Out) -> true
    ; otherwise ->
        functor(Error,F,_),
        ajoin(['** Unable to translate event error: ',F,' **'],Out)).
translate_event_error2(no_witness_found(Type,Var,_Predicate),Out) :-
    def_get_texpr_id(Var,Id),
    ajoin(['no witness was found for ',Type,' ',Id],Out).
translate_event_error2(simulation_error(_Events),Out) :-
    Out = 'no matching abstract event was found'.
translate_event_error2(action_not_executable(_Action),Out) :-
    Out = 'action was not executable'.
translate_event_error2(invalid_modification(Var,_Pre,_Post),Out) :-
    def_get_texpr_id(Var,Id),
    ajoin(['modification of variable ', Id, ' not allowed'],Out).
translate_event_error2(variant_negative(_CType,_Variant,_Value),Out) :-
    Out = 'enabled for negative variant'.
translate_event_error2(invalid_variant(anticipated,_Expr,_Pre,_Post),Out) :-
    Out = 'variant increased'.
translate_event_error2(invalid_variant(convergent,_Expr,_Pre,_Post),Out) :-
    Out = 'variant not decreased'.
translate_event_error2(invalid_theorem_in_guard(_Theorem),Out) :-
    Out = 'theorem in guard evaluates to false'.
translate_event_error2(event_wd_error(_TExpr,Source),Out) :-
    ajoin(['WD error for ',Source],Out).
translate_event_error2(event_other_error(Msg),Out) :- Out=Msg.

translate_state_error(abort_error(_TYPE,Msg,ErrTerm,ErrorContext),Out) :- !,
    translate_error_term(ErrTerm,ES),
    translate_error_context(ErrorContext,EC),
    ajoin([EC,': ',Msg,': ',ES],Out).
translate_state_error('CLPFD_integer_overflow',Out) :- !,
    Out = 'CLPFD integer overflow'.
translate_state_error(max_state_errors_reached(Nr),Out) :- !,
     ajoin(['Max. number of state errors reached: ', Nr],Out).
translate_state_error(Unknown,Out) :-
    add_error(translate_state_error,'Unknown state error: ',Unknown),
    Out = '*** Unknown State Error ***'.


explain_error_context1([H],Span) --> !,explain_error_context1(H,Span).
explain_error_context1(span_context(Span,Context),Span) --> !,
   explain_span(Span),"\n",
   explain_error_context2(Context).
explain_error_context1(Ctxt,unknown) --> explain_error_context2(Ctxt).

explain_error_context2([H]) --> !,explain_error_context2(H).
explain_error_context2(span_context(Span,Context)) --> !,
   explain_span(Span),"\n",
   explain_error_context2(Context).
explain_error_context2(checking_invariant) --> !,
    {get_specification_description_codes(invariant,I)}, I, ":\n ", %"INVARIANT:\n ",
    pp_current_state. % assumes explain is called in the right state ! ; otherwise we need to store the state id
explain_error_context2(checking_assertions) --> !,
    {get_specification_description_codes(assertions,A)}, A, ":\n ", %"ASSERTIONS:\n ",
    pp_current_state. % assumes explain is called in the right state ! ; otherwise we need to store the state id
explain_error_context2(checking_negation_of_invariant(State)) --> !,
    "not(INVARIANT):\n  State: ",
    pp_b_state(State).
explain_error_context2(operation('$setup_constants',StateID)) --> !,
    {get_specification_description_codes(properties,P)}, P, ":\n  State: ",
    pp_context_state(StateID).
explain_error_context2(operation(OpName,StateID)) --> !,
    {get_specification_description_codes(operation,OP)}, OP, ": ",  %"EVENT/OPERATION: ",
    {translate_operation_name(OpName,TOp)},
    ppterm(TOp), "\n ",
    pp_context_state(StateID).
explain_error_context2(checking_context(Check,Name)) --> !,
    ppterm(Check),ppterm(Name), "\n ".
explain_error_context2(X) --> "UNKNOWN ERROR CONTEXT:\n  ", ppterm(X).

:- use_module(specfile,[get_specification_description/2]).
get_specification_description_codes(Tag,Codes) :- get_specification_description(Tag,Atom), atom_codes(Atom,Codes).

explain_state_error(Error,Span,Out) :-
    explain_state_error2(Error,Span,Out,[]),!.
explain_state_error(_Error,unknown,"Sorry, the detailed output failed.\n").

explain_abort_error_type(well_definedness_error) --> !, "An expression was not well-defined.\n".
explain_abort_error_type(while_variant_error) --> !, "A while-loop VARIANT error occurred.\n".
explain_abort_error_type(while_invariant_violation) --> !, "A while-loop INVARIANT error occurred.\n".
explain_abort_error_type(precondition_error) --> !, "A precondition (PRE) error occurred.\n".
explain_abort_error_type(assert_error) --> !, "An ASSERT error occurred.\n".
explain_abort_error_type(Type) --> "Error occurred: ", ppterm(Type), "\n".

explain_state_error2(abort_error(TYPE,Msg,ErrTerm,ErrContext),Span) -->
    explain_abort_error_type(TYPE),
    "Reason:  ", ppterm(Msg), "\n",
    "Details: ", {translate_error_term(ErrTerm,ErrS)},ppterm(ErrS), "\n",
    "Context: ", explain_error_context1(ErrContext,Span).
explain_state_error2(max_state_errors_reached(Nr),unknown) -->
    "Too many error occurred for this state.\n",
    "Not all errors are shown.\n",
    "Number of errors is at least: ",   ppterm(Nr).
explain_state_error2(eventerror(_Event,Error,Trace),unknown) -->  % TO DO: also extract loc info ?
    {translate_event_error(Error,Msg)},
    ppatom(Msg),
    "\nA detailed trace containing the error:\n",
    "--------------------------------------\n",
    explain_event_trace(Trace).

show_parameter_values([],[]) --> !.
show_parameter_values([P|Prest],[V|Vrest]) -->
    show_parameter_value(P,V),
    show_parameter_values(Prest,Vrest).
show_parameter_value(P,V) -->
    "    ",pp_expr(P,_,_LR)," = ",pp_value(V),"\n".

explain_event_trace(Trace,Codes) :-
    explain_event_trace(Trace,Codes,[]).

explain_event_trace([]) --> !.
explain_event_trace([Step|Trest]) -->
    "\n",
    ( explain_event_step(Step) -> ""
    ; {functor(Step,F,_)} ->
        "  (no rule to explain event step ",ppatom(F),")\n"),
    explain_event_trace(Trest).
explain_event_step(event(Name,Section)) --> !,
    "Event ",ppterm(Name)," in model ",ppterm(Section),
    ":\n".
explain_event_step(true_guard(Parameters,Values,Guard)) --> !,
    ( {Parameters==[]} -> ""
    ; "  for the parameters:\n",
        show_parameter_values(Parameters,Values)),
    "  the guard is true:",
    explain_predicate(Guard,4),"\n".
explain_event_step(variant_checked_pre(CType,Variant,Value)) -->
    "  ",ppatom(CType)," event: checking if the variant if non-negative:\n",
    "    variant: ",pp_expr(Variant,_,_LR),"\n",
    "    its value: ",pp_value(Value),"\n".
explain_event_step(variant_negative(CType,Variant,Value)) -->
    explain_event_step(variant_checked_pre(CType,Variant,Value)),
    "  ERROR: variant is negative\n".
explain_event_step(variant_checked_post(CType,Variant,EntryValue,ExitValue)) -->
    "  ",ppatom(CType)," event: checking if the variant is ",
    ( {CType==convergent} -> "decreased:\n" ; "not increased:\n"),
    "    variant: ", pp_expr(Variant,_,_LR), "\n",
    "    its value before: ", pp_value(EntryValue),"\n",
    "    its value after:  ", pp_value(ExitValue),"\n".
explain_event_step(invalid_variant(CType,Variant,EntryValue,ExitValue)) -->
    explain_event_step(variant_checked_post(CType,Variant,EntryValue,ExitValue)),
    "  ERROR: variant has ",
    ({CType==convergent} -> "not been decreased\n"; "has been increased\n").
explain_event_step(eval_witness(Type,Id,Value,Predicate)) -->
    witness_intro(Id,Predicate,Type),
    "    found witness:\n",
    "    ", pp_expr(Id,_,_LR), " = ", pp_value(Value), "\n".
explain_event_step(no_witness_found(Type,Id,Predicate)) -->
    witness_intro(Id,Predicate,Type),
    "  ERROR: no solution for witness predicate found!\n".
explain_event_step(action(Lhs,_Rhs,Values)) -->
    "  executing an action:\n",
    show_assignments(Lhs,Values).
explain_event_step(action_set(Lhs,_Rhs,ValueSet,Values)) -->
    "  executing an action:\n    ",
    pp_expr_l(Lhs,_LR)," :: ",pp_value(ValueSet),"\n    choosing\n",
    show_assignments(Lhs,Values).
explain_event_step(action_pred(Ids,Pred,Values)) -->
    "  executing an action:\n    ",
    pp_expr_l(Ids,_LR1)," :| ",pp_expr(Pred,_,_LR2),"\n    choosing\n",
    show_assignments(Ids,Values).
explain_event_step(action_not_executable(TAction)) -->
    explain_action_not_executable(TAction).
explain_event_step(error(Error,_Id)) -->
    % the error marker serves to link to a stored state-error by its ID
    explain_event_step(Error).
explain_event_step(simulation_error(Errors)) -->
    "  no guard of a refined event was satisfiable:\n",
    explain_simulation_errors(Errors).
explain_event_step(invalid_theorem_in_guard(Theorem)) -->
    "  the following theorem evaluates to false:",
    explain_predicate(Theorem,4),"\n".
explain_event_step(invalid_modification(Var,Pre,Post)) -->
    "  the variable ", pp_expr(Var,_,_LR), " has been modified.\n",
    "    The event is not allowed to modify the variable because its abstract event does not modify it.\n",
    "    Old value: ", pp_value(Pre), "\n",
    "    New value: ", pp_value(Post), "\n".
explain_event_step(event_wd_error(TExpr,Source)) -->
    "    Well-Definedness ERROR for ", ppatom(Source), "\n",
    "    ", pp_expr(TExpr,_,_LR), "\n".
explain_event_step(event_other_error(Msg)) --> ppatom(Msg).

explain_action_not_executable(TAction) --> {is_assignment_to(TAction,IDs)},!,
    "  ERROR: the following assignment to ", ppatoms(IDs),"was not executable\n",
    "         (probably in conflict with another assignment):",
    translate_subst_with_indention(TAction,4).
explain_action_not_executable(TAction) -->
    "  ERROR: the following action was not executable:",
    translate_subst_with_indention(TAction,4).

is_assignment_to(b(assign(LHS,_),_,_),IDs) :- get_texpr_ids(LHS,IDs).

witness_intro(Id,Predicate,Type) -->
    "  evaluating witness for abstract ", ppatom(Type), " ", pp_expr(Id,_,_LR1), "\n",
    "    witness predicate: ", pp_expr(Predicate,_,_LR2), "\n".

show_assignments([],[]) --> !.
show_assignments([Lhs|Lrest],[Val|Vrest]) -->
    "    ",pp_expr(Lhs,_,_LimitReached), " := ", pp_value(Val), "\n",
    show_assignments(Lrest,Vrest).

/* unused at the moment:
explain_state([]) --> !.
explain_state([bind(Varname,Value)|Rest]) --> !,
    " ",ppterm(Varname)," = ",pp_value(Value),"\n",
    explain_state(Rest).
explain_guards([]) --> "".
explain_guards([Event|Rest]) -->
    {get_texpr_expr(Event,rlevent(Name,_Section,_Status,_Params,Guard,_Theorems,_Act,_VWit,_PWit,_Unmod,_Evt))},
    "\n",ppatom(Name),":",
    explain_predicate(Guard),
    explain_guards(Rest).
explain_predicate(Guard,I,O) :-
    explain_predicate(Guard,2,I,O).
*/
explain_predicate(Guard,Indention,I,O) :-
    pred_over_lines(0,'@grd',Guard,(Indention,I),(_,O)).

explain_simulation_errors([]) --> !.
explain_simulation_errors([Error|Rest]) -->
    explain_simulation_error(Error),
    explain_simulation_errors(Rest).
explain_simulation_error(event(Name,Section,Guard)) -->
    "    guard for event ", ppatom(Name),
    " in ", ppatom(Section), ":",
    explain_predicate(Guard,6),"\n".

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% pretty-print a state


print_state(State) :- b_state(State), !,print_bstate(State).
print_state(csp_and_b_root) :- csp_with_bz_mode, !,
   print('(MAIN || B)').
print_state(csp_and_b(CSPState,BState)) :- csp_with_bz_mode, !,
   print_bstate(BState), translate_cspm_state(CSPState,Text), print(Text).
print_state(CSPState) :- csp_mode,!,translate_cspm_state(CSPState,Text), print(Text).
print_state(State) :- animation_mode(xtl),!,print(State).
print_state(State) :- print('*** Unknown state: '),print(State).

b_state(root).
b_state(concrete_constants(_)).
b_state(const_and_vars(_,_)).
b_state(expanded_const_and_vars(_,_,_)).
b_state([bind(_,_)|_]).
b_state([]).

print_bstate(State) :- translate_bstate(State,Output), print('     '),print(Output).

translate_any_state(State,Output) :-
    pp_any_state(State,Codes,[]),
    atom_codes_with_limit(Output,Codes).
translate_bstate(State,Output) :-
    pp_b_state(State,Codes,[]),
    atom_codes_with_limit(Output,Codes).

% a version which tries to generate smaller strings
translate_bstate_limited(State,Output) :-
    temporary_set_preference(expand_avl_upto,2,CHNG),
    call_cleanup((pp_b_state(State,Codes,[]),
                   atom_codes_with_limit(Output,200,Codes)),
        reset_temporary_preference(expand_avl_upto,CHNG)).


pp_b_state(X) --> try_pp_b_state(X),!.
pp_b_state(X) --> {add_error(pp_b_state,'Could not translate state: ',X)}.

try_pp_b_state(VAR) --> {var(VAR)},!, "_?VAR?_", {add_error(pp_b_state,'Variable state: ',VAR)}.
try_pp_b_state(root) --> !, "root".
try_pp_b_state(concrete_constants(Constants)) --> !,"Constants: ",
   pp_b_state(Constants).
try_pp_b_state(const_and_vars(ID,Vars)) --> !,
   "Constants:",ppterm(ID),", Vars:",
   {set_translation_constants(ID)}, /* extract constants which stand for deferred set elements */
   pp_b_state(Vars),
   {clear_translation_constants}.
try_pp_b_state(expanded_const_and_vars(ID,Vars,_)) --> !, "EXPANDED ",
    try_pp_b_state(const_and_vars(ID,Vars)).
try_pp_b_state([]) --> !, "/* empty state */".
try_pp_b_state([bind(Varname,Value)|Rest]) --> !,
    "( ",ppterm(Varname),"=",
    pp_value(Value),
    ({Rest = []} -> []; " ",and_symbol,"\n       "),
    pp_b_state_list(Rest).


pp_b_state_list([]) --> !, " )".
pp_b_state_list([bind(Varname,Value)|Rest]) --> !,
    ppterm(Varname),"=",
    pp_value(Value),
    ({Rest = []} -> [] ; " ",and_symbol,"\n       "),
    pp_b_state_list(Rest).
pp_b_state_list(X) --> {add_error(pp_b_state_list,'Could not translate: ',X)}.

% a version of pp which generates no newline; can be used for printing SETUP_CONSTANTS, INITIALISATION
pp_b_state_comma_list([],_,_) --> !, ")".
pp_b_state_comma_list(_,Cur,Limit) --> {Cur >= Limit}, !, "...".
pp_b_state_comma_list([bind(Varname,Value)|Rest],Cur,Limit) --> !,
    %{print(c(Varname,Cur,Limit)),nl},
    start_size(Ref),
    ppterm(Varname),"=",
    pp_value(Value),
    ({Rest = []} -> [] ; ","),
    end_size(Ref,Size), % compute size increase wrt Ref point
    {Cur1 is Cur+Size},
    pp_b_state_comma_list(Rest,Cur1,Limit).
pp_b_state_comma_list(X,_,_) --> {add_error(pp_b_state_comma_list,'Could not translate: ',X)}.

start_size(X,X,X).
end_size(RefVar,Len,X,X) :- % compute how many chars the dcg has added wrt start_size
   len(RefVar,X,Len).
len(Var,X,Len) :- (var(Var) ; Var==X),!, Len=0.
len([],_,0).
len([_|T],X,Len) :- len(T,X,L1), Len is L1+1.

% can be used e.g. for setup_constants, initialise
translate_b_state_to_comma_list_codes(FUNCTORCODES,State,Limit,ResCodes) :-
     pp_b_state_comma_list(State,0,Limit,Codes,[]),
     append("(",Codes,C0),
     append(FUNCTORCODES,C0,ResCodes).
% ----------------

% printing and translating error contexts
print_context(State) :- translate_context(State,Output), print(Output).

translate_context(Context,Output) :-
    pp_b_context(Context,Codes,[]),
    atom_codes_with_limit(Output,250,Codes).

pp_b_context([]) --> !.
pp_b_context([C|Rest]) --> !,
    pp_b_context(C),
    pp_b_context(Rest).
pp_b_context(translate_context) --> !, " ERROR CONTEXT: translate_context". % error occurred within translate_context
pp_b_context(operation(Name,StateID)) --> !,
    " ERROR CONTEXT: ",
    {get_specification_description_codes(operation,OP)}, OP, ":", % "OPERATION:"
    ({var(Name)} -> ppterm('ALL') ; {translate_operation_name(Name,TName)},ppterm(TName)),
    ",",pp_context_state(StateID).
pp_b_context(checking_invariant) --> !,
    " ERROR CONTEXT: INVARIANT CHECKING,", pp_cur_context_state.
pp_b_context(checking_negation_of_invariant(State)) --> !,
    " ERROR CONTEXT: NEGATION_OF_INVARIANT CHECKING, State:", pp_b_state(State).
pp_b_context(checking_assertions) --> !,
    " ERROR CONTEXT: ASSERTION CHECKING,", pp_cur_context_state.
pp_b_context(checking_context(Check,Name)) --> !,
    ppterm(Check),ppterm(Name).
pp_b_context(unit_test_context(Module,TotNr,Call)) --> !,
   " ERROR CONTEXT: Unit Test ", ppterm(TotNr), " in module ", ppterm(Module), " calling ", pp_functor(Call).
pp_b_context(C) --> ppterm(C),pp_cur_context_state.

pp_functor(V) --> {var(V)},!, ppterm(V).
pp_functor(T) --> {functor(T,F,N)}, ppterm(F),"/",ppterm(N).

pp_cur_context_state --> {state_space:get_current_context_state(ID)}, !,pp_context_state(ID).
pp_cur_context_state --> ", unknown context state.".

% assumes we are in the right state:
pp_current_state --> {state_space:current_expression(ID,_)}, !,pp_context_state(ID).
pp_current_state --> ", unknown current context state.".

% TO DO: limit length/size of generated error description
pp_context_state(ID) --> {state_space:visited_expression(ID,State)},!, % we have a state ID
                          " State ID:", ppterm(ID), ",", pp_context_state2(State).
pp_context_state(State) --> pp_context_state3(State).

pp_context_state2(_) --> {debug:debug_mode(off)},!.
pp_context_state2(State) --> pp_context_state3(State).

pp_context_state3(State) --> " State: ",pp_any_state_with_limit(State,10).

pp_any_state_with_limit(State,Limit) -->
    {get_preference(expand_avl_upto,CurLim),
     (CurLim<0 ; Limit < CurLim),
     !,
     temporary_set_preference(expand_avl_upto,Limit,CHNG)},
    pp_any_state(State),
    {reset_temporary_preference(expand_avl_upto,CHNG)}.
pp_any_state_with_limit(State,_Limit) --> pp_any_state(State).

pp_any_state(X) --> try_pp_b_state(X),!.
pp_any_state(csp_and_b(P,B)) --> "CSP: ",{pp_csp_process(P,Atoms,[])},!,atoms_to_codelist(Atoms),
  " || B: ", try_pp_b_state(B).
pp_any_state(X) -->   {animation_mode(xtl)}, !, "XTL: ",ppterm(X). % CSP state
pp_any_state(P) -->  "CSP: ",{pp_csp_process(P,Atoms,[])},!,atoms_to_codelist(Atoms).
pp_any_state(X) -->   "Other formalism: ",ppterm(X). % CSP state

atoms_to_codelist([]) --> [].
atoms_to_codelist([Atom|T]) --> ppterm(Atom), atoms_to_codelist(T).

% ----------------

:- dynamic deferred_set_constant/3.

set_translation_context(const_and_vars(ConstID,_)) :- !,
 %% print_message(setting_translation_constants(ConstID)),
  set_translation_constants(ConstID).
set_translation_context(expanded_const_and_vars(ConstID,_,_)) :- !,
  set_translation_constants(ConstID).
set_translation_context(_).

set_translation_constants(_) :- clear_translation_constants,
    get_preference(dot_print_use_constants,false),!.
set_translation_constants(ConstID) :- var(ConstID),!,
   add_error(set_translation_constants,'Variable used as ConstID: ',ConstID).
set_translation_constants(ConstID) :-
    state_space:visited_expression(ConstID,concrete_constants(ConstantsStore)),!,
   %% print_message(setting_constants(ConstID)),%%
    (treat_constants(ConstantsStore) -> true ; print_message(fail)).
set_translation_constants(ConstID) :-
   add_error(set_translation_constants,'Unknown ConstID: ',ConstID).

clear_translation_constants :- %print_message(clearing),%%
    retractall(deferred_set_constant(_,_,_)).

treat_constants([]).
treat_constants([bind(CstName,Val)|T]) :-
   ((Val=fd(X,GSet),b_global_deferred_set(GSet))
     -> (deferred_set_constant(GSet,X,_)
          -> true /* duplicate def of value */
           ; assert(deferred_set_constant(GSet,X,CstName))
        )
      ; true
   ),
   treat_constants(T).



translate_bvalue_with_tlatype(Value,Type,Output) :-
    ( pp_tla_value(Type,Value,Codes,[]) ->
        atom_codes_with_limit(Output,Codes)
    ; add_error(translate_bvalue,'Could not translate TLA value: ',Value),
        Output='???').

pp_tla_value(function(_Type1,_Type2),[]) --> !,
    ppcodes("<<>>").
pp_tla_value(function(integer,T2),avl_set(Set)) -->
    {convert_avlset_into_sequence(Set,Seq)}, !,
    pp_tla_with_sep("<< "," >>",",",T2,Seq).
pp_tla_value(function(T1,T2),Set) -->
    {is_printable_set(Set,Values)},!,
    pp_tla_with_sep("(",")"," @@ ",function_value(T1,T2),Values).
pp_tla_value(function_value(T1,T2),(L,R)) -->
    !,pp_tla_value(T1,L),":>",pp_tla_value(T2,R).
pp_tla_value(set(Type),Set) -->
    {is_printable_set(Set,Values)},!,
    pp_tla_with_sep("{","}",",",Type,Values).
pp_tla_value(tuple(Types),Value) -->
    {pairs_to_list(Types,Value,Values,[]),!},
    pp_tla_with_sep("<< "," >>",",",Types,Values).
pp_tla_value(record(Fields),rec(FieldValues)) -->
    % TODO: Check if we can safely assume that Fields and FieldValues have the
    % same order
    !, {sort_tla_fields(Fields,FieldValues,RFields,RFieldValues)},
    pp_tla_with_sep("[","]",", ",RFields,RFieldValues).
pp_tla_value(field(Name,Type),field(_,Value)) -->
    !, ppatom_opt_scramble(Name)," |-> ",pp_tla_value(Type,Value).
pp_tla_value(_Type,Value) -->
    % fallback: use B's pretty printer
    pp_value(Value).

is_printable_set(avl_set(A),List) :- avl_domain(A,List).
is_printable_set([],[]).
is_printable_set([H|T],[H|T]).

pairs_to_list([_],Value) --> !,[Value].
pairs_to_list([_|Rest],(L,R)) -->
    pairs_to_list(Rest,L),[R].


sort_tla_fields([],_,[],[]).
sort_tla_fields([Field|RFields],ValueFields,RFieldTypes,ResultValueFields) :-
    ( Field=field(Name,Type) -> true
    ; Field=  opt(Name,Type) -> true),
    ( selectchk(field(Name,Value),ValueFields,RestValueFields),
        field_value_present(Field,Value,Result) ->
        % Found the field in the record value
        RFieldTypes       = [field(Name,Type)  |RestFields],
        ResultValueFields = [field(Name,Result)|RestValues],
        sort_tla_fields(RFields,RestValueFields,RestFields,RestValues)
    ; otherwise ->
        % didn't found the field in the value -> igore
        sort_tla_fields(RFields,ValueFields,RFieldTypes,ResultValueFields)
    ).
field_value_present(field(_,_),RecValue,RecValue). % Obligatory fields are always present
field_value_present(opt(_,_),OptValue,Value) :-
    % Optional fields are present if the field is of the form TRUE |-> Value.
    ( is_printable_set(OptValue,Values) -> Values=[(_TRUE,Value)]
    ; otherwise ->
        add_error(translate,'exptected set for TLA optional record field'),
        fail
    ).

pp_tla_with_sep(Start,End,Sep,Type,Values) -->
    ppcodes(Start),pp_tla_with_sep_aux(Values,End,Sep,Type).
pp_tla_with_sep_aux([],End,_Sep,_Type) -->
    ppcodes(End).
pp_tla_with_sep_aux([Value|Rest],End,Sep,Type) -->
    % If a single type is given, we interpret it as the type
    % for each element of the list, if it is a list, we interpret
    % it one different type for every value in the list.
    { ( Type=[CurrentType|RestTypes] -> true
      ; otherwise -> CurrentType = Type, RestTypes=Type) },
    pp_tla_value(CurrentType,Value),
    ( {Rest=[_|_]} -> ppcodes(Sep); {true} ),
    pp_tla_with_sep_aux(Rest,End,Sep,RestTypes).


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% pretty-print a value
translate_bvalue_to_codes(V,Output) :-
   ( pp_value(V,_LimitReached,Codes,[]) ->
       Output=Codes
    ; add_error(translate_bvalue_to_codes,'Could not translate bvalue: ',V),
        Output="???").
translate_bvalue(V,Output) :-
    %set_up_limit_reached(Codes,1000000,LimitReached), % we could set a very high-limit, like max_atom_length
    ( pp_value(V,_LimitReached,Codes,[]) ->
        atom_codes_with_limit(Output,Codes) % just catches representation error
    ; add_error(translate_bvalue,'Could not translate bvalue: ',V),
        Output='???').
:- use_module(preferences).
translate_bvalue_with_limit(V,Limit,Output) :-
    get_preference(expand_avl_upto,Max),
    ((Max>Limit % no sense in printing larger AVL trees
     ; Max = -1) % or setting limit to -1 for full value
      -> set_preference(expand_avl_upto,Limit)
        ; true),
    call_cleanup(translate_bvalue_with_limit_aux(V,Limit,Output),
                 set_preference(expand_avl_upto,Max)).
translate_bvalue_with_limit_aux(V,Limit,Output) :-
    set_up_limit_reached(Codes,Limit,LimitReached),
    ( pp_value(V,LimitReached,Codes,[]) ->
        atom_codes_with_limit(Output,Limit,Codes)
        % ,length(Codes,Len), (Len>Limit -> format('pp(~w) codes:~w, limit:~w, String=~s~n~n',[LimitReached,Len,Limit,Codes]) ; true)
    ; add_error(translate_bvalue_with_limit,'Could not translate bvalue: ',V),
        Output='???').

translate_bvalues(Values,Output) :-
    %set_up_limit_reached(Codes,1000000,LimitReached), % we could set a very high-limit, like max_atom_length
    pp_value_l(Values,',',_LimitReached,Codes,[]),!,
    atom_codes_with_limit(Output,Codes).
translate_bvalues(Values,O) :-
    add_internal_error('Call failed: ',translate_bvalues(Values,O)), O='??'.

translate_bvalue_for_expression(Value,TExpr,Output) :-
    animation_minor_mode(tla),
    expression_has_tla_type(TExpr,TlaType),!,
    translate_bvalue_with_tlatype(Value,TlaType,Output).
translate_bvalue_for_expression(Value,TExpr,Output) :-
    get_texpr_type(TExpr,Type),
    translate_bvalue_with_type(Value,Type,Output).

translate_bvalue_for_expression_with_limit(Value,TExpr,_Limit,Output) :-
    animation_minor_mode(tla),
    expression_has_tla_type(TExpr,TlaType),!,
    translate_bvalue_with_tlatype(Value,TlaType,Output). % TO DO: treat Limit
translate_bvalue_for_expression_with_limit(Value,TExpr,Limit,Output) :-
    get_texpr_type(TExpr,Type),
    translate_bvalue_with_type_and_limit(Value,Type,Limit,Output).

expression_has_tla_type(TExpr,Type) :-
    get_texpr_info(TExpr,Infos),
    memberchk(tla_type(Type),Infos).

% a more refined pretty printing: takes Type information into account; useful for detecting sequences
translate_bvalue_with_type(Value,_,Output) :- var(Value),!,
    translate_bvalue(Value,Output).
translate_bvalue_with_type(Value,Type,Output) :-
    adapt_value_according_to_type(Type,Value,NewValue),
    translate_bvalue(NewValue,Output).

translate_bvalue_with_type_and_limit(Value,Type,Limit,Output) :-
    get_preference(expand_avl_upto,CurLim),
    SetLim is Limit//2, % at least two symbols per element
    %print(translate(SetLim,CurLim)),nl,
    (CurLim<0 ; SetLim < CurLim),!,
    temporary_set_preference(expand_avl_upto,SetLim,CHNG),
    translate_bvalue_with_type_and_limit2(Value,Type,Limit,Output),
    reset_temporary_preference(expand_avl_upto,CHNG).
translate_bvalue_with_type_and_limit(Value,Type,Limit,Output) :-
    translate_bvalue_with_type_and_limit2(Value,Type,Limit,Output).
translate_bvalue_with_type_and_limit2(Value,_,Limit,Output) :- var(Value),!,
    translate_bvalue_with_limit(Value,Limit,Output).
translate_bvalue_with_type_and_limit2(Value,Type,Limit,Output) :-
    adapt_value_according_to_type(Type,Value,NewValue),
    translate:translate_bvalue_with_limit(NewValue,Limit,Output).
    %debug:watch(translate:translate_bvalue_with_limit(NewValue,Limit,Output)).

:- use_module(avl_tools,[quick_avl_approximate_size/2]).
adapt_value_according_to_type(_,Var,R) :- var(Var),!,R=Var.
adapt_value_according_to_type(T,V,R) :- var(T),!,
   add_internal_error('Variable type: ',adapt_value_according_to_type(T,V,R)),
   R=V.
adapt_value_according_to_type(integer,V,R) :- !,R=V.
adapt_value_according_to_type(string,V,R) :- !,R=V.
adapt_value_according_to_type(boolean,V,R) :- !,R=V.
adapt_value_according_to_type(global(_),V,R) :- !,R=V.
adapt_value_according_to_type(couple(TA,TB),(VA,VB),R) :- !, R=(RA,RB),
   adapt_value_according_to_type(TA,VA,RA),
   adapt_value_according_to_type(TB,VB,RB).
adapt_value_according_to_type(set(Type),avl_set(A),Res) :- check_is_non_empty_avl(A),
   quick_avl_approximate_size(A,S),S<20,
   custom_explicit_sets:expand_custom_set_to_list(avl_set(A),List),!,
   maplist(adapt_value_according_to_type(Type),List,Res).
adapt_value_according_to_type(set(_Type),V,R) :- !,R=V.
adapt_value_according_to_type(seq(Type),V,R) :- !, % the type tells us it is a sequence
    (convert_set_into_sequence(V,VS)
        -> l_adapt_value_according_to_type(VS,Type,AVS),
           R=sequence(AVS)
        ;  R=V).
adapt_value_according_to_type(record(_Fields),Value,R) :- !, % to do: convert inside record
    R=Value.
adapt_value_according_to_type(freetype(_),Value,R) :- !, R=Value.
adapt_value_according_to_type(any,Value,R) :- !, R=Value.
adapt_value_according_to_type(Type,Value,R) :- print(adapt_value_according_to_type_unknown(Type,Value)),nl,
     R=Value.

l_adapt_value_according_to_type([],_Type,R) :- !,R=[].
l_adapt_value_according_to_type([H|T],Type,[AH|AT]) :-
    adapt_value_according_to_type(Type,H,AH),
    l_adapt_value_according_to_type(T,Type,AT).

pp_value_with_type(E,T,LimitReached) --> {adapt_value_according_to_type(T,E,AdaptedE)},
    pp_value(AdaptedE,LimitReached).

pp_value(V,In,Out) :- set_up_limit_reached(In,1000,LimitReached),
   pp_value(V,LimitReached,In,Out).

% LimitReached is a flag: when it is grounded to limit_reached this instructs pp_value to stop generating output
pp_value(_,LimitReached) --> {LimitReached==limit_reached},!, "...".
pp_value(V,_) --> {var(V)},!,underscore_symbol.
pp_value('$VAR'(N),_) --> !,pp_numberedvar(N).
pp_value(X,_) --> {cyclic_term(X)},!,underscore_symbol,"cyclic",underscore_symbol.
pp_value(fd(X),_) --> !,ppatom(X).  %% ???
pp_value(fd(X,GSet),_) --> {var(X)},!,
   ppatom(GSet),":", ppnumber(X). %":??".
pp_value(fd(X,GSet),_) -->
    {b_global_sets:is_b_global_constant_hash(GSet,X,Res)},!,
    ppatom_opt_scramble(Res).
pp_value(fd(X,GSet),_) --> {deferred_set_constant(GSet,X,Cst)},!,
         ppatom_opt_scramble(Cst).
pp_value(fd(X,M),_) --> !,ppatom_opt_scramble(M),ppnumber(X).
pp_value(int(X),_) --> !,ppnumber(X).
pp_value(string(X),_) --> !,string_start_symbol,ppstring_opt_scramble(X),string_end_symbol.
pp_value(global_set(X),_) --> {atomic(X),integer_set_mapping(X,Y)},!,ppatom(Y).
pp_value(term(X),_) --> {var(X)},!,"term(",underscore_symbol,")".
pp_value(freetype(X),_) --> {pretty_freetype(X,P)},!,ppatom_opt_scramble(P).
pp_value(pred_true /* bool_true */,_) --> %!,"TRUE". % TO DO: in latex_mode: surround by mathit
   {constants_in_mode(pred_true,Symbol)},!,ppatom(Symbol).
pp_value(pred_false /* bool_false */,_) --> %!,"FALSE".
   {constants_in_mode(pred_false,Symbol)},!,ppatom(Symbol).
%pp_value(bool_true) --> !,"TRUE". % old version; still in some test traces which are printed
%pp_value(bool_false) --> !,"FALSE".
pp_value([],_) --> !,empty_set_symbol.
pp_value(freeval(_,Case,Value),_) -->
      {ground(Case),ground(Value),Value=term(Case)},!,ppatom_opt_scramble(Case).
pp_value(sequence(List),LimitReached) --> !,
    ({List=[]} -> pp_empty_sequence ; pp_sequence_with_limit(List,LimitReached)).
pp_value([Head|Tail],LimitReached) --> {get_preference(translate_print_all_sequences,true),
                           convert_set_into_sequence([Head|Tail],Elements)},
   !,
   pp_sequence(Elements,LimitReached).
pp_value([Head|Tail],LimitReached) --> !, {set_brackets(L,R)},
    ppatom(L),
    pp_value_l_with_limit([Head|Tail],',',LimitReached),
    ppatom(R).
%pp_value([Head|Tail]) --> !,
%    {( convert_set_into_sequence([Head|Tail],Elements) ->
%       (Start,End) = ('[',']')
%     ; otherwise ->
%       Elements = [Head|Tail],
%       (Start,End) = ('{','}'))},
%    ppatom(Start),pp_value_l(Elements,','),ppatom(End).
pp_value( (A,B) ,LimitReached) --> !,
    "(",pp_inner_value(A,LimitReached),
        maplet_symbol,
        pp_value(B,LimitReached),")".
pp_value(avl_set(A),LimitReached) --> !,
    {check_is_non_empty_avl(A),avl_size(A,Sz)},
    {set_brackets(LBrace,RBrace)},
    ( {size_is_in_set_limit(Sz),
       %(Sz>2 ; get_preference(translate_print_all_sequences,true)),
        get_preference(translate_print_all_sequences,true), % no longer try and convert any sequence longer than 2 to sequence notation
        avl_max(A,(int(Sz),_)), % a sequence has minimum int(1) and maximum int(Sz)
        convert_avlset_into_sequence(A,Seq)} ->
        pp_sequence(Seq,LimitReached)
    ; {otherwise} ->
        ( {Sz=0} -> left_set_bracket," /* empty avl_set */ ",right_set_bracket
        ; {(size_is_in_set_limit(Sz) ; Sz < 3)} -> % if Sz 3 we will print at least two elements anyway
            {avl_domain(A,List)},
            ppatom(LBrace),pp_value_l(List,',',LimitReached),ppatom(RBrace)
        ; {(Sz<5 ; \+ size_is_in_set_limit(4))} ->
            {avl_min(A,Min),avl_max(A,Max)},
            hash_card_symbol, % "#"
            ppnumber(Sz),":", left_set_bracket,
            pp_value(Min,LimitReached),",",ldots,",",pp_value(Max,LimitReached),right_set_bracket
        ; {otherwise} ->
            {avl_min(A,Min),avl_next(Min,A,Nxt),avl_max(A,Max),avl_prev(Max,A,Prev)},
            hash_card_symbol, % "#",
            ppnumber(Sz),":", left_set_bracket,
            pp_value(Min,LimitReached),",",pp_value(Nxt,LimitReached),",",ldots,",",
            pp_value(Prev,LimitReached),",",pp_value(Max,LimitReached),right_set_bracket )).

pp_value(field(Name,Value),LimitReached) --> !,
    ppatom_opt_scramble(Name),":",pp_value(Value,LimitReached). % : for fields has priority 120 in French manual
pp_value(rec(Rec),LimitReached) --> !,
    {function_like_in_mode(rec,Symbol)},
    ppatom(Symbol), "(",pp_value_l(Rec,',',LimitReached),")".
pp_value(struct(Rec),LimitReached) --> !,
    {function_like_in_mode(struct,Symbol)},
    ppatom(Symbol), "(", pp_value_l(Rec,',',LimitReached),")".
pp_value(term(no_value_for(Id)),_) --> !,
    "undefined ",ppatom(Id).
pp_value(closure(Ids,Type,B),_LimitReached) -->
   {var(Ids) ; var(Type) ; var(B)},!,
   add_internal_error('Illegal value: ',pp_value_illegal_closure(Ids,Type,B)),
   "<< ILLEGAL ",write_to_codes(closure(Ids,Type,B))," >>".
pp_value(closure([Id],[Type],Membership),LimitReached) -->
    { get_texpr_expr(Membership,member(Elem,Set)),
      get_texpr_id(Elem,Id),get_texpr_type(Elem,Type),!},
    pp_expr_m(Set,299,LimitReached).
pp_value(closure(Variables,Types,Predicate),LimitReached) --> {\+ size_is_in_set_limit(1)},
   !, % do not print body; just print hash value
    {make_closure_ids(Variables,Types,Ids), term_hash(Predicate,PH)},
    left_set_bracket,  % { Ids | #PREDICATE#(HASH) }
    pp_expr_l_pair_in_mode(Ids,LimitReached),
    pp_such_that_bar,
    " ",hash_card_symbol,"PREDICATE",hash_card_symbol,"(",ppnumber(PH),") ", right_set_bracket.
pp_value(closure(Variables,Types,Predicate),LimitReached) --> !,
    {make_closure_ids(Variables,Types,Ids)},
    pp_comprehension_set(Ids,Predicate,[],LimitReached).% TODO: propagate LimitReached
pp_value(freeval(_,Case,Value),LimitReached) --> {ground(Case)},!,
    ppatom_opt_scramble(Case),"(",pp_value(Value,LimitReached),")".
pp_value(X,_) -->  {animation_mode(xtl)},!,
    write_to_codes(X).
pp_value(X,_) -->  % the << >> pose problems when checking against FDR
    "<< ",write_to_codes(X)," >>".

% avoid printing parentheses:
% (x,y,z) = ((x,y),z)
pp_inner_value( AB , LimitReached) --> {nonvar(AB),AB=(A,B)}, !, % do not print parentheses in this context
        pp_inner_value(A,LimitReached),maplet_symbol,
        pp_value(B,LimitReached).
pp_inner_value( Value , LimitReached) --> pp_value( Value , LimitReached).

size_is_in_set_limit(Size) :- get_preference(expand_avl_upto,Max),
    (Max<0 -> true /* no limit */
     ; Size =< Max).

% instantiate LimitReached argument as soon as a list exceeds a certain limit
set_up_limit_reached(_,Neg,_) :- Neg<0,!. % negative number means unlimited
set_up_limit_reached(_,0,LimitReached) :- !, LimitReached = limit_reached.
set_up_limit_reached(List,Limit,LimitReached) :-
   block_set_up_limit_reached(List,Limit,LimitReached).
:- block block_set_up_limit_reached(-,?,?).
block_set_up_limit_reached([],_,_).
block_set_up_limit_reached([_|T],Limit,LimitReached) :-
  (Limit<1 -> LimitReached=limit_reached
   ; L1 is Limit-1, block_set_up_limit_reached(T,L1,LimitReached)).

% pretty print LimitReached, requires %:- block block_set_up_limit_reached(-,?,-).
/*
pp_lr(LR) --> {LR==limit_reached},!, " *LR* ".
pp_lr(LR) --> {frozen(LR,translate:block_set_up_limit_reached(_,Lim,_))},!, " ok(", ppnumber(Lim),") ".
pp_lr(LR) --> {frozen(LR,G)},!, " ok(", ppterm(G),") ".
pp_lr(_) --> " ok ".
*/


pp_value_l_with_limit(V,Sep,LimitReached) --> {get_preference(expand_avl_upto,Max)},
   pp_value_l(V,Sep,Max,LimitReached).
pp_value_l(V,Sep,LimitReached) --> pp_value_l(V,Sep,-1,LimitReached).

pp_value_l(V,_Sep,_,_) --> {var(V)},!,"...".
pp_value_l(_,_,_,LimitReached) --> {LimitReached==limit_reached},!,"...".
pp_value_l('$VAR'(N),_Sep,_,_) --> !,"}\\/{",pp_numberedvar(N),"}".
pp_value_l([],_Sep,_,_) --> !.
pp_value_l([Expr|Rest],Sep,Limit,LimitReached) -->
    ( {nonvar(Rest),Rest=[]} ->
        pp_value(Expr,LimitReached)
    ; {Limit=0} -> "..."
    ; {otherwise} ->
        pp_value(Expr,LimitReached),
        % no separator for closure special case
        ({nonvar(Rest) , Rest = closure(_,_,_)} -> {true} ; ppatom(Sep)) ,
        {L1 is Limit-1} ,
        % convert avl_set(_) in a list's tail to a Prolog list
        {nonvar(Rest) , Rest = avl_set(_) -> custom_explicit_sets:expand_custom_set_to_list(Rest,LRest) ; LRest = Rest} ,
        pp_value_l(LRest,Sep,L1,LimitReached)).
pp_value_l(avl_set(A),_Sep,_,LimitReached) --> pp_value(avl_set(A),LimitReached).
pp_value_l(closure(A,B,C),_Sep,_,LimitReached) --> "}\\/", pp_value(closure(A,B,C),LimitReached).

make_closure_ids([],[],[]).
make_closure_ids([V|Vrest],[T|Trest],[TExpr|TErest]) :-
    create_texpr(identifier(V),T,[],TExpr),
    make_closure_ids(Vrest,Trest,TErest).

% symbol for starting and ending a sequence:
pp_begin_sequence --> {animation_minor_mode(tla)},!,"<<".
pp_begin_sequence --> {get_preference(translate_print_cs_style_sequences,true)},!,"".
pp_begin_sequence --> "[".
pp_end_sequence --> {animation_minor_mode(tla)},!,">>".
pp_end_sequence --> {get_preference(translate_print_cs_style_sequences,true)},!,"".
pp_end_sequence --> "]".

pp_separator_sequence('') :- get_preference(translate_print_cs_style_sequences,true),!.
pp_separator_sequence(',').

% string for empty sequence
pp_empty_sequence --> {animation_minor_mode(tla)},!, "<< >>".
pp_empty_sequence --> {get_preference(translate_print_cs_style_sequences,true)},!,
  ( {latex_mode} -> "\\lambda" ; [955]). % 955 is lambda symbol in Unicode
pp_empty_sequence --> "[]".

% symbols for function application:
pp_function_left_bracket --> {animation_minor_mode(tla)},!, "[".
pp_function_left_bracket --> "(".

pp_function_right_bracket --> {animation_minor_mode(tla)},!, "]".
pp_function_right_bracket --> ")".

pp_sequence(Elements,LimitReached) --> {pp_separator_sequence(Sep)},
   pp_begin_sequence,
   pp_value_l(Elements,Sep,LimitReached),
   pp_end_sequence.
pp_sequence_with_limit(Elements,LimitReached) --> {pp_separator_sequence(Sep)},
   pp_begin_sequence,
   pp_value_l_with_limit(Elements,Sep,LimitReached),
   pp_end_sequence.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% machines

:- use_module(eventhandling,[register_event_listener/3]).
:- register_event_listener(clear_specification,reset_translate,
                           'Reset Translation Caches.').
reset_translate :- retractall(bugly_scramble_id_cache(_,_)), retractall(non_det_constants(_,_)).
%reset_translate :- set_print_type_infos(none),
%    set_preference(translate_suppress_rodin_positions_flag,false).

suppress_rodin_positions :-
    set_preference(translate_suppress_rodin_positions_flag,true).

set_print_type_infos(none) :- !,
    set_preference(translate_force_all_typing_infos,false),
    set_preference(translate_print_typing_infos,false).
set_print_type_infos(needed) :- !,
    set_preference(translate_force_all_typing_infos,false),
    set_preference(translate_print_typing_infos,true).
set_print_type_infos(all) :- !,
    set_preference(translate_force_all_typing_infos,true),
    set_preference(translate_print_typing_infos,true).
set_print_type_infos(Err) :-
    add_internal_error('Illegal type: ',set_print_type_infos(Err)).


:- use_module(tools_files,[put_codes/2]).
print_machine(M) :-
    nl, translate_machine(M,Msg,true), put_codes(Msg,user_output), nl,
    flush_output(user_output),!.
print_machine(M) :- add_internal_error('Printing failed: ',print_machine(M)).

%
translate_machine(M,Codes,AdditionalInfo) :-
    temporary_set_preference(translate_ids_to_parseable_format,true,CHNG),
    retractall(print_additional_machine_info),
    (AdditionalInfo=true -> assert(print_additional_machine_info) ; true),
    call_cleanup(translate_machine1(M,(0,Codes),(_,[])),
                 reset_temporary_preference(translate_ids_to_parseable_format,CHNG)).

% useful if we wish to translate just a selection of sections without MACHINE/END
translate_section_list(SL,Codes) :- init_machine_translation,
    translate_machine2(SL,SL,no_end,(0,Codes),(_,[])).

translate_machine1(machine(Name,Sections)) -->
    indent('MACHINE '), {adapt_machine_name(Name,AName)}, insertstr(AName),
    {init_machine_translation},
    translate_machine2(Sections,Sections,end).
translate_machine2([],_,end) --> !, insertstr('\nEND\n').
translate_machine2([],_,_) --> !, insertstr('\n').
translate_machine2([P|Rest],All,End) -->
    translate_mpart(P,All),
    translate_machine2(Rest,All,End).

adapt_machine_name('dummy(uses)',R) :- !,R='MAIN'.
adapt_machine_name(X,X).

:- dynamic section_header_generated/1.
:- dynamic print_additional_machine_info/0.
print_additional_machine_info.

init_machine_translation :- retractall(section_header_generated(_)).

% start a part of a section
mpstart(Title,I) -->
    insertstr('\n'),insertstr(Title),
    indention_level(I,I2), {I2 is I+2}.
% end a part of a section
mpend(I) -->
    indention_level(_,I).

mpstart_section(Section,Title,AltTitle,I,In,Out) :-
   (\+ section_header_generated(Section)
      -> mpstart(Title,I,In,Out), assert(section_header_generated(Section))
      ;  mpstart(AltTitle,I,In,Out) /* use alternative title; section header already generated */
   ).

translate_mpart(Section/I,All) --> %{print(Section),nl},
         ( {I=[]}  -> {true}
         ; translate_mpart2(Section,I,All) -> {true}
         ; {otherwise} ->
             insertstr('\nSection '),insertstr(Section),insertstr(': '),
             insertstr('<< pretty-print failed >>')
         ).
translate_mpart2(deferred_sets,I,_) -->
      mpstart_section(sets,'SETS /* deferred */',' ; /* deferred */',P),
      indent_expr_l_sep(I,';'),mpend(P).
translate_mpart2(enumerated_sets,_I,_) --> [].  % these are now pretty printed below
     %mpstart('ENUMERATED SETS',P),indent_expr_l_sep(I,';'),mpend(P).
translate_mpart2(enumerated_elements,I,_) -->  %{print(enum_els(I)),nl},
      {translate_enums(I,[],Res)},
      mpstart_section(sets,'SETS /* enumerated */',' ; /* enumerated */',P),
      indent_expr_l_sep(Res,';'),mpend(P).
translate_mpart2(parameters,I,_) --> mpstart('PARAMETERS',P),indent_expr_l_sep(I,','),mpend(P).
translate_mpart2(internal_parameters,I,_) --> {print_additional_machine_info},!,
       mpstart('/* INTERNAL_PARAMETERS',P),indent_expr_l_sep(I,','),insertstr(' */'),mpend(P).
translate_mpart2(internal_parameters,_I,_) --> [].
translate_mpart2(abstract_variables,I,_) --> mpstart('ABSTRACT_VARIABLES',P),indent_expr_l_sep(I,','),mpend(P).
translate_mpart2(concrete_variables,I,_) --> mpstart('CONCRETE_VARIABLES',P),indent_expr_l_sep(I,','),mpend(P).
translate_mpart2(abstract_constants,I,_) --> mpstart('ABSTRACT_CONSTANTS',P),indent_expr_l_sep(I,','),mpend(P).
translate_mpart2(concrete_constants,I,_) --> mpstart('CONCRETE_CONSTANTS',P),indent_expr_l_sep(I,','),mpend(P).
translate_mpart2(promoted,I,_) --> {print_additional_machine_info},!,
       mpstart('/* PROMOTED OPERATIONS',P),indent_expr_l_sep(I,','),insertstr(' */'),mpend(P).
translate_mpart2(promoted,_I,_) --> [].
translate_mpart2(unpromoted,I,_) --> {print_additional_machine_info},!,
       mpstart('/* NOT PROMOTED OPERATIONS',P),indent_expr_l_sep(I,','),insertstr(' */'),mpend(P).
translate_mpart2(unpromoted,_I,_) --> [].
translate_mpart2(constraints,I,All) --> mpart_typing(constraints,[parameters],All,I).
translate_mpart2(invariant,I,All) -->  mpart_typing(invariant, [abstract_variables,concrete_variables],All,I).
translate_mpart2(linking_invariant,_I,_) --> [].
translate_mpart2(properties,I,All) --> mpart_typing(properties,[abstract_constants,concrete_constants],All,I).
translate_mpart2(assertions,I,_) -->
     mpstart_spec_desc(assertions,P),
     %indent_expr_l_sep(I,';'),
     preds_over_lines(1,'@thm','; ',I),
     mpend(P). % TO DO:
translate_mpart2(initialisation,S,_) --> mpstart_spec_desc(initialisation,P),translate_inits(S),mpend(P).
translate_mpart2(definitions,Defs,_) --> {print_additional_machine_info},!,
        mpstart('/* DEFINITIONS',P),translate_defs(Defs),insertstr(' */'),mpend(P).
translate_mpart2(definitions,_Defs,_) --> [].
translate_mpart2(operation_bodies,Ops,_) --> mpstart_spec_desc(operations,P),translate_ops(Ops),mpend(P).
translate_mpart2(used,Used,_) --> {print_additional_machine_info},!,
        mpstart('/* USED',P),translate_used(Used),insertstr(' */'),mpend(P).
translate_mpart2(used,_Used,_) --> [].
translate_mpart2(freetypes,Freetypes,_) --> mpstart('FREETYPES',P),translate_freetypes(Freetypes),mpend(P).
translate_mpart2(meta,_Infos,_) --> [].
translate_mpart2(operators,Operators,_) -->
    insertstr('\n/* Event-B operators:'), % */
    indention_level(I,I2), {I2 is I+2},
    translate_eventb_operators(Operators),
    indention_level(I2,I),
    insertstr('\n*/').
translate_mpart2(values,Values,_) -->
    mpstart('VALUES',P),indent_expr_l_sep(Values,';'),mpend(P).

% Add typing predicates to a predicate
mpart_typing(Title,Section,Sections,PredI) -->
    {mpart_typing2(Section,Sections,PredI,PredO)},
    ( {is_truth(PredO)} -> [] % TO DO: in animation_minor_mode(z) for INVARIANT: force adding typing predicates (translate_print_typing_infos)
    ; {otherwise} ->
        mpstart_spec_desc(Title,P),
        section_pred_over_lines(0,Title,PredO),
        mpend(P)).

mpstart_spec_desc(Title,P) --> {get_specification_description(Title,Atom)},!, mpstart(Atom,P).
mpstart_spec_desc(Title,P) --> mpstart(Title,P).

mpart_typing2(Sections,AllSections,PredI,PredO) :-
    get_preference(translate_print_typing_infos,true),!,
    get_all_ids(Sections,AllSections,Ids),
    add_typing_predicates(Ids,PredI,PredO).
mpart_typing2(_Section,_Sections,Pred,Pred).

get_all_ids([],_Sections,[]).
get_all_ids([Section|Srest],Sections,Ids) :-
    memberchk(Section/Ids1,Sections),
    append(Ids1,Ids2,Ids),
    get_all_ids(Srest,Sections,Ids2).

add_optional_typing_predicates(Ids,In,Out) :-
    ( get_preference(translate_print_typing_infos,true) -> add_typing_predicates(Ids,In,Out)
     ; is_truth(In) -> add_typing_predicates(Ids,In,Out)
     ; otherwise    -> In=Out).

add_normal_typing_predicates(Ids,In,Out) :-  % used to call add_typing_predicates directly
    (add_optional_typing_predicates(Ids,In,Out) -> true
      ; add_internal_error('Failed: ',add_normal_typing_predicates(Ids)), In=Out).

add_typing_predicates([],P,P) :- !.
add_typing_predicates(Ids,Pin,Pout) :-
    remove_already_typed_ids(Pin,Ids,UntypedIds),
    KeepSeq=false,
    generate_typing_predicates(UntypedIds,KeepSeq,Typing),
    conjunction_to_list(Pin,Pins),
    remove_duplicate_predicates(Typing,Pins,Typing2),
    append(Typing2,[Pin],Preds),
    conjunct_predicates(Preds,Pout).

remove_already_typed_ids(_TExpr,Ids,Ids) :-
    get_preference(translate_force_all_typing_infos,true),!.
remove_already_typed_ids(TExpr,Ids,UntypedIds) :-
    get_texpr_expr(TExpr,Expr),
    remove_already_typed_ids2(Expr,Ids,UntypedIds).
remove_already_typed_ids2(conjunct(A,B),Ids,UntypedIds) :- !,
    remove_already_typed_ids(A,Ids,I1),
    remove_already_typed_ids(B,I1,UntypedIds).
remove_already_typed_ids2(lazy_let_pred(_,_,A),Ids,UntypedIds) :- !,
    remove_already_typed_ids(A,Ids,UntypedIds). % TO DO: check for variable clases with lazy_let ids ???
remove_already_typed_ids2(Expr,Ids,UntypedIds) :-
    is_typing_predicate(Expr,Id),
    create_texpr(identifier(Id),_,_,TId),
    select(TId,Ids,UntypedIds),!.
remove_already_typed_ids2(_,Ids,Ids).
is_typing_predicate(member(A,_),Id) :- get_texpr_id(A,Id).
is_typing_predicate(subset(A,_),Id) :- get_texpr_id(A,Id).
is_typing_predicate(subset_strict(A,_),Id) :- get_texpr_id(A,Id).

remove_duplicate_predicates([],_Old,[]).
remove_duplicate_predicates([Pred|Prest],Old,Result) :-
    ( is_duplicate_predicate(Pred,Old) -> Result = Rest
    ; otherwise -> Result = [Pred|Rest]),
    remove_duplicate_predicates(Prest,Old,Rest).
is_duplicate_predicate(Pred,List) :-
    remove_all_infos(Pred,Pattern),
    memberchk(Pattern,List).

generate_typing_predicates(TIds,Preds) :-
   generate_typing_predicates(TIds,true,Preds).
generate_typing_predicates(TIds,KeepSeq,Preds) :-
    maplist(generate_typing_predicate(KeepSeq), TIds, Preds).
generate_typing_predicate(KeepSeq,TId,Pred) :-
    get_texpr_type(TId,Type),
    remove_all_infos_and_ground(TId,TId2), % clear all infos
    (type_set(Type,KeepSeq,TSet) -> create_texpr(member(TId2,TSet),pred,[],Pred)
     ; add_error(generate_typing_predicate,'Illegal type: ',Type),
       is_truth(Pred)
     ).

% translate a ProB type to a typed expression representing the maximal type
set_type_to_maximal_texpr(ProBType,TExpr) :- replace_seq(ProBType,MType),type_set(MType,TExpr).

% replace all occurences of seq/1 by couple(....)
replace_seq(set(T),Res) :- !, Res=set(RT),replace_seq(T,RT).
replace_seq(seq(T),Res) :- !, Res=set(couple(integer,RT)),replace_seq(T,RT).
replace_seq(couple(A,B),Res) :- !, Res=couple(RA,RB),replace_seq(A,RA), replace_seq(B,RB).
replace_seq(X,X).

% translate a ProB type into a B expression so that we can write x:TSet
type_set(Type,TSet) :- type_set(Type,true,TSet).
type_set(Type,KeepSeq,TSet) :-
    ( var(Type) -> add_error_and_fail(translate,'type_set Type is variable: ',Type)
    ; otherwise ->
        type_set2(Type,KeepSeq,Set,Infos),
        create_texpr(Set,set(Type),Infos,TSet)).
type_set2(set(T),KeepSeq,pow_subset(Set),[]) :-
    type_set(T,KeepSeq,Set).
type_set2(seq(T),KeepSeq,Res,Info) :-
  (KeepSeq = true
   -> Res=seq(Set), Info=[], type_set(T,KeepSeq,Set) % keep sequence types as is
    ; type_set2(set(couple(integer,T)),KeepSeq,Res,Info)). % write as POW(INTEGER*T)
type_set2(couple(TA,TB),KeepSeq,cartesian_product(SetA,SetB),[]) :-
    type_set(TA,KeepSeq,SetA), type_set(TB,KeepSeq,SetB).
%type_set2(global(G),_,identifier(G),[given_set]).
type_set2(global(G),_,value(global_set(G)),[given_set]). % avoids variable capture/clash
type_set2(integer,_,integer_set('INTEGER'),[]).
type_set2(string,_,string_set,[]).
type_set2(boolean,_,bool_set,[]).
type_set2(record(TFields), KeepSeq, struct(Rec),[]) :-
    create_texpr(rec(SFields),record(TFields),[],Rec),
    type_set_fields(TFields,KeepSeq,SFields).
%type_set2(freetype(Id),identifier(Id),[given_set]).
type_set2(freetype(Id),_,freetype_set(Id),[]).
type_set2(constant(L),_,value(Terms),[]) :-  % these types occur in Event-B for Theory plugin inductive datatypes
    maplist(create_term,L,Terms).
create_term(T,term(T)).
type_set_fields([],_,[]).
type_set_fields([field(Id,Type)|TFrest],KeepSeq,[field(Id,TSet)|EFrest]) :-
    type_set(Type,KeepSeq,TSet),
    type_set_fields(TFrest,KeepSeq,EFrest).


% translate enumerated constant list into enumerate set definition
translate_enums([],Acc,Acc).
translate_enums([EnumCst|T],Acc,Res) :- %get_texpr_id(EnumCst,Id),
    get_texpr_type(EnumCst,global(GlobalSet)),
    insert_enum_cst(Acc,EnumCst,GlobalSet,Acc2),
    translate_enums(T,Acc2,Res).

insert_enum_cst([],ID,Type,[enumerated_set_def(Type,[ID])]).
insert_enum_cst([enumerated_set_def(Type,Lst)|T],ID,Type2,[enumerated_set_def(Type,Lst2)|TT]) :-
   (Type=Type2
     -> Lst2 = [ID|Lst], TT=T
     ;  Lst2 = Lst, insert_enum_cst(T,ID,Type2,TT)
   ).

% pretty-print the initialisation section of a machine
translate_inits(Inits) -->
    ( {is_list_simple(Inits)} ->
        translate_inits2(Inits)
    ; {otherwise} ->
        indention_level(I,I2),{I2 is I+2},
        translate_subst_begin_end(Inits),
        indention_level(_,I)).
translate_inits2([]) --> !.
translate_inits2([init(Name,Subst)|Rest]) -->
    indent('/* '),insertstr(Name),insertstr(': */ '),
    translate_subst_begin_end(Subst),
    translate_inits2(Rest).

% pretty-print the definitions of a machine
translate_defs([]) --> !.
translate_defs([Def|Rest]) --> translate_def(Def),translate_defs(Rest).
translate_def(definition_decl(Name,DefType,_Pos,Args,Expr,_Deps)) -->
    {def_description(DefType,Desc)}, indent(Desc),insertstr(Name),
    {transform_raw_list(Args,TArgs)},
    translate_op_params(TArgs),
    ( {show_def_body(Expr)}
      -> insertstr(' == '), {transform_raw(Expr,TExpr)},
         (translate_def_body(DefType,TExpr) -> [] ; insertstr('CANNOT PRETTY PRINT'))
      ;  {true}
    ),
    insertstr(';').
def_description(substitution,'SUBSTITUTION ').
def_description(expression,'EXPRESSION ').
def_description(predicate,'PREDICATE ').
translate_def_body(substitution,B) --> translate_subst_begin_end(B).
translate_def_body(expression,B) --> indent_expr(B).
translate_def_body(predicate,B) --> indent_expr(B).

show_def_body(integer(_,_)).
show_def_body(boolean_true(_)).
show_def_body(boolean_false(_)).
% show_def_body(_)  % comment in to pretty print all defs

% ------------- RAW EXPRESSIONS

% try and print raw machine term or parts thereof (e.g. sections)
print_raw_machine_terms(Var) :- var(Var), !,print('VAR !!'),nl.
print_raw_machine_terms([]) :- !.
print_raw_machine_terms([H|T]) :- !,
   print_raw_machine_terms(H), print(' '),
   print_raw_machine_terms(T).
print_raw_machine_terms(Term) :- raw_machine_term(Term,String,Sub),!,
   format('~n~w ',[String]),
   print_raw_machine_terms(Sub),nl.
print_raw_machine_terms(expression_definition(A,B,C,D)) :- !,
   print_raw_machine_terms(predicate_definition(A,B,C,D)).
print_raw_machine_terms(substitution_definition(A,B,C,D)) :- !,
   print_raw_machine_terms(predicate_definition(A,B,C,D)).
print_raw_machine_terms(predicate_definition(_,Name,Paras,RHS)) :-
   Paras==[],!,
   format('~n  ~w == ',[Name]),
   print_raw_machine_terms(RHS),nl.
print_raw_machine_terms(predicate_definition(_,Name,Paras,RHS)) :- !,
   format('~n  ~w(',[Name]),
   print_raw_machine_terms_sep(Paras,','),
   format(') == ',[]),
   print_raw_machine_terms(RHS),nl.
print_raw_machine_terms(operation(_,Name,Return,Paras,RHS)) :- !,
   format('~n  ',[]),
   (Return=[] -> true
     ; print_raw_machine_terms_sep(Return,','),
       format(' <-- ',[])
   ),
   print_raw_machine_terms(Name),
   (Paras=[] -> true
     ; format(' (',[]),
       print_raw_machine_terms_sep(Paras,','),
       format(')',[])
    ),
   format(' = ',[]),
   print_raw_machine_terms(RHS),nl.
print_raw_machine_terms(Term) :- print_raw_bexpr(Term).


print_raw_machine_terms_sep([],_) :- !.
print_raw_machine_terms_sep([H],_) :- !,
   print_raw_machine_terms(H).
print_raw_machine_terms_sep([H|T],Sep) :- !,
   print_raw_machine_terms(H),print(Sep),print_raw_machine_terms_sep(T,Sep).

raw_machine_term(machine(M),'',M).
raw_machine_term(generated(_,M),'',M).
raw_machine_term(machine_header(_,Name,_Params),Name,[]). % TO DO: treat Params
raw_machine_term(abstract_machine(_,_,Header,M),'MACHINE',[Header,M]).
raw_machine_term(properties(_,P),'PROPERTIES',P).
raw_machine_term(operations(_,P),'OPERATIONS',P).
raw_machine_term(definitions(_,P),'DEFINITIONS',P).
raw_machine_term(constants(_,P),'CONSTANTS',P).
raw_machine_term(variables(_,P),'VARIABLES',P).
raw_machine_term(invariant(_,P),'INVARIANT',P).
raw_machine_term(assertions(_,P),'ASSERTIONS',P).
raw_machine_term(constraints(_,P),'CONSTRAINTS',P).
raw_machine_term(sets(_,P),'SETS',P).
raw_machine_term(deferred_set(_,P),P,[]). % TO DO: enumerated_set ...
%raw_machine_term(identifier(_,P),P,[]).

print_raw_bexpr(Raw) :- % a tool (not perfect) to print raw ASTs
   transform_raw(Raw,TExpr), print_bexpr_or_subst(TExpr).

transform_raw_list(Var,Res) :-  var(Var),!,
   add_internal_error('Var raw expression list:',transform_raw_list(Var,Res)),
   Res= [b(identifier('$$VARIABLE_LIST$$'),any,[raw])].
transform_raw_list(Args,TArgs) :- maplist(transform_raw,Args,TArgs).

transform_raw(Var,Res) :- %print(raw(Var)),nl,
   var(Var), !, add_internal_error('Var raw expression:',transform_raw(Var,Res)),
   Res= b(identifier('$$VARIABLE$$'),any,[raw]).
transform_raw(precondition(_,Pre,Body),Res) :- !, Res= b(precondition(TP,TB),subst,[raw]),
   transform_raw(Pre,TP),
   transform_raw(Body,TB).
transform_raw(identifier(_,M),Res) :- !, Res= b(identifier(M),any,[raw]).
transform_raw(integer(_,M),Res) :- !, Res= b(integer(M),integer,[raw]).
transform_raw(let_expression(_,_Ids,Eq,Body),Res) :- !,
   transform_raw(conjunct(_,Eq,Body),Res). % TO DO: fix and generate let_expression(Ids,ListofExprs,Body)
transform_raw(let_predicate(_,_Ids,Eq,Body),Res) :- !,
   transform_raw(conjunct(_,Eq,Body),Res). % ditto
transform_raw(forall(_,Ids,Body),Res) :- !,
   (Body=implication(_,LHS,RHS) -> true ; LHS=truth,RHS=Body),
   transform_raw(forall(_,Ids,LHS,RHS),Res).
transform_raw(record_field(_,Rec,identifier(_,Field)),Res) :- !, Res = b(record_field(TRec,Field),any,[]),
   transform_raw(Rec,TRec).
transform_raw(rec_entry(_,identifier(_,Field),Rec),Res) :- !, Res = field(Field,TRec),
   transform_raw(Rec,TRec).
transform_raw(couple(_,L),Res) :- !, transform_raw_list_to_couple(L,Res). % couples are represented by lists
transform_raw(function(_,F,L),Res) :- !, transform_raw(F,TF),
   Res = b(function(TF,Args),any,[]),
   transform_raw_list_to_couple(L,Args). % args are represented by lists
transform_raw(Atom,Res) :- atomic(Atom),!,Res=Atom.
transform_raw([H|T],Res) :- !, maplist(transform_raw,[H|T],Res).
transform_raw(OtherOp,b(Res,Type,[used_ids([])])) :- OtherOp =..[F,_Pos|Rest],
     maplist(transform_raw,Rest,TRest),
     (get_type(F,FT) -> Type=FT ; Type=any),
     Res =.. [F|TRest].
transform_raw_list_to_couple([R],Res) :- !, transform_raw(R,Res).
transform_raw_list_to_couple([R1|T],Res) :- !, Res=b(couple(TR1,TT),any,[]),
    transform_raw(R1,TR1),transform_raw_list_to_couple(T,TT).
get_type(conjunct,pred).
get_type(disjunct,pred).
get_type(implication,pred).
get_type(equivalence,pred).
get_type(member,pred).
get_type(equal,pred).
get_type(not_equal,pred).
get_type(not_member,pred).
get_type(subset,pred).
get_type(not_subset,pred).

% -------------


% pretty-print the operations of a machine
translate_ops([]) --> !.
translate_ops([Op|Rest]) -->
    translate_op(Op),
    ({Rest=[]} -> {true}; insertstr(';'),indent),
    translate_ops(Rest).
translate_op(Op) -->
    { get_texpr_expr(Op,operation(Id,Res,Params,Body)) },
    translate_operation(Id,Res,Params,Body).
translate_operation(Id,Res,Params,Body) -->
    indent,translate_op_results(Res),
    pp_expr_indent(Id),
    translate_op_params(Params),
    insertstr(' = '),
    indention_level(I1,I2),{I2 is I1+2,type_infos_in_subst(Params,Body,Body2)},
    translate_subst_begin_end(Body2),
    indention_level(_,I1).
translate_op_results([]) --> !.
translate_op_results(Ids) --> pp_expr_indent_l(Ids), insertstr(' <-- ').
translate_op_params([]) --> !.
translate_op_params(Ids) --> insertstr('('),pp_expr_indent_l(Ids), insertstr(')').

translate_subst_begin_end(TSubst) -->
    {get_texpr_expr(TSubst,Subst),subst_needs_begin_end(Subst),
     create_texpr(block(TSubst),subst,[],Block)},!,
    translate_subst(Block).
translate_subst_begin_end(Subst) -->
    translate_subst(Subst).

subst_needs_begin_end(assign(_,_)).
subst_needs_begin_end(assign_single_id(_,_)).
subst_needs_begin_end(parallel(_)).
subst_needs_begin_end(sequence(_)).
subst_needs_begin_end(operation_call(_,_,_)).

type_infos_in_subst([],Subst,Subst) :- !.
type_infos_in_subst(Ids,SubstIn,SubstOut) :-
    get_preference(translate_print_typing_infos,true),!,
    type_infos_in_subst2(Ids,SubstIn,SubstOut).
type_infos_in_subst(_Ids,Subst,Subst).
type_infos_in_subst2(Ids,SubstIn,SubstOut) :-
    get_texpr_expr(SubstIn,precondition(P1,S)),!,
    get_texpr_info(SubstIn,Info),
    create_texpr(precondition(P2,S),pred,Info,SubstOut),
    add_typing_predicates(Ids,P1,P2).
type_infos_in_subst2(Ids,SubstIn,SubstOut) :-
    create_texpr(precondition(P,SubstIn),pred,[],SubstOut),
    generate_typing_predicates(Ids,Typing),
    conjunct_predicates(Typing,P).





% pretty-print the internal section about included and used machines
translate_used([]) --> !.
translate_used([Used|Rest]) -->
    translate_used2(Used),
    translate_used(Rest).
translate_used2(includeduse(Name,Id,NewTExpr)) -->
    indent,pp_expr_indent(NewTExpr),
    insertstr(' --> '), insertstr(Name), insertstr(':'), insertstr(Id).

% pretty-print the internal information about freetypes
translate_freetypes([]) --> !.
translate_freetypes([Freetype|Frest]) -->
    translate_freetype(Freetype),
    translate_freetypes(Frest).
translate_freetype(freetype(Name,Cases)) -->
    {pretty_freetype(Name,PName)},
    indent(PName),insertstr(': '),
    indention_level(I1,I2),{I2 is I1+2},
    translate_freetype_cases(Cases),
    indention_level(_,I1).
translate_freetype_cases([]) --> !.
translate_freetype_cases([case(Name,constant(_))|Rest]) -->
    !,indent(Name),translate_freetype_cases(Rest).
translate_freetype_cases([case(Name,Type)|Rest]) -->
    {pretty_type(Type,PT)},
    indent(Name),
    insertstr('('),insertstr(PT),insertstr(')'),
    translate_freetype_cases(Rest).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% substitutions

translate_subst_or_bexpr(Stmt,String) :- get_texpr_type(Stmt,subst),!,
   translate_substitution(Stmt,String).
translate_subst_or_bexpr(ExprOrPred,String) :-
    translate_bexpression(ExprOrPred,String).

translate_subst_or_bexpr_with_limit(Stmt,Limit,String) :-
   translate_subst_or_bexpr_with_limit(Stmt,Limit,report_errors,String).
translate_subst_or_bexpr_with_limit(Stmt,_Limit,ReportErrors,String) :- get_texpr_type(Stmt,subst),!,
   translate_substitution(Stmt,String,ReportErrors). % TO DO: use limit
translate_subst_or_bexpr_with_limit(ExprOrPred,Limit,ReportErrors,String) :-
    translate_bexpression_with_limit(ExprOrPred,Limit,ReportErrors,String).

print_subst(Stmt) :- translate_substitution(Stmt,T), print(T).
translate_substitution(Stmt,String) :- translate_substitution(Stmt,String,report_errors).
translate_substitution(Stmt,String,_) :- translate_subst_with_indention(Stmt,0,Codes,[]),
   (Codes = [10|C] -> true ; Codes=[13|C] -> true ; Codes=C), % peel off leading newline
   atom_codes_with_limit(String, C),!.
translate_substitution(Stmt,String,report_errors) :-
   add_error(translate_substitution,'Could not translate substitution: ',Stmt),String='???'.

translate_subst_with_indention(TS,Indention,I,O) :-
    translate_subst(TS,(Indention,I),(_,O)).

translate_subst(TS) -->
    ( {get_texpr_expr(TS,S)} ->
        %indent_rodin_label(TS), % comment in to pretty print substitution labels
        translate_subst2(S)
    ; {otherwise} -> translate_subst2(TS)).

% will print (first) rodin or pragma label indendent
:- public indent_rodin_label/3.
indent_rodin_label(_TExpr) --> {get_preference(translate_suppress_rodin_positions_flag,true),!}.
indent_rodin_label(_TExpr) --> {get_preference(bugly_pp_scrambling,true),!}.
indent_rodin_label(TExpr) --> {bsyntaxtree:get_texpr_labels(TExpr,Names)},!, % note: this will only get the first label
   indent('/* @'),pp_ids_indent(Names),insertstr('*/ '). % this syntax cannot be read back in
indent_rodin_label(_TExpr) --> [].

pp_ids_indent([]) --> !, [].
pp_ids_indent([ID]) --> !,pp_expr_indent(identifier(ID)).
pp_ids_indent([ID|T]) --> !,pp_expr_indent(identifier(ID)), insertstr(' '),pp_ids_indent(T).
pp_ids_indent(X) --> {add_error(pp_ids_indent,'Not a list of atoms: ',pp_ids_indent(X))}.

translate_subst2(skip) -->
    indent(skip).
translate_subst2(operation(Id,Res,Params,Body)) --> translate_operation(Id,Res,Params,Body). % not really a substition that can appear normally
translate_subst2(precondition(P,S)) -->
    indent('PRE '), pred_over_lines(2,'@grd',P), indent('THEN'), insert_subst(S), indent('END').
translate_subst2(assertion(P,S)) -->
    indent('ASSERT '), pp_expr_indent(P), indent('THEN'), insert_subst(S), indent('END').
translate_subst2(block(S)) -->
    indent('BEGIN'), insert_subst(S), indent('END').
translate_subst2(assign(L,R)) -->
    indent,pp_expr_indent_l(L),insertstr(' := '),pp_expr_indent_l(R).
translate_subst2(assign_single_id(L,R)) -->
    translate_subst2(assign([L],[R])).
translate_subst2(becomes_element_of(L,R)) -->
    indent,pp_expr_indent_l(L),insertstr(' :: '),pp_expr_indent(R).
translate_subst2(becomes_such(L,R)) -->
    indent,pp_expr_indent_l(L),insertstr(' : '), insertstr('('),
    { add_optional_typing_predicates(L,R,R1) },
    pp_expr_indent(R1), insertstr(')').
translate_subst2(evb2_becomes_such(L,R)) --> translate_subst2(becomes_such(L,R)).
translate_subst2(if([Elsif|Rest])) -->
    { get_if_elsif(Elsif,P,S) },
    indent('IF '), pp_expr_indent(P), insertstr(' THEN'),
    insert_subst(S),
    translate_ifs(Rest).
translate_subst2(choice(Ss)) --> indent(' CHOICE'),
    split_over_lines(Ss,'OR'),
    indent('END'). % indentation seems too far
translate_subst2(parallel(Ss)) -->
    split_over_lines(Ss,'||').
translate_subst2(init_statement(S)) --> insert_subst(S).
translate_subst2(sequence(Ss)) -->
    split_over_lines(Ss,';').
translate_subst2(operation_call(Id,Rs,As)) -->
    indent,translate_op_results(Rs),
    pp_expr_indent(Id),
    translate_op_params(As).
translate_subst2(identifier(op(Id))) --> % shouldn't normally appear
    indent,pp_expr_indent(identifier(Id)).
translate_subst2(external_subst_call(Symbol,Args)) -->
    indent,
    pp_expr_indent(identifier(Symbol)),
    translate_op_params(Args).
translate_subst2(any(Ids,Pred,Subst)) -->
    indent('ANY '), pp_expr_indent_l(Ids),
    indent('WHERE '),
    {add_optional_typing_predicates(Ids,Pred,Pred2)},
    pred_over_lines(2,'@grd',Pred2), indent('THEN'),
    insert_subst(Subst),
    indent('END').
translate_subst2(select(Whens)) -->
    translate_whens(Whens,'SELECT '),
    indent('END').
translate_subst2(select(Whens,Else)) -->
    translate_whens(Whens,'SELECT '),
    indent('ELSE'), insert_subst(Else),
    indent('END').
translate_subst2(var(Ids,S)) -->
    indent('VAR '),
    pp_expr_indent_l(Ids),
    indent('IN'),insert_subst(S),
    indent('END').
translate_subst2(let(Ids,P,S)) -->
    indent('LET '),
    pp_expr_indent_l(Ids),
    insertstr(' BE '), pp_expr_indent(P),
    indent('IN'), insert_subst(S),
    indent('END').
translate_subst2(lazy_let_subst(TID,P,S)) -->
    indent('LET '),
    pp_expr_indent_l([TID]),
    insertstr(' BE '), pp_expr_indent(P), % could be expr or pred
    indent('IN'), insert_subst(S),
    indent('END').
translate_subst2(case(Expression,Cases,ELSE)) -->
% CASE E OF EITHER m THEN G OR n THEN H ... ELSE I END END
    indent('CASE '),
    pp_expr_indent(Expression), insertstr(' OF'),
    indent('EITHER '), translate_cases(Cases),
    indent('ELSE '), insert_subst(ELSE), % we could drop this if ELSE is skip ?
    indent('END END').
translate_subst2(while(Pred,Subst,Inv,Var)) -->
    indent('WHILE '), pp_expr_indent(Pred),
    indent('DO'),insert_subst(Subst),
    indent('INVARIANT '),pp_expr_indent(Inv),
    indent('VARIANT '),pp_expr_indent(Var),
    indent('END').
translate_subst2(while1(Pred,Subst,Inv,Var)) -->
    indent('WHILE /* 1 */ '), pp_expr_indent(Pred),
    indent('DO'),insert_subst(Subst),
    indent('INVARIANT '),pp_expr_indent(Inv),
    indent('VARIANT '),pp_expr_indent(Var),
    indent('END').
translate_subst2(rlevent(Id,Section,Status,Parameters,Guard,Theorems,Actions,VWitnesses,PWitnesses,_Unmod,Refines)) -->
    indent,
    insert_status(Status),
    insertstr('EVENT '), insertstr(Id), insertstr(' = /'), insertstr('* of machine '),
    insertstr(Section),insertstr(' */'),
    insert_variant(Status),
    ( {Parameters=[], get_texpr_expr(Guard,truth)} ->
        {true}                  % indent('BEGIN ')
    ; {Parameters=[]} ->
        indent('WHEN '),
        pred_over_lines(2,'@grd',Guard)
    ; {otherwise} ->
        indent('ANY '),pp_expr_indent_l(Parameters),
        indent('WHERE '),
        pred_over_lines(2,'@grd',Guard) ),
    ( {VWitnesses=[],PWitnesses=[]} ->
        []
    ; {otherwise} ->
        {append(VWitnesses,PWitnesses,Witnesses)},
        indent('WITH '),pp_witness_l(Witnesses) ),
    {( Actions=[] ->
       create_texpr(skip,subst,[],Subst)
     ; otherwise ->
       create_texpr(parallel(Actions),subst,[],Subst))},
    ( {Theorems=[]} -> {true}
    ; {otherwise} ->
        indent('THEOREMS '),
        preds_over_lines(2,'@thm',Theorems)),
    indent('THEN '), insert_subst(Subst),
    pp_refines_l(Refines,Id),
    indent('END').

% translate cases of a CASE statement
translate_cases([]) --> !,[].
translate_cases([CaseOr|T]) -->
     {get_texpr_expr(CaseOr,case_or(Exprs,Subst))},!,
     pp_expr_indent_l(Exprs),
     insertstr(' THEN '),
     insert_subst(Subst),
     ({T==[]} -> {true}
      ; indent('OR '), translate_cases(T)).
translate_cases(L) -->
    {add_internal_error('Cannot translate CASE list: ',translate_cases(L,_,_))}.

insert_status(TStatus) -->
    {get_texpr_expr(TStatus,Status),
     status_string(Status,String)},
    insertstr(String).
status_string(ordinary,'').
status_string(anticipated(_),'ANTICIPATED ').
status_string(convergent(_),'CONVERGENT ').

insert_variant(TStatus) -->
    {get_texpr_expr(TStatus,Status)},
    insert_variant2(Status).
insert_variant2(ordinary) --> !.
insert_variant2(anticipated(Variant)) --> insert_variant3(Variant).
insert_variant2(convergent(Variant)) --> insert_variant3(Variant).
insert_variant3(Variant) -->
    indent('USING VARIANT '),pp_expr_indent(Variant).

pp_refines_l([],_) --> [].
pp_refines_l([Ref|Rest],Id) -->
    pp_refines(Ref,Id),pp_refines_l(Rest,Id).
pp_refines(Refined,_Id) -->
    % indent(Id), insertstr(' REFINES '),
    indent('REFINES '),
    insert_subst(Refined).

pp_witness_l([]) --> [].
pp_witness_l([Witness|WRest]) -->
        pp_witness(Witness),pp_witness_l(WRest).
pp_witness(Expr) -->
    indention_level(I1,I2),
    {get_texpr_expr(Expr,witness(Id,Pred)),
     I2 is I1+2},
    indent, pp_expr_indent(Id), insertstr(': '),
    pp_expr_indent(Pred),
    indention_level(_,I1).


translate_whens([],_) --> !.
translate_whens([When|Rest],T) -->
    {get_texpr_expr(When,select_when(P,S))},!,
    indent(T), pred_over_lines(2,'@grd',P),
    indent('THEN '),
    insert_subst(S),
    translate_whens(Rest,'WHEN ').
translate_whens(L,_) -->
    {add_internal_error('Cannot translate WHEN: ',translate_whens(L,_,_,_))}.

split_over_lines([],_) --> !.
split_over_lines([S|Rest],Symbol) --> !,
    indention_level(I1,I2),{atom_codes(Symbol,X),length(X,N),I2 is I1+N+1},
    translate_subst_check(S),
    split_over_lines2(Rest,Symbol,I1,I2).
split_over_lines(S,Symbol) --> {add_error(split_over_lines,'Illegal argument: ',Symbol:S)}.

split_over_lines2([],_,_,_) --> !.
split_over_lines2([S|Rest],Symbol,I1,I2) -->
    indention_level(_,I1), indent(Symbol),
    indention_level(_,I2), translate_subst(S),
    split_over_lines2(Rest,Symbol,I1,I2).

% print a predicate over several lines, at most one conjunct per line
%pred_over_lines(N,Pred) --> pred_over_lines(N,'@pred',Pred).
pred_over_lines(N,Lbl,Pred) -->
    {conjunction_to_list(Pred,List)},
    preds_over_lines(N,Lbl,List).
section_pred_over_lines(N,Title,Pred) -->
    ({get_eventb_default_label(Title,Lbl)} -> [] ; {Lbl='@pred'}),
    pred_over_lines(N,Lbl,Pred).
get_eventb_default_label(properties,'@axm').
get_eventb_default_label(assertions,'@thm').

% print a list of predicates over several lines, at most one conjunct per line
preds_over_lines(N,Lbl,Preds) --> preds_over_lines(N,Lbl,'& ',Preds).
% preds_over_lines(IndentationIncrease,EventBDefaultLabel,ClassicalBSeperator,ListOfPredicates)
preds_over_lines(N,Lbl,Sep,Preds) -->
    indention_level(I1,I2),{I2 is I1+N},
    preds_over_lines1(Preds,Lbl,1,Sep),
    indention_level(_,I1).
preds_over_lines1([],Lbl,Nr,Sep) --> !, preds_over_lines1([b(truth,pred,[])],Lbl,Nr,Sep).
preds_over_lines1([H|T],Lbl,Nr,Sep) -->
    indent('  '), pp_label(Lbl,Nr),
    %({T==[]} -> pp_expr_indent(H) ; pp_expr_m_indent(H,40)),
    ({T==[]} -> pp_pred_nested(H,conjunct,0) ; pp_pred_nested(H,conjunct,40)),
    {N1 is Nr+1},
    preds_over_lines2(T,Lbl,N1,Sep).
preds_over_lines2([],_,_,_Sep) --> !.
preds_over_lines2([E|Rest],Lbl,Nr,Sep) -->
    ({force_eventb_rodin_mode} -> indent('  '), pp_label(Lbl,Nr) ; indent(Sep)),
    pp_pred_nested(E,conjunct,40),
    {N1 is Nr+1},
    preds_over_lines2(Rest,Lbl,N1,Sep).

% print event-b label for Rodin/Camille:
pp_label(Lbl,Nr) -->
   ({force_eventb_rodin_mode}
     -> {atom_codes(Lbl,C1), number_codes(Nr,NC), append(C1,NC,AC), atom_codes(A,AC)},
        pp_atom_indent(A), pp_atom_indent(' ')
      ; []).

% a version of nested_print_bexpr / nbp that does not directly print to stream conjunct
pp_pred_nested(TExpr,CurrentType,_) --> {TExpr = b(E,pred,_)},
    {get_binary_op(E,Ascii,LHS,RHS), binary_infix(NewType,Ascii,Prio,_Assoc)},
    !,
    pp_rodin_label_indent(TExpr), % print any label
    inc_lvl(CurrentType,NewType),
    pp_pred_nested(LHS,NewType,Prio),
    {translate_in_mode(NewType,Ascii,Symbol)},
    indent('  '),pp_atom_indent(Symbol),
    indent('  '),
    pp_pred_nested(RHS,NewType,Prio),
    dec_lvl(CurrentType,NewType).
pp_pred_nested(TExpr,_,_) --> {is_nontrivial_negation(TExpr,NExpr,InnerType,Prio)},
    !,
    pp_rodin_label_indent(TExpr), % print any label
    {translate_in_mode(negation,'not',Symbol)},
    pp_atom_indent(Symbol),
    inc_lvl(other,negation), % always need parentheses for negation
    pp_pred_nested(NExpr,InnerType,Prio),
    dec_lvl(other,negation).
pp_pred_nested(Pred,_CurrentType,CurMinPrio) --> pp_expr_m_indent(Pred,CurMinPrio).

is_nontrivial_negation(b(negation(NExpr),pred,_),NExpr,NewType,Prio) :-
    get_texpr_expr(NExpr,E),
    (E=negation(_) -> NewType=other,Prio=0
    ; get_binary_op(E,Ascii,_,_), binary_infix(NewType,Ascii,Prio,_Assoc)).

pp_rodin_label_indent(b(_,_,Infos),(I,S),(I,T)) :- pp_rodin_label(Infos,S,T).
% note: below we will print unnecessary parentheses in case of Atelier-B mode; but for readability it maye be better to add them
inc_lvl(New,Old) --> {New=Old}, !,[].
inc_lvl(_,_) --> pp_atom_indent('('), % not strictly necessary if higher_prio
                 increase_indentation_level, indent('  ').
dec_lvl(New,Old) --> {New=Old}, !,[].
dec_lvl(_,_) --> decrease_indentation_level, indent('  '),pp_atom_indent(')').

%higher_prio(conjunct,implication).
%higher_prio(disjunct,implication).
% priority of equivalence changes in Rodin vs Atelier-B, maybe better add parentheses

translate_ifs([]) --> !,
    indent('END').
translate_ifs([Elsif]) -->
    {get_if_elsif(Elsif,P,S),
     optional_type(P,truth)},!,
    indent('ELSE'), insert_subst(S), indent('END').
translate_ifs([Elsif|Rest]) -->
    {get_if_elsif(Elsif,P,S)},!,
    indent('ELSIF '), pp_expr_indent(P), insertstr(' THEN'),
    insert_subst(S),
    translate_ifs(Rest).
translate_ifs(ElseList) -->
  {functor(ElseList,F,A),add_error(translate_ifs,'Could not translate IF-THEN-ELSE: ',F/A-ElseList),fail}.

get_if_elsif(Elsif,P,S) :-
  (optional_type(Elsif,if_elsif(P,S)) -> true
    ; add_internal_error('Is not an if_elsif:',get_if_elsif(Elsif,P,S)), fail).

insert_subst(S) -->
    indention_level(I,I2),{I2 is I+2},
    translate_subst_check(S),
    indention_level(_,I).

translate_subst_check(S) --> translate_subst(S),!.
translate_subst_check(S) -->
  {b_functor(S,F,A),add_error(translate_subst,'Could not translate substitution: ',F/A-S),fail}.

b_functor(b(E,_,_),F,A) :- !,functor(E,F,A).
b_functor(E,F,A) :- functor(E,F,A).

indent_expr(Expr) -->
    indent, pp_expr_indent(Expr).
%indent_expr_l([]) --> !.
%indent_expr_l([Expr|Rest]) -->
%    indent_expr(Expr), indent_expr_l(Rest).
indent_expr_l_sep([],_) --> !.
indent_expr_l_sep([Expr|Rest],Sep) -->
    indent_expr(Expr),
    {(Rest=[] -> RealSep='' ;  RealSep=Sep)},
    insert_atom(RealSep), % the threaded argument is a pair, not directly a string !
    indent_expr_l_sep(Rest,Sep).
%indention_level(L) --> indention_level(L,L).
increase_indentation_level --> indention_level(L,New), {New is L+1}.
decrease_indentation_level --> indention_level(L,New), {New is L-1}.
indention_level(Old,New,(Old,S),(New,S)).
indention_codes(Old,New,(Indent,Old),(Indent,New)).
indent --> indent('').
indent(M,(I,S),(I,T)) :- indent2(I,M,S,T).
indent2(Level,Msg) -->
    "\n",do_indention(Level),ppatom(Msg).

insert_atom(Sep,(I,S),(I,T)) :- ppatom(Sep,S,T).

insertstr(M,(I,S),(I,T)) :- ppterm(M,S,T).
insertcodes(M,(I,S),(I,T)) :- ppcodes(M,S,T).

do_indention(0,T,R) :- !, R=T.
do_indention(N,[32|I],O) :-
    N>0,N2 is N-1, do_indention(N2,I,O).

optional_type(Typed,Expr) :- get_texpr_expr(Typed,E),!,Expr=E.
optional_type(Expr,Expr).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% expressions and predicates

% pretty-type an expression in an indent-environment
% currently, the indent level is just thrown away
pp_expr_indent(Expr,(I,S),(I,T)) :-
    pp_expr(Expr,_,_LimitReached,S,T).
pp_expr_indent_l(Exprs,(I,S),(I,T)) :-
    pp_expr_l(Exprs,_LR,S,T).
pp_expr_m_indent(Expr,MinPrio,(I,S),(I,T)) :-
    pp_expr_m(Expr,MinPrio,_LimitReached,S,T).
pp_atom_indent(A,(I,S),(I,T)) :- ppatom(A,S,T).


is_boolean_value(b(B,boolean,_),BV) :- boolean_aux(B,BV).
boolean_aux(boolean_true,pred_true).
boolean_aux(boolean_false,pred_false).
boolean_aux(value(V),BV) :- nonvar(V),!,BV=V.


constants_in_mode(F,S) :-
   constants(F,S1), translate_in_mode(F,S1,S).

constants(pred_true,'TRUE').
constants(pred_false,'FALSE').
constants(boolean_true,'TRUE').
constants(boolean_false,'FALSE').
constants(max_int,'MAXINT').
constants(min_int,'MININT').
constants(empty_set,'{}').
constants(bool_set,'BOOL').
constants(string_set,'STRING').
constants(empty_sequence,'[]').
constants(event_b_identity,'id').

constants(truth,Res) :- eventb_translation_mode,!,Res=true.
constants(truth,Res) :- animation_minor_mode(tla),!,Res='TRUE'.
constants(truth,'(TRUE:BOOL)'). % __truth;  we could also do TRUE=TRUE
constants(falsity,Res) :- eventb_translation_mode,!,Res=false.
constants(falsity,Res) :- animation_minor_mode(tla),!,Res='FALSE'.
constants(falsity,'(TRUE=FALSE)'). % __falsity

function_like_in_mode(F,S) :-
   function_like(F,S1),
   translate_in_mode(F,S1,S).

function_like(convert_bool,bool).
function_like(successor,succ).
function_like(predecessor,pred).
function_like(max,max).
function_like(min,min).
function_like(card,card).
function_like(pow_subset,'POW').
function_like(pow1_subset,'POW1').
function_like(fin_subset,'FIN').
function_like(fin1_subset,'FIN1').
function_like(identity,id).
function_like(first_projection,prj1).
function_like(first_of_pair,'__first_of_pair').  % will be dealt with separately to generate parsable representation
function_like(second_projection,prj2).
function_like(second_of_pair,'__second_of_pair'). % will be dealt with separately to generate  parsable representation
function_like(iteration,iterate).
function_like(reflexive_closure,closure).
function_like(closure,closure1).
function_like(domain,dom).
function_like(range,ran).
function_like(seq,seq).
function_like(seq1,seq1).
function_like(iseq,iseq).
function_like(iseq1,iseq1).
function_like(perm,perm).
function_like(size,size).
function_like(first,first).
function_like(last,last).
function_like(front,front).
function_like(tail,tail).
function_like(rev,rev).
function_like(general_concat,conc).
function_like(general_union,union).
function_like(general_intersection,inter).
function_like(trans_function,fnc).
function_like(trans_relation,rel).
function_like(tree,tree).
function_like(btree,btree).
function_like(const,const).
function_like(top,top).
function_like(sons,sons).
function_like(prefix,prefix).
function_like(postfix,postfix).
function_like(sizet,sizet).
function_like(mirror,mirror).
function_like(rank,rank).
function_like(father,father).
function_like(son,son).
function_like(subtree,subtree).
function_like(arity,arity).
function_like(bin,bin).
function_like(left,left).
function_like(right,right).
function_like(infix,infix).

function_like(rec,rec).
function_like(struct,struct).

function_like(negation,not).
function_like(bag_items,items).

function_like(finite,finite). % from Event-B, TO DO: if \+ eventb_translation_mode then print as S:FIN(S)

unary_prefix(unary_minus,-,210).
unary_prefix(mu,'MU',210) :- animation_minor_mode(z).

unary_prefix_parentheses(compaction,'compaction').
unary_prefix_parentheses(bag_items,'bag_items').
unary_prefix_parentheses(mu,'MU') :- \+ animation_minor_mode(z). % write with () for external function

unary_postfix(reverse,'~',230). % relational inverse


always_surround_by_parentheses(parallel_product).
always_surround_by_parentheses(composition).

binary_infix_symbol(b(T,_,_),Symbol) :- functor(T,F,2), binary_infix_in_mode(F,Symbol,_,_).

% EXPR * EXPR --> EXPR
binary_infix(composition,';',20,left).
binary_infix(overwrite,'<+',160,left).
binary_infix(direct_product,'><',160,left).
binary_infix(parallel_product,'||',20,left).
binary_infix(concat,'^',160,left).
binary_infix(relations,'<->',125,left).
binary_infix(partial_function,'+->',125,left).
binary_infix(total_function,'-->',125,left).
binary_infix(partial_injection,'>+>',125,left).
binary_infix(total_injection,'>->',125,left).
binary_infix(partial_surjection,'+->>',125,left).
binary_infix(total_surjection,Symbol,125,left) :-
  (eventb_translation_mode -> Symbol = '->>'; Symbol = '-->>').
binary_infix(total_bijection,'>->>',125,left).
binary_infix(partial_bijection,'>+>>',125,left).
binary_infix(total_relation,'<<->',125,left).             % only in Event-B
binary_infix(surjection_relation,'<->>',125,left).        % only in Event-B
binary_infix(total_surjection_relation,'<<->>',125,left). % only in Event-B
binary_infix(insert_front,'->',160,left).
binary_infix(insert_tail,'<-',160,left).
binary_infix(domain_restriction,'<|',160,left).
binary_infix(domain_subtraction,'<<|',160,left).
binary_infix(range_restriction,'|>',160,left).
binary_infix(range_subtraction,'|>>',160,left).
binary_infix(intersection,'/\\',160,left).
binary_infix(union,'\\/',160,left).
binary_infix(restrict_front,'/|\\',160,left).
binary_infix(restrict_tail,'\\|/',160,left).
binary_infix(couple,'|->',160,left).
binary_infix(interval,'..',170,left).
binary_infix(add,+,180,left).
binary_infix(minus,-,180,left).
binary_infix(set_subtraction,'\\',180,left) :- eventb_translation_mode,!.
binary_infix(set_subtraction,-,180,left).
binary_infix(minus_or_set_subtract,-,180,left).
binary_infix(multiplication,*,190,left).
binary_infix(cartesian_product,**,190,left) :- eventb_translation_mode,!.
binary_infix(cartesian_product,*,190,left).
binary_infix(mult_or_cart,*,190,left). % in case type checker not yet run
binary_infix(div,/,190,left).
binary_infix(floored_div,div,190,left).
binary_infix(modulo,mod,190,left).
binary_infix(power_of,**,200,right).


% PRED * PRED --> PRED
binary_infix(implication,'=>',30,left).
binary_infix(conjunct,'&',40,left).
binary_infix(disjunct,or,40,left).
binary_infix(equivalence,'<=>',Prio,left) :- % in Rodin this has the same priority as implication
   (eventb_translation_mode -> Prio=30 ; Prio=60).


% EXPR * EXPR --> PRED
binary_infix(equal,=,60,left).
binary_infix(not_equal,'/=',160,left).
binary_infix(less_equal,'<=',160,left).
binary_infix(less,'<',160,left).
binary_infix(greater_equal,'>=',160,left).
binary_infix(greater,'>',160,left).
binary_infix(member,':',60,left).
binary_infix(not_member,'/:',160,left).
binary_infix(subset,'<:',110,left).
binary_infix(subset_strict,'<<:',110,left).
binary_infix(not_subset,'/<:',110,left).
binary_infix(not_subset_strict,'/<<:',110,left).

binary_infix(values_entry,'=',60,left).

% atelierb_mode(pp): translation for AtelierB's PP/ML prover
% atelierb_mode(native): translation to native B supported by AtelierB
:- dynamic latex_mode/0, unicode_mode/0, atelierb_mode/1, force_eventb_rodin_mode/0.

%latex_mode.
%unicode_mode.
%force_eventb_rodin_mode. % Force Event-B output even if not in eventb minor mode

eventb_translation_mode :- animation_minor_mode(eventb),!.
eventb_translation_mode :- force_eventb_rodin_mode.

set_force_eventb_mode :- assert(force_eventb_rodin_mode).
unset_force_eventb_mode :-
  (retract(force_eventb_rodin_mode) -> true ; add_internal_error('Was not in forced Event-B mode: ',force_eventb_rodin_mode)).

set_unicode_mode :- assert(unicode_mode).
set_latex_mode :- assert(latex_mode).
unset_unicode_mode :-
  (retract(unicode_mode) -> true ; add_internal_error('Was not in Unicode mode: ',unset_unicode_mode)).
unset_latex_mode :-
  (retract(latex_mode) -> true
   ; add_internal_error('Was not in Latex mode: ',unset_latex_mode)).

% TO DO: provide better stack-based setting/unsetting of modes or use options parameter
set_translation_mode(ascii) :- !,retractall(unicode_mode), retractall(latex_mode).
set_translation_mode(unicode) :- !, set_unicode_mode.
set_translation_mode(latex) :- !, set_latex_mode.
set_translation_mode(Mode) :- add_internal_error('Illegal mode:',set_translation_mode(Mode)).

%get_translation_mode(M) :- unicode_mode,!, M=unicode_mode.
%get_translation_mode(M) :- latex_mode,!, M=latex_mode.
%get_translation_mode(ascii).

unset_translation_mode(ascii) :- !.
unset_translation_mode(unicode) :- !,unset_unicode_mode.
unset_translation_mode(latex) :- !,unset_latex_mode.
unset_translation_mode(Mode) :- add_internal_error('Illegal mode:',unset_translation_mode(Mode)).


exists_symbol --> {latex_mode},!, "\\exists ".
exists_symbol --> {unicode_mode},!, [8707].
exists_symbol --> "#".
forall_symbol --> {latex_mode},!, "\\forall ".
forall_symbol --> {unicode_mode},!, [8704].
forall_symbol --> "!".
dot_symbol --> {latex_mode},!, "\\cdot ".
dot_symbol --> {unicode_mode},!, [183]. %"·". % dot also used in Rodin
dot_symbol --> ".".
set_brackets(X,Y) :- latex_mode,!,X='\\{', Y='\\}'.
set_brackets('{','}').
left_set_bracket --> {latex_mode},!, "\\{ ".
left_set_bracket --> "{".
right_set_bracket --> {latex_mode},!, "\\} ".
right_set_bracket --> "}".
maplet_symbol --> {latex_mode},!, "\\mapsto ".
maplet_symbol --> {unicode_mode},!, [8614].
maplet_symbol --> "|->".

lambda_symbol --> {unicode_mode},!, [955].  % '\x3BB\'
lambda_symbol --> {latex_mode},!, "\\lambda ".
lambda_symbol --> "%".

and_symbol --> {unicode_mode},!, [8743].  % ''\x2227\''
and_symbol --> {latex_mode},!, "\\wedge ".
and_symbol --> "&".

hash_card_symbol --> {latex_mode},!, "\\# ".
hash_card_symbol --> "#".
ldots --> {latex_mode},!, "\\ldots ".
ldots --> "...".

empty_set_symbol --> {get_preference(translate_print_all_sequences,true)},!, pp_empty_sequence.
empty_set_symbol --> {unicode_mode},!, [8709].
empty_set_symbol --> {latex_mode},!, "\\emptyset ".
empty_set_symbol --> "{}".

underscore_symbol --> {latex_mode},!, "\\_".
underscore_symbol --> "_".

string_start_symbol  --> {latex_mode},!, "\\textnormal{``".
string_start_symbol  --> """".
string_end_symbol  --> {latex_mode},!, "''}".
string_end_symbol  --> """".


unary_postfix_in_mode(Op,Trans2,Prio) :-
    unary_postfix(Op,Trans,Prio), % print(op(Op,Trans)),nl,
    translate_in_mode(Op,Trans,Trans2).

binary_infix_in_mode(Op,Trans2,Prio,Assoc) :-
    binary_infix(Op,Trans,Prio,Assoc), % print(op(Op,Trans)),nl,
    translate_in_mode(Op,Trans,Trans2).

latex_integer_set_translation('NATURAL', '\\mathbb N ').   % \nat in bsymb.sty
latex_integer_set_translation('NATURAL1', '\\mathbb N_1 '). % \natn
latex_integer_set_translation('INTEGER', '\\mathbb Z '). % \intg

latex_translation(empty_set, '\\emptyset ').
latex_translation(implication, '\\mathbin\\Rightarrow ').
latex_translation(conjunct,'\\wedge ').
latex_translation(disjunct,'\\vee ').
latex_translation(equivalence,'\\mathbin\\Leftrightarrow ').
latex_translation(negation,'\\neg ').
latex_translation(not_equal,'\\neq ').
latex_translation(less_equal,'\\leq ').
latex_translation(greater_equal,'\\geq ').
latex_translation(member,'\\in ').
latex_translation(not_member,'\\not\\in ').
latex_translation(subset,'\\subseteq ').
latex_translation(subset_strict,'\\subset ').
latex_translation(not_subset,'\\not\\subseteq ').
latex_translation(not_subset_strict,'\\not\\subset ').
latex_translation(union,'\\cup ').
latex_translation(intersection,'\\cap ').
latex_translation(couple,'\\mapsto ').
latex_translation(cartesian_product,'\\times').
latex_translation(rec,'\\mathit{rec}').
latex_translation(struct,'\\mathit{struct}').
latex_translation(convert_bool,'\\mathit{bool}').
latex_translation(max,'\\mathit{max}').
latex_translation(min,'\\mathit{min}').
latex_translation(modulo,'\\mod ').
latex_translation(card,'\\mathit{card}').
latex_translation(successor,'\\mathit{succ}').
latex_translation(predecessor,'\\mathit{pred}').
latex_translation(domain,'\\mathit{dom}').
latex_translation(range,'\\mathit{ran}').
latex_translation(size,'\\mathit{size}').
latex_translation(first,'\\mathit{first}').
latex_translation(last,'\\mathit{last}').
latex_translation(front,'\\mathit{front}').
latex_translation(tail,'\\mathit{tail}').
latex_translation(rev,'\\mathit{rev}').
latex_translation(seq,'\\mathit{seq}').
latex_translation(seq1,'\\mathit{seq}_{1}').
latex_translation(perm,'\\mathit{perm}').
latex_translation(fin_subset,'\\mathit{FIN}').
latex_translation(fin1_subset,'\\mathit{FIN}_{1}').
latex_translation(first_projection,'\\mathit{prj}_{1}').
latex_translation(second_projection,'\\mathit{prj}_{2}').
latex_translation(pow_subset,'\\mathbb P\\hbox{}'). % POW  \pow would require bsymb.sty
latex_translation(pow1_subset,'\\mathbb P_1'). % POW1  \pown would require bsymb.sty
latex_translation(concat,'\\hat{~}').
latex_translation(relations,'\\mathbin\\leftrightarrow'). % <->, \rel requires bsymb.sty
latex_translation(total_relation,'\\mathbin{\\leftarrow\\mkern-14mu\\leftrightarrow}'). % <<-> \trel requires bsymb.sty
latex_translation(total_surjection_relation,'\\mathbin{\\leftrightarrow\\mkern-14mu\\leftrightarrow}'). % <<->>  \strel requires bsymb.sty
latex_translation(surjection_relation,'\\mathbin{\\leftrightarrow\\mkern-14mu\\rightarrow}'). % <->>  \srel requires bsymb.sty
latex_translation(partial_function,'\\mathbin{\\mkern6mu\\mapstochar\\mkern-6mu\\rightarrow}'). % +->  \pfun requires bsymb.sty, but \mapstochar is not supported by Mathjax
latex_translation(partial_injection,'\\mathbin{\\mkern9mu\\mapstochar\\mkern-9mu\\rightarrowtail}'). % >+>  \pinj requires bsymb.sty
latex_translation(partial_surjection,'\\mathbin{\\mkern6mu\\mapstochar\\mkern-6mu\\twoheadrightarrow}'). % >+>  \psur requires bsymb.sty
latex_translation(total_function,'\\mathbin\\rightarrow'). % -->  \tfun would require bsymb.sty
latex_translation(total_surjection,'\\mathbin\\twoheadrightarrow'). % -->>  \tsur requires bsymb.sty
latex_translation(total_injection,'\\mathbin\\rightarrowtail'). % >->  \tinj requires bsymb.sty
latex_translation(total_bijection,'\\mathbin{\\rightarrowtail\\mkern-18mu\\twoheadrightarrow}'). % >->>  \tbij requires bsymb.sty
latex_translation(domain_restriction,'\\mathbin\\lhd'). % <|  domres requires bsymb.sty
latex_translation(range_restriction,'\\mathbin\\rhd'). % |>  ranres requires bsymb.sty
latex_translation(domain_subtraction,'\\mathbin{\\lhd\\mkern-14mu-}'). % <<|  domsub requires bsymb.sty
latex_translation(range_subtraction,'\\mathbin{\\rhd\\mkern-14mu-}'). % |>>  ransub requires bsymb.sty
latex_translation(overwrite,'\\mathbin{\\lhd\\mkern-9mu-}'). % <+  \ovl requires bsymb.sty
latex_translation(general_sum,'\\Sigma ').
latex_translation(general_product,'\\Pi ').
latex_translation(lambda,'\\lambda ').
latex_translation(quantified_union,'\\bigcup\\nolimits'). % \Union requires bsymb.sty
latex_translation(quantified_intersection,'\\bigcap\\nolimits'). % \Inter requires bsymb.sty
%latex_translation(truth,'\\top').
%latex_translation(falsity,'\\bot').
latex_translation(truth,'{\\color{olive} \\top}'). % requires \usepackage{xcolor} in Latex
latex_translation(falsity,'{\\color{red} \\bot}').
latex_translation(boolean_true,'{\\color{olive} \\mathit{TRUE}}').
latex_translation(boolean_false,'{\\color{red} \\mathit{FALSE}}').
latex_translation(pred_true,'{\\color{olive} \\mathit{TRUE}}').
latex_translation(pred_false,'{\\color{red} \\mathit{FALSE}}').
latex_translation(reverse,'^{-1}').

unicode_translation(implication, '\x21D2\').
unicode_translation(conjunct,'\x2227\').
unicode_translation(disjunct,'\x2228\').
unicode_translation(negation,'\xAC\').
unicode_translation(equivalence,'\x21D4\').
unicode_translation(not_equal,'\x2260\').
unicode_translation(less_equal,'\x2264\').
unicode_translation(greater_equal,'\x2265\').
unicode_translation(member,'\x2208\').
unicode_translation(not_member,'\x2209\').
unicode_translation(subset,'\x2286\').
unicode_translation(subset_strict,'\x2282\').
unicode_translation(not_subset,'\x2288\').
unicode_translation(not_subset_strict,'\x2284\').
unicode_translation(union,'\x222A\').
unicode_translation(intersection,'\x2229\').
unicode_translation(cartesian_product,'\xD7\').
unicode_translation(couple,'\x21A6\').
unicode_translation(div,'\xF7\').
%unicode_translation(floored_div,'\xF7\'). % should we provide another Unicode character here ?
unicode_translation(power_of,'\x2191\'). % does not exist in Rodin ?? upwards arrow used; find better symbol
unicode_translation(interval,'\x2025\').
unicode_translation(domain_restriction,'\x25C1\').
unicode_translation(domain_subtraction,'\x2A64\').
unicode_translation(range_restriction,'\x25B7\').
unicode_translation(range_subtraction,'\x2A65\').
unicode_translation(relations,'\x2194\').
unicode_translation(partial_function,'\x21F8\').
unicode_translation(total_function,'\x2192\').
unicode_translation(partial_injection,'\x2914\').
unicode_translation(partial_surjection,'\x2900\').
unicode_translation(total_injection,'\x21A3\').
unicode_translation(total_surjection,'\x21A0\').
unicode_translation(total_bijection,'\x2916\').
unicode_translation('INTEGER','\x2124\').
unicode_translation('NATURAL','\x2115\').
unicode_translation('NATURAL1','\x2115\\x2081\'). % \x2115\ is subscript 1
unicode_translation(pow_subset,'\x2119\').
unicode_translation(pow1_subset,'\x2119\\x2081\'). % \x2115\ is subscript 1
unicode_translation(lambda,'\x3BB\').
unicode_translation(general_product,'\x220F\').
unicode_translation(general_sum,'\x2211\').
unicode_translation(empty_set,'\x2205\').
unicode_translation(truth,'\x22A4\'). % 8868 in decimal
unicode_translation(falsity,'\x22A5\'). % 8869 in decimal
unicode_translation(direct_product,'\x2297\').
unicode_translation(parallel_product,'\x2225\').
unicode_translation(reverse,'\x207B\\xB9\') :- \+ force_eventb_rodin_mode. % the one ¹ is ASCII 185
% this symbol is not accepted by Rodin
% unicode_translation(infinity,'\x221E\'). % 8734 in decimal
unicode_translation(concat,'\x2312\').
unicode_translation(insert_front,'\x21FE\').
unicode_translation(insert_tail,'\x21FD\').
unicode_translation(restrict_front,'\x2191\').
unicode_translation(restrict_tail,'\x2192\').
unicode_translation(forall, '\x2200\'). % usually forall_symbol used
unicode_translation(exists, '\x2203\'). % usually exists_symbol used

atelierb_pp_translation(set_minus,'_moinsE').
atelierb_pp_translation(cartesian_product,'_multE').
atelierb_pp_translation('INTEGER','INTEGER').
atelierb_pp_translation('INT','INTEGER').
atelierb_pp_translation('NATURAL','NATURAL').
atelierb_pp_translation('NAT','NATURAL').
atelierb_pp_translation(boolean_true,btrue).
atelierb_pp_translation(boolean_false,bfalse).
atelierb_pp_translation(truth,'TRUE').
atelierb_pp_translation(falsity,'FALSE').

quantified_in_mode(F,S) :-
   quantified(F,S1), translate_in_mode(F,S1,S).

translate_in_mode(F,S1,S) :-
    ( unicode_mode, unicode_translation(F,S) -> true
    ; latex_mode, latex_translation(F,S) -> true
    ; atelierb_mode(Mode), Mode == pp, atelierb_pp_translation(F,S) -> true
    ; S1=S).

quantified(general_sum,'SIGMA').
quantified(general_product,'PI').
quantified(quantified_union,'UNION').
quantified(quantified_intersection,'INTER').
quantified(lambda,X) :- atom_codes(X,[37]).


translate_prolog_constructor(C,R) :- unary_prefix(C,R,_),!.
translate_prolog_constructor(C,R) :- unary_postfix(C,R,_),!.
translate_prolog_constructor(C,R) :- binary_infix_in_mode(C,R,_,_),!.
translate_prolog_constructor(C,R) :- function_like_in_mode(C,R),!.
translate_prolog_constructor(C,R) :- constants_in_mode(C,R),!.
translate_prolog_constructor(C,R) :- quantified_in_mode(C,R),!.

% translate the Prolog constuctor of an AST node into a form for printing to the user
translate_prolog_constructor_in_mode(Constructor,Result) :-
  unicode_mode,
  unicode_translation(Constructor,Unicode),!, Result=Unicode.
translate_prolog_constructor_in_mode(Constructor,Result) :-
  latex_mode,
  latex_translation(Constructor,Latex),!, Result=Latex.
translate_prolog_constructor_in_mode(C,R) :- translate_prolog_constructor(C,R).

translate_subst_or_bexpr_in_mode(latex,TExpr,String) :- !, assert(latex_mode),
   call_cleanup(translate_subst_or_bexpr(TExpr,String), (retract(latex_mode)->true)).
translate_subst_or_bexpr_in_mode(unicode,TExpr,String) :- !, assert(unicode_mode),
   call_cleanup(translate_subst_or_bexpr(TExpr,String), (retract(unicode_mode)->true)).
translate_subst_or_bexpr_in_mode(atelierb_pp,TExpr,String) :- !, assert(atelierb_mode(pp)),
   call_cleanup(translate_subst_or_bexpr(TExpr,String), (retract(atelierb_mode(_))->true)).
translate_subst_or_bexpr_in_mode(Mode,TExpr,String) :-
   (Mode==ascii -> true ; add_internal_error('Illegal mode: ',translate_subst_or_bexpr_in_mode(Mode,TExpr,String))),
   translate_subst_or_bexpr(TExpr,String).


translate_bexpression_to_unicode(TExpr,String) :-
    set_unicode_mode,
    call_cleanup(translate_bexpression(TExpr,String),unset_unicode_mode).

translate_bexpression(TExpr,String) :-
    (pp_expr(TExpr,String) -> true
     ; (add_error(translate_bexpression,'Could not translate bexpression: ',TExpr),String='???')).

translate_bexpression_to_codes(TExpr,Codes) :-
    reset_pp,
    pp_expr(TExpr,_,_LimitReached,Codes,[]).

pp_expr(TExpr,String) :-
    translate_bexpression_to_codes(TExpr,Codes),
    atom_codes_with_limit(String, Codes).

translate_bexpression_with_limit(T,S) :- translate_bexpression_with_limit(T,200,report_errors,S).
translate_bexpression_with_limit(TExpr,Limit,String) :-
    translate_bexpression_with_limit(TExpr,Limit,report_errors,String).
translate_bexpression_with_limit(TExpr,Limit,ReportErrors,String) :-
    (catch_call(pp_expr_with_limit(TExpr,Limit,String)) -> true
     ; (ReportErrors=report_errors,
        add_error(translate_bexpression,'Could not translate bexpression: ',TExpr),String='???')).

pp_expr_with_limit(TExpr,Limit,String) :-
    set_up_limit_reached(Codes,Limit,LimitReached),
    reset_pp,
    pp_expr(TExpr,_,LimitReached,Codes,[]),
    atom_codes_with_limit(String, Limit, Codes).



% pretty-type an expression, if the expression has a priority >MinPrio, parenthesis
% can be ommitted, if not the expression has to be put into parenthesis
pp_expr_m(TExpr,MinPrio,LimitReached,S,Srest) :-
    add_outer_paren(Prio,MinPrio,S,Srest,X,Xrest), % use co-routine to instantiate S as soon as possible
    pp_expr(TExpr,Prio,LimitReached,X,Xrest).

:- block add_outer_paren(-,?,?,?,?,?).
add_outer_paren(Prio,MinPrio,S,Srest,X,Xrest) :-
    ( Prio > MinPrio -> % was >=, but problem with & / or with same priority  or with non associative operators !
        S=X, Srest=Xrest
    ; otherwise ->
        [Open] = "(", [Close] = ")",
        S = [Open|X], Xrest = [Close|Srest]).


:- use_module(translate_keywords,[classical_b_keyword/1, translate_keyword_id/2]).
translated_identifier('_zzzz_binary',R) :- !,
  (latex_mode -> R='z''''' ; R='z__'). % TO DO: could cause clash with user IDs
translated_identifier('_zzzz_unary',R) :- !,
  (latex_mode -> R='z''' ; R='z_'). % TO DO: ditto
translated_identifier('__RANGE_LAMBDA__',R) :- !,
  (latex_mode -> R='\\lambda\'' ; R = 'RANGE_LAMBDA__'). %ditto, could clash with user IDs !!
% TO DO: do we need to treat _prj_arg1__, _prj_arg2__, _lambda_result_ here ?
translated_identifier(ID,Res) :- preference(bugly_pp_scrambling,true),
   !,
   bugly_scramble_id(ID,Res).
translated_identifier(NameWithDots,R) :-
    get_preference(translate_ids_to_parseable_format,true),
    atom_codes(NameWithDots,Codes),[Dot]=".",
    memberchk(Dot,Codes),!,
    replace_codes(Codes,Dot,"__",NewCodes,[]),
    atom_codes(R,NewCodes).
translated_identifier(NameStartingWithUnderscores,R) :-
    get_preference(translate_ids_to_parseable_format,true),
    atom_codes(NameStartingWithUnderscores,Codes),[Underscore]="_",
    Codes = [Underscore,Underscore|NewCodes],!,
    atom_codes(R,NewCodes).
translated_identifier(ID,R) :- atom(ID),
    get_preference(translate_ids_to_parseable_format,true),
    \+ eventb_translation_mode,
    % most useful when converting Event-B to classical B
    classical_b_keyword(ID),
    !,
    translate_keyword_id(ID,R).
translated_identifier(ID,Result) :- %print_message(translate(ID)),
    latex_mode,!,
    my_greek_latex_escape_atom(ID,Greek,Res),
    (Greek=greek -> Result = Res ; ajoin(['\\mathit{',Res,'}'],Result)).
translated_identifier(X,X).

:- use_module(tools,[latex_escape_atom/2]).

greek_symbol('Delta').
greek_symbol('Gamma').
greek_symbol('Lambda').
greek_symbol('Phi').
greek_symbol('Pi').
greek_symbol('Psi').
greek_symbol('Rho').
greek_symbol('Omega').
greek_symbol('Sigma').
greek_symbol('Theta').
greek_symbol('Upsilon').
greek_symbol('Xi').
greek_symbol('alpha').
greek_symbol('beta').
greek_symbol('delta').
greek_symbol('chi').
greek_symbol('epsilon').
greek_symbol('eta').
greek_symbol('gamma').
greek_symbol('kappa').
greek_symbol('lambda').
greek_symbol('mu').
greek_symbol('nu').
greek_symbol('phi').
greek_symbol('psi').
greek_symbol('rho').
greek_symbol('sigma').
greek_symbol('tau').
greek_symbol('theta').
greek_symbol('upsilon').
greek_symbol('varepsilon').
greek_symbol('varphi').
greek_symbol('varpi').
greek_symbol('varrho').
greek_symbol('varsigma').
greek_symbol('vartheta').
greek_symbol('xi').
greek_symbol('zeta').
% other Latex math symbols
greek_symbol('vdash').
greek_symbol('models').


my_greek_latex_escape_atom(A,greek,Res) :- greek_symbol(A),get_preference(latex_pp_greek_ids,true),!,
   atom_concat('\\',A,Res).
my_greek_latex_escape_atom(A,no_greek,EA) :- latex_escape_atom(A,EA).

% ppatom + scramble if BUGYLY is TRUE
ppatom_opt_scramble(Name) --> {get_preference(bugly_pp_scrambling,true)},!,
   {bugly_scramble_id(Name,ScrName)},
   ppatom(ScrName).
ppatom_opt_scramble(Name) --> pp_atom_opt_latex(Name).

:- use_module(tools,[b_escape_string_atom/2]).
% a version of ppatom which encodes/quotes symbols inside strings such as quotes "
ppstring_opt_scramble(Name) --> {var(Name)},!,ppatom(Name).
ppstring_opt_scramble(Name) --> {get_preference(bugly_pp_scrambling,true)},!,
   {bugly_scramble_id(Name,ScrName)},
   ppatom(ScrName).
ppstring_opt_scramble(Name) --> {b_escape_string_atom(Name,EscName)},
   pp_atom_opt_latex(EscName).

pp_atom_opt_latex(Name) --> {latex_mode},!,
   {my_greek_latex_escape_atom(Name,_,EscName)},
   % should we add  \mathrm{.} or \mathit{.}?
   ppatom(EscName).
pp_atom_opt_latex(Name) --> ppatom(Name).

pp_atom_opt_latex_mathit(Name) --> {latex_mode},!,
   {latex_escape_atom(Name,EscName)},
   "\\mathit{",ppatom(EscName),"}".
pp_atom_opt_latex_mathit(Name) --> ppatom(Name).

pp_atom_opt_mathit(EscName) --> {latex_mode},!,
   % we assume already escaped
   "\\mathit{",ppatom(EscName),"}".
pp_atom_opt_mathit(Name) --> ppatom(Name).

pp_space --> {latex_mode},!, "\\ ".
pp_space --> " ".

opt_scramble_id(ID,Res) :- get_preference(bugly_pp_scrambling,true),!,
   bugly_scramble_id(ID,Res).
opt_scramble_id(ID,ID).

:- use_module(probsrc(gensym),[gensym/2]).
:- dynamic bugly_scramble_id_cache/2.
bugly_scramble_id(ID,Res) :- var(ID),!, add_internal_error('Illegal call: ',bugly_scramble_id(ID,Res)), ID=Res.
bugly_scramble_id(ID,Res) :- bugly_scramble_id_cache(ID,ScrambledID),!,
   Res=ScrambledID.
bugly_scramble_id(ID,Res) :- %print(gen_id(ID,Res)),nl,
   genbuglynr(Nr), ajoin(['aa',Nr],ScrambledID),
   assert(bugly_scramble_id_cache(ID,ScrambledID)),
   Res = ScrambledID.

:- dynamic bugly_count/1.
bugly_count(0).
genbuglynr(Nr) :-
    retract(bugly_count(Nr)), N1 is Nr + 1,
    assert(bugly_count(N1)).

replace_codes([],_,_) --> !.
replace_codes([C|Rest],S,N) --> replace_codes2(C,S,N),replace_codes(Rest,S,N).
replace_codes2(S,S,N,I,O) :- !,append(N,O,I).
replace_codes2(S,_,_) --> [S].

is_lambda_result_id(b(identifier(ID),_,_INFO),Suffix) :-  % _INFO=[lambda_result], sometiems _INFO=[]
    is_lambda_result_name(ID,Suffix).
is_lambda_result_name(LAMBDA_RESULT,Suffix) :- atomic(LAMBDA_RESULT),
   atom_codes(LAMBDA_RESULT,[95,108,97,109,98,100,97,95,114,101,115,117,108,116,95|Suffix]). % _lambda_result_

pp_expr(TE,P) --> %{print('OBSOLETE'),nl,nl},
                  pp_expr(TE,P,_LimitReached).

pp_expr(TExpr,Prio,_) --> {var(TExpr)},!,"_",{Prio=500}.
pp_expr(_,Prio,LimitReached) --> {LimitReached==limit_reached},!,"...",{Prio=500}.
pp_expr(b(Expr,Type,Info),Prio,LimitReached) --> !,
    pp_expr0(Expr,Type,Info,Prio,LimitReached).
pp_expr(Expr,Prio,LimitReached) -->
    pp_expr1(Expr,any,[],Prio,LimitReached).

pp_expr0(identifier(ID),_Type,_Info,Prio,_LimitReached) --> {is_lambda_result_name(ID,Suffix)},!, {Prio=500},
    {append("LAMBDA_RESULT___",Suffix,ASCII), atom_codes(R,ASCII)}, ppatom(R).
pp_expr0(Expr,_Type,Info,Prio,_LimitReached) -->
    {eventb_translation_mode},
    pp_theory_operator(Expr,Info,Prio),!.
pp_expr0(Expr,Type,Info,Prio,LimitReached) -->
    {check_info(Expr,Type,Info)},
    pp_rodin_label(Info),
    (pp_expr1(Expr,Type,Info,Prio,LimitReached) -> {true}
      ; {add_error(translate,'Could not translate:',Expr,Expr),fail}
     ).

check_info(Expr,_,Info)  :- var(Info), add_error(translate,'Illegal info field for expression: ', Expr),fail.
check_info(_,_,_).

pp_theory_operator(general_sum(_,Membercheck,_),_Info,500) -->
    {get_texpr_expr(Membercheck,member(_,Arg))},
    ppatom('SUM('),pp_expr(Arg,_),ppatom(')').
pp_theory_operator(general_product(_,Membercheck,_),_Info,500) -->
    {get_texpr_expr(Membercheck,member(_Couple,Arg))},
    ppatom('PRODUCT('),pp_expr(Arg,_),ppatom(')').
pp_theory_operator(function(_,Arg),Info,500) -->
    {memberchk_in_info(theory_operator(O,N),Info),decouplise_expr(N,Arg,Args)},
    ppatom(O),"(",pp_expr_l_sep(Args,",",_LR),")".
pp_theory_operator(member(Arg,_),Info,500) -->
    {memberchk_in_info(theory_operator(O,N),Info),decouplise_expr(N,Arg,Args)},
    ppatom(O),"(",pp_expr_l_sep(Args,",",_LR),")".

decouplise_expr(1,E,R) :- !,R=[E].
decouplise_expr(N,E,R) :-
    get_texpr_expr(E,couple(A,B)),!,
    N2 is N-1,
    decouplise_expr(N2,A,R1),append(R1,[B],R).
decouplise_expr(N,E,[E]) :-
    print_message(call_failed(decouplise_expr(N,E,_))),nl.

% will pretty print (first) rodin or pragma label
pp_rodin_label(_Infos) --> {preference(translate_suppress_rodin_positions_flag,true),!}.
pp_rodin_label(_Infos) --> {preference(bugly_pp_scrambling,true),!}.
pp_rodin_label(Infos) --> {bsyntaxtree:get_info_labels(Infos,Label)},!,
       pp_start_label_pragma,
       ppatoms_opt_latex(Label),
       pp_end_label_pragma.
pp_rodin_label(_Infos) --> [].

pp_start_label_pragma --> {atelierb_mode(Mode), Mode == pp},!, "/*@label ".
pp_start_label_pragma --> "/* @". % shorter version, for viewing in UI
pp_end_label_pragma --> " */ ".

ppatoms([]) --> !, [].
ppatoms([ID|T]) --> !,ppatom(ID), " ", ppatoms(T).
ppatoms(X) --> {add_error(ppatoms,'Not a list of atoms: ',ppatoms(X))}.

ppatoms_opt_latex([]) --> !, [].
ppatoms_opt_latex([ID]) --> !,pp_atom_opt_latex(ID).
ppatoms_opt_latex([ID|T]) --> !,pp_atom_opt_latex(ID), " ", ppatoms_opt_latex(T).
ppatoms_opt_latex(X) --> {add_error(ppatoms_opt_latex,'Not a list of atoms: ',ppatoms_opt_latex(X))}.

:- use_module(bsyntaxtree,[is_set_type/2]).
:- load_files(library(system), [when(compile_time), imports([environ/2])]).
pp_expr1(Expr,_,_,Prio,_) --> {var(Expr)},!,"_",{Prio=500}.
pp_expr1(event_b_comprehension_set(Ids,E,P),Type,_Info,Prio,LimitReached) -->
    {b_ast_cleanup:rewrite_event_b_comprehension_set(Ids,E,P,Type, NewExpression)},!,
    pp_expr(NewExpression,Prio,LimitReached).
pp_expr1(comprehension_set([I1,I2,_I3],P),_,Info,500,LimitReached) -->
    /* This comprehension set was a projection function (prj1/prj2) */
    {member_in_info(was(Prj),Info),(Prj=prj1;Prj=prj2),!,
     lookup_membership(I1,P,Set1), lookup_membership(I2,P,Set2)},
    ppatom(Prj),"(",pp_expr(Set1,_,LimitReached),",",pp_expr(Set2,_),")".
pp_expr1(union(b(event_b_identity,Type,_), b(closure(Rel),Type,_)),_,Info,500,LimitReached) -->
    /* closure(Rel) = id \/ closure1(Rel) */
    {member_in_info(was(reflexive_closure),Info)},!,
    "closure(",pp_expr(Rel,_,LimitReached),")".
pp_expr1(comprehension_set([_],_),_,Info,500,_LimitReached) -->
    {memberchk_in_info(freetype(P),Info),!},ppatom(P).
% used instead of constants(Expr,Symbol) case below:
pp_expr1(greater_equal(A,Y),Type,Info,Prio,LimitReached) --> % x:NATURAL was rewritten to x>=0, see test 499, 498
    {memberchk_in_info(was(member(A,B)),Info), Y = b(integer(_),_,_)},
    pp_expr1(member(A,B),Type,Info,Prio,LimitReached).
pp_expr1(comprehension_set([TID],b(B,_,_)),Type,Info,Prio,LimitReached) -->
    {memberchk_in_info(was(integer_set(S)),Info)},
    {S='INTEGER' -> B=truth
     ; get_texpr_id(TID,ID),B=greater_equal(TID2,b(integer(I),_,_)), get_texpr_id(TID2,ID),
       (I=0 -> S='NATURAL' ; I=1,S='NATURAL1')}, % TO DO: check bounds
    !,
    pp_expr1(integer_set(S),Type,Info,Prio,LimitReached).
pp_expr1(interval(b(A,_,_),B),Type,Info,Prio,LimitReached) -->
    {memberchk_in_info(was(integer_set(S)),Info)},
    {B=b(max_int,integer,_)}, % TO DO ? allow value(int(Mx))
    {A=min_int -> S='INT' ; A=integer(0) -> S='NAT' ; A=integer(1),S='NAT1'},
    !,
    pp_expr1(integer_set(S),Type,Info,Prio,LimitReached).
pp_expr1(falsity,_,Info,Prio,LimitReached) --> {memberchk_in_info(was(Pred),Info)},!,
    ({(unicode_mode ; latex_mode)}
     -> {translate_in_mode(falsity,'falsity',Symbol)},
         ppatom(Symbol),
         ({get_preference(pp_propositional_logic_mode,true)} -> {true}
              ; " ", enter_comment, " ", pp_expr2(Pred,Prio,LimitReached), " ", exit_comment)
     ; enter_comment, " falsity ",exit_comment, " ",
       pp_expr2(Pred,Prio,LimitReached)). % Pred is not wrapped
pp_expr1(truth,_,Info,Prio,LimitReached) --> {memberchk_in_info(was(Pred),Info)},!,
    ({(unicode_mode ; latex_mode)}
     -> {translate_in_mode(truth,'truth',Symbol)},
        ppatom(Symbol),
        ({get_preference(pp_propositional_logic_mode,true)} -> {true}
         ; " ",enter_comment, " ", pp_expr2(Pred,Prio,LimitReached), " ", exit_comment)
     ; enter_comment, " truth ", exit_comment, " ",
       pp_expr2(Pred,Prio,LimitReached)). % Pred is not wrapped
    % TO DO: do this for other expressions as well; but then we have to ensure that ast_cleanup generates complete was(_) infos
% :- load_files(library(system), [when(compile_time), imports([environ/2])]). % directive moved above to avoid Spider warning
pp_expr1(event_b_identity,Type,_Info,500,_LimitReached) -->
    {\+ eventb_translation_mode},  %{atelierb_mode(Mode), Mode == pp},
    {is_set_type(Type,couple(ElType,ElType))},
    !,
     "id(", {pretty_type(ElType,S)},ppatom(S), ")".
:- if(environ(prob_safe_mode,true)).
pp_expr1(exists(Parameters,_),_,Info,_Prio,_LimitReached) -->
   {\+ member_in_info(used_ids(_),Info),
    add_error(translate,'Missing used_ids Info for exists: ',Parameters:Info),fail}.
%pp_expr1(exists(Ids,P1),_,Info,250) --> !, { member_in_info(used_ids(Used),Info)},
%    exists_symbol,pp_expr_ids_in_mode(Ids,LimitReached),
%    {add_normal_typing_predicates(Ids,P1,P)},
%    " /* Used = ", ppterm(Used), " */ ",
%    ".",pp_expr_m(P,221).
:- endif.
pp_expr1(Expr,_,Info,Prio,LimitReached) --> {member_in_info(sharing(ID,Count,_,_),Info),number(Count),Count>1},!,
   "( ",enter_comment," CSE ",ppnumber(ID), ":#", ppnumber(Count),
    ({member_in_info(contains_wd_condition,Info)} -> " (wd) " ;  " "),exit_comment, " ",
    pp_expr2(Expr,Prio,LimitReached), ")".
%pp_expr1(Expr,_,Info,Prio) --> {member_in_info(contains_wd_condition,Info)},!,
%   "( /* (wd) */ ",    pp_expr2(Expr,Prio), ")".
% pp_expr1(Expr,subst,_Info,Prio) --> !, translate_subst2(Expr,Prio). % TO DO: also allow substitutions here
pp_expr1(value(V),Type,_,Prio,LimitReached) --> !,
  {(nonvar(V),V=closure(_,_,_) -> Prio=300 ; Prio=500)}, pp_value_with_type(V,Type,LimitReached).
pp_expr1(comprehension_set(Ids,P1),_,Info,500,LimitReached) --> !,
    pp_comprehension_set(Ids,P1,Info,LimitReached).
pp_expr1(Expr,_,_Info,Prio,LimitReached) --> pp_expr2(Expr,Prio,LimitReached).

lookup_membership(TId,P,Set) :-
    get_texpr_id(TId,Id),get_texpr_id(Temp,Id),
    create_texpr(member(Temp,Set),pred,_,Member),
    member_in_conjunction(Member,P),!.
lookup_membership(TId,P,b(set_extension([EqValue]),set(T),[])) :-
    get_texpr_id(TId,Id),get_texpr_id(Temp,Id),
    create_texpr(equal(Temp,EqValue),pred,_,Equal),
    get_texpr_type(EqValue,T),
    member_in_conjunction(Equal,P),!.
lookup_membership(TId,_P,atom_string(TS)) :- % atom_string used as wrapper for pp_expr2
    get_texpr_type(TId,Type),pretty_type(Type,TS).
%    add_error(translate,'Failed: ',lookup_membership(TId,P,Set)), Set=unknown.


pp_expr2(Expr,_,_LimitReached) --> {var(Expr)},!,"_".
pp_expr2(_,_,LimitReached) --> {LimitReached==limit_reached},!,"...".
pp_expr2(atom_string(V),500,_) --> !,pp_atom_opt_latex_mathit(V). % hardwired_atom
pp_expr2(global_set(V),500,_) --> !,ppatom_opt_scramble(V).
pp_expr2(freetype_set(V),500,_) --> !,{pretty_freetype(V,P)},ppatom_opt_scramble(P).
pp_expr2(lazy_lookup_expr(I),500,_) --> !, ppatom_opt_scramble(I).
pp_expr2(lazy_lookup_pred(I),500,_) --> !, ppatom_opt_scramble(I).
pp_expr2(identifier(I),500,_) --> !,
    {( I=op(Id) -> true; I=Id)},
    ( {atomic(Id)} -> ({translated_identifier(Id,TId)},ppatom(TId))
    ; {otherwise} ->
        "'",ppterm(Id), "'").
pp_expr2(integer(N),500,_) --> !, ppnumber(N).
pp_expr2(integer_set(S),500,_) --> !,
    {integer_set_mapping(S,T)},ppatom(T).
pp_expr2(string(S),500,_) --> !, string_start_symbol, ppstring_opt_scramble(S), string_end_symbol.
pp_expr2(set_extension(Ext),500,LimitReached) --> !, {set_brackets(L,R)},
    pp_expr_wrap_l(L,Ext,R,LimitReached).
pp_expr2(sequence_extension(Ext),500,LimitReached) --> !,
    pp_begin_sequence,
    ({get_preference(translate_print_cs_style_sequences,true)} -> pp_expr_l_sep(Ext,"",LimitReached)
      ; pp_expr_l_sep(Ext,",",LimitReached)),
    pp_end_sequence.
pp_expr2(assign(LHS,RHS),10,LimitReached) --> !,
    pp_expr_wrap_l(',',LHS,'',LimitReached), ":=", pp_expr_wrap_l(',',RHS,'',LimitReached).
pp_expr2(assign_single_id(LHS,RHS),10,LimitReached) --> !, pp_expr2(assign([LHS],[RHS]),10,LimitReached).
pp_expr2(parallel(RHS),10,LimitReached) --> !,
    pp_expr_wrap_l('||',RHS,'',LimitReached).
pp_expr2(sequence(RHS),10,LimitReached) --> !,
    pp_expr_wrap_l(';',RHS,'',LimitReached).
pp_expr2(event_b_comprehension_set(Ids,E,P1),500,LimitReached) --> !, % normally conversion above should trigger; this is if we call pp_expr for untyped expressions
    pp_event_b_comprehension_set(Ids,E,P1,LimitReached).
pp_expr2(recursive_let(Id,S),500,LimitReached) --> !,
    ({eventb_translation_mode} -> "" % otherwise we get strange characters in Rodin
      ; enter_comment," recursive ID ", pp_expr(Id,_,LimitReached), " ", exit_comment),
    pp_expr(S,_,LimitReached).
pp_expr2(image(A,B),300,LimitReached) --> !,
    pp_expr_m(A,249,LimitReached),"[", % was 0; but we may have to bracket A; e.g., f <| {2} [{2}] is not ok; 250 is priority of lambda
    pp_expr_m(B,0,LimitReached),"]". % was 500, now set to 0: we never need an outer pair of () !?
pp_expr2(function(A,B),300,LimitReached) --> !,
    pp_expr_m(A,249,LimitReached), % was 0; but we may have to bracket A; e.g., f <| {2} (2) is not ok; 250 is priority of lambda
    pp_function_left_bracket,
    pp_expr_m(B,0,LimitReached), % was 500, now set to 0: we never need an outer pair of () !?
    pp_function_right_bracket.
pp_expr2(definition(A,B),300,LimitReached) --> !, % definition call; usually inlined,...
    ppatom(A),
    pp_function_left_bracket,
    pp_expr_l_sep(B,",",LimitReached),
    pp_function_right_bracket.
pp_expr2(operation_call_in_expr(A,B),300,LimitReached) -->  !,
    pp_expr_m(A,249,LimitReached),
    pp_function_left_bracket,
    pp_expr_l_sep(B,",",LimitReached),
    pp_function_right_bracket.
pp_expr2(enumerated_set_def(GS,ListEls),200,LimitReached) --> !,  % for pretty printing enumerate set defs
    {reverse(ListEls,RLE)}, /* they have been inserted in inverse order */
    ppatom_opt_scramble(GS), "=", pp_expr_wrap_l('{',RLE,'}',LimitReached).
pp_expr2(forall(Ids,D1,P),Prio,LimitReached) --> !,
    ({eventb_translation_mode} -> {Prio=60} ; {Prio=250}), % in Rodin forall/exists cannot be mixed with &, or, <=>, ...
    forall_symbol,pp_expr_ids_in_mode(Ids,LimitReached),
    {add_normal_typing_predicates(Ids,D1,D)},
    dot_symbol,pp_expr_m(b(implication(D,P),pred,[]),221,LimitReached).
pp_expr2(exists(Ids,P1),Prio,LimitReached) --> !,
    ({eventb_translation_mode} -> {Prio=60} ; {Prio=250}),
    exists_symbol,pp_expr_ids_in_mode(Ids,LimitReached),
    {add_normal_typing_predicates(Ids,P1,P)},
    dot_symbol,
    ({eventb_translation_mode} -> {MinPrio=29} ; {MinPrio=500}), % used to be 221, but #x.x>7 or #x.not(...) are not parsed by Atelier-B or ProB, x.x and x.not are parsed as composed identifiers
    pp_expr_m(P,MinPrio,LimitReached).
pp_expr2(record_field(R,I),250,LimitReached) --> !,
    pp_expr_m(R,251,LimitReached),"'",ppatom_opt_scramble(I).
pp_expr2(rec(Fields),500,LimitReached) --> !,
    {function_like_in_mode(rec,Symbol)},
    ppatom(Symbol), "(",pp_expr_fields(Fields,LimitReached),")".
pp_expr2(struct(Rec),500,LimitReached) -->
    {get_texpr_expr(Rec,rec(Fields)),Val=false ; get_texpr_expr(Rec,value(rec(Fields)))},!,
    {function_like_in_mode(struct,Symbol)},
    ppatom(Symbol), "(",
    ({Val==false} -> pp_expr_fields(Fields,LimitReached)
     ; pp_value_l(Fields,',',LimitReached)),
    ")".
pp_expr2(freetype_case(FT,L,Expr),500,LimitReached) --> !,
    pp_freetype_term('__is_ft_case',FT,L,Expr,LimitReached).
pp_expr2(freetype_constructor(_FT,L,Expr),500,LimitReached) --> !,
    ppatom_opt_scramble(L),ppatom('('),pp_expr(Expr,_,LimitReached),ppatom(')').
pp_expr2(freetype_destructor(FT,L,Expr),500,LimitReached) --> !,
    pp_freetype_term('__ft~',FT,L,Expr,LimitReached).
pp_expr2(let_predicate(Ids,Exprs,P),1,LimitReached) --> !,
    pp_expr_let_exists(Ids,Exprs,P,LimitReached). % instead of pp_expr_let
pp_expr2(let_expression(Ids,Exprs,P),1,LimitReached) --> !,
    pp_expr_let(Ids,Exprs,P,LimitReached).
pp_expr2(let_expression_global(Ids,Exprs,P),1,LimitReached) --> !, " /", "* global *", "/ ",
    pp_expr_let(Ids,Exprs,P,LimitReached).
pp_expr2(lazy_let_pred(Id,Expr,P),Pr,LimitReached) --> !, pp_expr2(lazy_let_expr(Id,Expr,P),Pr,LimitReached).
pp_expr2(lazy_let_subst(Id,Expr,P),Pr,LimitReached) --> !, pp_expr2(lazy_let_expr(Id,Expr,P),Pr,LimitReached).
pp_expr2(lazy_let_expr(Id,Expr,P),1,LimitReached) --> !,
    pp_expr_let([Id],[Expr],P,LimitReached).
pp_expr2(norm_conjunct(Cond,[]),1,LimitReached) --> !, % norm_conjunct: flattened version generated by b_interpreter_check,...
    "( ",pp_expr(Cond,_,LimitReached), ")".
pp_expr2(norm_conjunct(Cond,[H|T]),1,LimitReached) --> !,
    "( ",pp_expr(Cond,_,LimitReached), ") ", and_symbol, " (", pp_expr2(norm_conjunct(H,T),_,LimitReached), ")".
pp_expr2(assertion_expression(Cond,_Msg,Expr),1,LimitReached) --> !,
    " ", enter_comment, " ASSERT ",
    pp_expr_m(Cond,30,LimitReached),
    " IN ", exit_comment,
    pp_expr_m(Expr,30,LimitReached).
%pp_expr2(assertion_expression(Cond,_Msg,Expr),1) --> !,
%    "__ASSERT ",pp_expr_m(Cond,30),
%    " IN ", pp_expr_m(Expr,30).
pp_expr2(partition(S,Elems),500,LimitReached) -->
    {eventb_translation_mode ;
     \+ atelierb_mode(_), length(Elems,Len), Len>50 % we need to print a quadratic number of disjoints
      },!,
    "partition(",pp_expr(S,_,LimitReached),
    ({Elems=[]} -> ")" ; pp_expr_wrap_l(',',Elems,')',LimitReached)).
pp_expr2(partition(S,Elems),500,LimitReached) --> !,
    "(",pp_expr(S,_,LimitReached), " = ",
     ({Elems=[]} -> "{})"
      ; pp_expr_l_sep(Elems,"\\/",LimitReached), pp_all_disjoint(Elems,LimitReached),")").
pp_expr2(if_then_else(If,Then,Else),1,LimitReached) --> {animation_minor_mode(z)},!,
    "\\IF ",pp_expr_m(If,30,LimitReached),
    " \\THEN ",pp_expr_m(Then,30,LimitReached),
    " \\ELSE ",pp_expr_m(Else,30,LimitReached).
%pp_expr2(if_then_else(If,Then,Else),1) --> {unicode_mode},!,
%    "if ",pp_expr_m(If,30), " then ",pp_expr_m(Then,30), " else ",pp_expr_m(Else,30).
pp_expr2(if_then_else(If,Then,Else),Prio,LimitReached) --> {atelierb_mode(_)},!,
    % print IF-THEN-ELSE using a translation that Atelier-B can understand:
    {get_texpr_type(Then,Type),
     ARG = b(boolean_true,integer,[]), AT=boolean, % we could use unit type or BOOL here
     gen_fresh_id_if_necessary('_zzzz_unary',b(if_then_else(If,Then,Else),Type,[]),AID1),
     gen_fresh_id_if_necessary('_zzzz_binary',b(if_then_else(If,Then,Else),Type,[]),AID2),
     Id1 = b(identifier(AID1),AT,[]), % a dummy argument
     Id2 = b(identifier(AID2),Type,[]), % The result
     Eq1 = b(equal(Id2,Then),pred,[]), Eq2 = b(equal(Id2,Else),pred,[]),
     Pred1 = b(implication(If,Eq1),pred,[]),
     Pred2 = b(implication(b(negation(If),pred,[]),Eq2),pred,[]),
     Pred = b(conjunct(Pred1,Pred2),pred,[]),
     FUN = b(comprehension_set([Id1,Id2],Pred),set(couple(AT,Type)),[]),
     Expr = b(function(FUN,ARG),Type,[])},
     % construct {d,x| If => x=Then & not(if) => x=Else}(TRUE)
     pp_expr(Expr,Prio,LimitReached).
pp_expr2(if_then_else(If,Then,Else),1,LimitReached) -->
    pp_atom_opt_mathit('IF'),pp_space, % "IF ",
    pp_expr_m(If,30,LimitReached),
    pp_space, pp_atom_opt_mathit('THEN'),pp_space, %" THEN ",
    pp_expr_m(Then,30,LimitReached),
    pp_space, pp_atom_opt_mathit('ELSE'),pp_space, %" ELSE ",
    pp_expr_m(Else,30,LimitReached),
    pp_space, pp_atom_opt_mathit('END'). %" END"
pp_expr2(kodkod(Id,Identifiers),300,LimitReached) -->
    "KODKOD_CALL(",ppnumber(Id),": ",pp_expr_ids(Identifiers,LimitReached),")".
pp_expr2(Expr,500,_) -->
    {constants_in_mode(Expr,Symbol)},!,ppatom(Symbol).
pp_expr2(equal(A,B),Prio,LimitReached) -->
    {get_preference(pp_propositional_logic_mode,true), % a mode for printing propositional logic formuli
     is_boolean_value(B,BV),
     get_texpr_id(A,_)},!,
    ({BV=pred_true} -> pp_expr(A,Prio,LimitReached)
       ; pp_expr2(negation(b(equal(A,b(boolean_true,boolean,[])),pred,[])),Prio,LimitReached)).
pp_expr2(Expr,Prio,LimitReached) -->
    {functor(Expr,F,1),
    unary_prefix(F,Symbol,Prio),!,
    arg(1,Expr,Arg),APrio is Prio+1},
    ppatom(Symbol), " ",
    pp_expr_m(Arg,APrio,LimitReached).
pp_expr2(Expr,500,LimitReached) -->
    {functor(Expr,F,1),
    unary_prefix_parentheses(F,Symbol),!,
    arg(1,Expr,Arg)},
    pp_atom_opt_latex(Symbol), "(", pp_expr(Arg,_,LimitReached), ")".
pp_expr2(Expr,Prio,LimitReached) -->
    {functor(Expr,F,1),
    unary_postfix_in_mode(F,Symbol,Prio),!,
    arg(1,Expr,Arg),APrio is Prio+1},
    pp_expr_m(Arg,APrio,LimitReached),ppatom(Symbol).
pp_expr2(power_of(Left,Right),Prio,LimitReached) --> {latex_mode},!, % special case, as we need to put {} around RHS
    {Prio=200, LPrio is Prio+1, RPrio = Prio},
    pp_expr_m(Left,LPrio,LimitReached),
    "^{",
    pp_expr_m(Right,RPrio,LimitReached),
    "}".
pp_expr2(Expr,OPrio,LimitReached) -->
    {functor(Expr,F,2),
    binary_infix_in_mode(F,Symbol,Prio,Ass),!,
    arg(1,Expr,Left),
    arg(2,Expr,Right),
    ( Ass = left, binary_infix_symbol(Left,Symbol)  -> LPrio is Prio-1, RPrio is Prio+1
    ; Ass = right, binary_infix_symbol(Right,Symbol) -> LPrio is Prio+1, RPrio is Prio-1
    ; otherwise   -> LPrio is Prio+1, RPrio is Prio+1)},
    % Note: Prio+1 is actually not necessary, Prio would be sufficient, as pp_expr_m uses a strict comparison <
    ({always_surround_by_parentheses(F)} -> "(",{OPrio=1000} ; {OPrio=Prio}),
    pp_expr_m(Left,LPrio,LimitReached),
    " ", ppatom(Symbol), " ",
    pp_expr_m(Right,RPrio,LimitReached),
    ({always_surround_by_parentheses(F)} -> ")" ; []).
pp_expr2(first_of_pair(X),500,LimitReached) -->  {get_texpr_type(X,couple(From,To))},!,
    "prj1(", % TO DO: Latex version
    ({eventb_translation_mode} -> "" % no need to print types in Event-B
      ; {pretty_type(From,FromT), pretty_type(To,ToT)},
        pp_atom_opt_latex(FromT), ",", pp_atom_opt_latex(ToT),
        ")("
    ),
    pp_expr(X,_,LimitReached),")".
pp_expr2(second_of_pair(X),500,LimitReached) -->  {get_texpr_type(X,couple(From,To))},!,
    "prj2(", % TO DO: Latex version
    ({eventb_translation_mode} -> "" % no need to print types in Event-B
      ; {pretty_type(From,FromT), pretty_type(To,ToT)},
        pp_atom_opt_latex(FromT), ",", pp_atom_opt_latex(ToT),
        ")("
    ),
    pp_expr(X,_,LimitReached),")".
pp_expr2(Call,500,LimitReached) --> {external_call(Call,Symbol,Args)},!,
    pp_external_call(Symbol,Args,LimitReached).
pp_expr2(card(A),500,LimitReached) --> {latex_mode, get_preference(latex_pp_greek_ids,true)},!,
    "|",pp_expr_m(A,0,LimitReached),"|".
pp_expr2(Expr,500,LimitReached) -->
    {functor(Expr,F,_),
    function_like_in_mode(F,Symbol),!,
    Expr =.. [F|Args]},
    ppatom(Symbol),pp_expr_wrap_l('(',Args,')',LimitReached).
pp_expr2(Expr,250,LimitReached) -->
    {functor(Expr,F,3),
    quantified_in_mode(F,Symbol),
    Expr =.. [F,Ids,P1,E],
    !,
    add_normal_typing_predicates(Ids,P1,P)},
    ppatom(Symbol),pp_expr_ids(Ids,LimitReached),".(",
    pp_expr_m(P,11,LimitReached),pp_such_that_bar(E),
    pp_expr_m(E,11,LimitReached),")".
pp_expr2(Expr,Prio,_LimitReached) -->
    {functor(Expr,F,N),
     format('Unknown functor ~w/~w in pp_expr2~n expression: ~w~n',[F,N,Expr]),
     %add_internal_error('Unknown Expression: ',pp_expr2(Expr,Prio)),
     Prio=20},
        ppterm(Expr).


pp_external_call('STRING_LENGTH',[Arg],LimitReached) -->
    {get_preference(allow_sequence_operators_on_strings,true)},!,
    pp_expr2(size(Arg),_,LimitReached).
pp_external_call('STRING_APPEND',[Arg1,Arg2],LimitReached) -->
    {get_preference(allow_sequence_operators_on_strings,true)},!,
    pp_expr2(concat(Arg1,Arg2),_,LimitReached).
pp_external_call('STRING_CONC',[Arg1],LimitReached) -->
    {get_preference(allow_sequence_operators_on_strings,true)},!,
    pp_expr2(general_concat(Arg1),_,LimitReached).
pp_external_call(Symbol,Args,LimitReached) -->
    ({invisible_external_pred(Symbol)}
     -> "1=1 /* ",pp_expr_m(atom_string(Symbol),20,LimitReached),pp_expr_wrap_l('(',Args,') */',LimitReached)
      ; pp_expr_m(atom_string(Symbol),20,LimitReached),
        pp_expr_wrap_l('(',Args,')',LimitReached) % pp_expr_wrap_l('/*EXT:*/(',Args,')')
     ).

invisible_external_pred('LEQ_SYM_BREAK'). % just for symmetry breaking foralls,...
external_call(external_function_call(Symbol,Args),Symbol,Args).
external_call(external_pred_call(Symbol,Args),Symbol,Args).
external_call(external_subst_call(Symbol,Args),Symbol,Args).

pp_all_disjoint([H1,H2],LimitReached) --> !, " ",and_symbol," ", pp_disjoint(H1,H2,LimitReached).
pp_all_disjoint([H1|T],LimitReached) --> pp_all_disjoint_aux(T,H1,LimitReached), pp_all_disjoint(T,LimitReached).
pp_all_disjoint([],_) --> "".

pp_all_disjoint_aux([],_,_) --> "".
pp_all_disjoint_aux([H2|T],H1,LimitReached) --> " ",and_symbol," ",
    pp_disjoint(H1,H2,LimitReached), pp_all_disjoint_aux(T,H1,LimitReached).

pp_disjoint(H1,H2,LimitReached) --> pp_expr(H1,_), "/\\", pp_expr(H2,_,LimitReached), " = {}".


:- use_module(bsyntaxtree,[is_a_disjunct/3]).

pp_comprehension_set([ID1|T],Body,Info,LimitReached) --> {is_a_disjunct(Body,B1,B2),
    get_last(T,ID1,_FrontIDs,LastID),
    is_lambda_result_id(LastID,_Suffix)},!, % we seem to have the union of two lambda expressions
    "(", pp_comprehension_set([ID1|T],B1,Info,LimitReached),
    " \\/ ", pp_comprehension_set([ID1|T],B2,Info,LimitReached), ")".
pp_comprehension_set([b(identifier('_pred_'),integer,_),
                      b(identifier('_lambda_result_'),integer,_)],Body,_,_) -->
   {Body = b(equal(LR,T),pred,_),
    LR = b(identifier('_lambda_result_'),integer,_),
    T = b(minus(ARG,One),integer,_),
    One = b(integer(1),integer,_),
    ARG = b(identifier('_pred_'),integer,_)},
   !,
   "pred".
pp_comprehension_set([b(identifier('_succ_'),integer,_),
                      b(identifier('_lambda_result_'),integer,_)],Body,_,_) -->
   {Body = b(equal(LR,T),pred,_),
    LR = b(identifier('_lambda_result_'),integer,_),
    T = b(add(ARG,One),integer,_),
    One = b(integer(1),integer,_),
    ARG = b(identifier('_succ_'),integer,_)},
   !,
   "succ".
pp_comprehension_set([ID1|T],Body,Info,LimitReached) --> {get_last(T,ID1,FrontIDs,LastID),
    is_lambda_result_id(LastID,Suffix),
     % nl, print(lambda(Body,T,ID1)),nl,
    (is_an_equality(Body,From,ToExpr) -> LambdaBody = b(truth,pred,[])
      ; is_a_conjunct(Body,LambdaBody,Equality),
        is_an_equality(Equality,From,ToExpr)),
    is_lambda_result_id(From,Suffix)},!,
    {add_normal_typing_predicates(FrontIDs,LambdaBody,TLambdaBody)},
    ({eventb_translation_mode} -> "(" ; ""), % put brackets around the lambda in Rodin
    pp_annotations(Info,Body),
    lambda_symbol, % "%"
    pp_lambda_identifiers(FrontIDs,LimitReached),
    ".",
    ({eventb_translation_mode} -> {IPrio=30} ; {IPrio=11}, "("), % In Rodin it is not ok to write (P|E)
    pp_expr_m(TLambdaBody,IPrio,LimitReached), % Check 11 against prio of . and |
    pp_such_that_bar(ToExpr),
    pp_expr_m(ToExpr,IPrio,LimitReached),
    ")".
pp_comprehension_set(Ids,P1,Info,LimitReached) -->
    pp_annotations(Info,P1),
    left_set_bracket,
    pp_expr_l_pair_in_mode(Ids,LimitReached),
    {add_normal_typing_predicates(Ids,P1,P)},
    pp_such_that_bar(P),
    pp_expr_m(P,11,LimitReached),
    right_set_bracket.

pp_annotations(INFO,_) --> {member(prob_annotation('SYMBOLIC'),INFO)},!,
   "/*@symbolic*/ ".
pp_annotations(_,b(_,_,INFO)) --> {member(prob_annotation('SYMBOLIC'),INFO)},!,
   "/*@symbolic*/ ".
% TO DO: maybe also print other annotations like memoize, recursive ?
pp_annotations(_,_) --> "".

pp_event_b_comprehension_set(Ids,E,P1,LimitReached) -->
    left_set_bracket,pp_expr_l_pair_in_mode(Ids,LimitReached),
    {add_normal_typing_predicates(Ids,P1,P)},
    pp_such_that_bar(P),pp_expr_m(P,11,LimitReached),
    " . ", pp_expr_m(E,11,LimitReached),right_set_bracket.

pp_lambda_identifiers([H1,H2|T],LimitReached) --> {\+ eventb_translation_mode},!,
    "(",pp_expr_l([H1,H2|T],LimitReached),")".
pp_lambda_identifiers(L,LimitReached) --> pp_expr_l_pair_in_mode(L,LimitReached).

pp_such_that_bar(_) --> {latex_mode},!, "\\mid ".
pp_such_that_bar(b(unary_minus(_),_,_)) --> !, " | ".  % otherwise AtelierB complains about illegal token |-
pp_such_that_bar(_Next) --> "|".
pp_such_that_bar --> {latex_mode},!, "\\mid ".
pp_such_that_bar --> "|".

is_an_equality(b(equal(A,B),_,_),A,B).

integer_set_mapping(A,B) :- unicode_mode, unicode_translation(A,B),!.
integer_set_mapping(A,B) :- latex_mode, latex_integer_set_translation(A,B),!.
integer_set_mapping(A,B) :- atelierb_mode(Mode), Mode == pp, atelierb_pp_translation(A,B),!.
integer_set_mapping(A,B) :-
    eventb_translation_mode, eventb_integer_mapping(A,B),!.
integer_set_mapping(ISet,Res) :- atomic(ISet),!,Res=ISet.
integer_set_mapping(_ISet,'integer_set(??)').

eventb_integer_mapping('INTEGER','INT').
eventb_integer_mapping('NATURAL','NAT').
eventb_integer_mapping('NATURAL1','NAT1').

:- dynamic comment_level/1.
reset_pp :- retractall(comment_level(_)).
enter_comment --> {retract(comment_level(N))},!, "(*", {N1 is N+1, assert(comment_level(N1))}.
enter_comment --> "/*", {assert(comment_level(1))}.
exit_comment --> {retract(comment_level(N))},!,
                  ({N>1} -> "*)", {N1 is N-1, assert(comment_level(N1))} ; "*/").
exit_comment --> "*/", {add_internal_error('Unmatched closing comment:',exit_comment)}.
% TO DO: ensure reset_pp is called when starting to pretty print, in case timeout occurs in previous pretty prints

%get_last([b(identifier(_lambda_result_10),set(couple(integer,set(couple(integer,integer)))),[])],b(identifier(i),integer,[]),[b(identifier(i),integer,[])],b(identifier(_lambda_result_10),set(couple(integer,set(couple(integer,integer)))),[]))

get_last([],Last,[],Last).
get_last([H2|T],H1,[H1|LT],Last) :- get_last(T,H2,LT,Last).

pp_expr_wrap_l(Pre,Expr,Post,LimitReached) -->
    ppatom(Pre),pp_expr_l(Expr,LimitReached),ppatom(Post).
pp_freetype_term(Term,FT,L,Expr,LimitReached) -->
    {pretty_freetype(FT,P)},
    ppatom(Term),"(",ppatom_opt_scramble(P),",",
    ppatom(L),",",pp_expr_m(Expr,500,LimitReached),")".

% print a list of expressions, seperated by commas
pp_expr_l_pair_in_mode(List,LimitReached) --> {eventb_translation_mode},!,
    {maplet_symbol(MapletStr,[])},
    pp_expr_l_sep(List,MapletStr,LimitReached).
pp_expr_l_pair_in_mode(List,LimitReached) --> pp_expr_l_sep(List,",",LimitReached).
pp_expr_l(List,LimitReached) --> pp_expr_l_sep(List,",",LimitReached).
pp_expr_l_sep([Expr],_,LimitReached) --> !,
    pp_expr_m(Expr,0,LimitReached).
pp_expr_l_sep(List,Sep,LimitReached) --> pp_expr_l2(List,Sep,LimitReached).
pp_expr_l2([],_Sep,_) --> !.
pp_expr_l2([Expr|Rest],Sep,LimitReached) -->
    pp_expr_m(Expr,116,LimitReached),pp_expr_l3(Rest,Sep,LimitReached).
pp_expr_l3([],_Sep,_) --> !.
pp_expr_l3(Rest,Sep,LimitReached) -->
    Sep,pp_expr_l2(Rest,Sep,LimitReached).

% print the fields of a record
pp_expr_fields([field(Name,Expr)],LimitReached) --> !,
    ppatom_opt_scramble(Name),":",pp_expr_m(Expr,120,LimitReached).
pp_expr_fields(Fields,LimitReached) -->
    pp_expr_fields2(Fields,LimitReached).
pp_expr_fields2([],_) --> !.
pp_expr_fields2([field(Name,Expr)|Rest],LimitReached) -->
    ppatom_opt_scramble(Name),":",
    pp_expr_m(Expr,116,LimitReached),
    pp_expr_fields3(Rest,LimitReached).
pp_expr_fields3([],_) --> !.
pp_expr_fields3(Rest,LimitReached) -->
    ",",pp_expr_fields2(Rest,LimitReached).

% TO DO: test more fully; identifiers seem to be wrapped in brackets
pp_expr_let_exists(Ids,Exprs,P,LimitReached) -->
    exists_symbol,
    ({eventb_translation_mode} ->  % otherwise we get strange characters in Rodin, no (.) allowed in Rodin
        pp_expr_ids_in_mode(Ids,LimitReached),
        ".("
     ; " /* LET */ (",
        pp_expr_l_pair_in_mode(Ids,LimitReached),
       ").("
     ),
     pp_expr_let_pred_exprs(Ids,Exprs,LimitReached),
    " ",and_symbol," ", pp_expr_m(P,40,LimitReached), ")".

pp_expr_let_pred_exprs([],[],_) --> !.
pp_expr_let_pred_exprs([Id|Irest],[Expr|Erest],LimitReached) -->
    " ",pp_expr_let_id(Id,LimitReached),
    "=",pp_expr_m(Expr,400,LimitReached),
    ( {Irest=[]} -> [] ; " ", and_symbol),
    pp_expr_let_pred_exprs(Irest,Erest,LimitReached).

% print a LET expression (no B syntax, used in translated Z
% models)
pp_expr_let(Ids,Exprs,P,LimitReached) -->
    "LET ", pp_expr_ids_in_mode(Ids,LimitReached),
    " BE ", pp_expr_let_pred_exprs(Ids,Exprs,LimitReached),
    " IN ",pp_expr_m(P,5,LimitReached),
    " END".

pp_expr_let_id(ID,LimitReached) --> {atomic(ID),!, print(unwrapped_let_id(ID)),nl},
    pp_expr_m(identifier(ID),500,LimitReached).
pp_expr_let_id(ID,LimitReached) --> pp_expr_m(ID,499,LimitReached).

% print a list of identifiers
pp_expr_ids_in_mode([],_) --> !.
pp_expr_ids_in_mode(Ids,LimitReached) --> {eventb_translation_mode ; Ids=[_]},!,
    pp_expr_l(Ids,LimitReached). % no (.) allowed in Event-B; not necessary in B if only one id
pp_expr_ids_in_mode(Ids,LimitReached) --> "(",pp_expr_l(Ids,LimitReached),")".
pp_expr_ids([],_) --> !.
pp_expr_ids(Ids,LimitReached) -->
%    ( {Ids=[Id]} -> pp_expr_m(Id,221)
%    ; {otherwise} ->
    "(",pp_expr_l(Ids,LimitReached),")".


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% pretty print types for error messages

pretty_type(Type,String) :-
    pretty_type_l([Type],[String]).

pretty_type_l(Types,Strings) :-
    extract_vartype_names(Types,N),
    pretty_type2_l(Types,N,Strings).
pretty_type2_l([],_,[]).
pretty_type2_l([T|TRest],Names,[S|SRest]) :-
    pretty_type2(T,Names,noparen,S),
    pretty_type2_l(TRest,Names,SRest).

:- if(current_prolog_flag(dialect,sicstus)).
:- if((current_prolog_flag(version_data,sicstus(4,X,_,_,_)),X<3)).
:- use_module(library(terms),[term_variables/2]). % is built-in in SICSTUS 4.3
:- endif.
:- endif.
extract_vartype_names(Types,names(Variables,Names)) :-
    term_variables(Types,Variables),
    name_variables(Variables,1,Names).

pretty_type2(X,names(Vars,Names),_,Name) :- var(X),!,exact_member_lookup(X,Name,Vars,Names).
pretty_type2(any,_,_,'?').
pretty_type2(set(T),N,_,Text) :- nonvar(T),T=couple(A,B),!,
    pretty_type2(A,N,paren,AT), pretty_type2(B,N,paren,BT),
    binary_infix_in_mode(relations,Symbol,_,_), % <->
    ajoin(['(',AT,Symbol,BT,')'],Text).
pretty_type2(set(T),N,_,Text) :-
    pretty_type2(T,N,noparen,TT), function_like_in_mode(pow_subset,POW),
    ajoin([POW,'(',TT,')'],Text).
pretty_type2(seq(T),N,_,Text) :-
    pretty_type2(T,N,noparen,TT), ajoin(['seq(',TT,')'],Text).
pretty_type2(couple(A,B),N,Paren,Text) :-
    pretty_type2(A,N,paren,AT),pretty_type2(B,N,paren,BT),
    binary_infix_in_mode(cartesian_product,Cart,_,_),
    ajoin([AT,Cart,BT],Prod),
    ( Paren == noparen ->
      Text = Prod
    ; otherwise ->
      ajoin(['(',Prod,')'],Text)).
pretty_type2(string,_,_,'STRING').
pretty_type2(integer,_,_,Atom) :- integer_set_mapping('INTEGER',Atom).
pretty_type2(boolean,_,_,'BOOL').
pretty_type2(global(G),_,_,A) :- ajoin([G],A).
pretty_type2(freetype(Id),N,_,A) :- pretty_freetype2(Id,N,A).
pretty_type2(pred,_,_,predicate).
pretty_type2(subst,_,_,substitution).
pretty_type2(constant(List),_N,_,A) :- ajoin_with_sep(List,',',P), ajoin(['{',P,'}'],A).
pretty_type2(record(Fields),N,_,Text) :-
    pretty_type_fields(Fields,N,FText),
    ajoin(['struct(',FText,')'],Text).
pretty_type2(op(Params,Results),N,_,Text) :-
    pretty_type_l(Params,N,PText),
    ( nonvar(Results),Results=[] ->
        ajoin(['operation(',PText,')'],Text)
    ; otherwise ->
        pretty_type_l(Results,N,RText),
        ajoin([RText,'<--operation(',PText,')'],Text) ).
pretty_type2(definition(DefType,_,_),_,_,DefType).
pretty_type2(witness,_,_,witness).
pretty_type_l(L,_,'...') :- var(L),!.
pretty_type_l([],_,'') :- !.
pretty_type_l([E|Rest],N,Text) :-
    pretty_type2(E,N,noparen,EText),
    ( nonvar(Rest),Rest=[] ->
        EText=Text
    ; otherwise ->
        pretty_type_l(Rest,N,RText),
        ajoin([EText,',',RText],Text)).

pretty_type_fields(L,_,'...') :- var(L),!.
pretty_type_fields([],_,'').
pretty_type_fields([field(Name,Type)|FRest],N,Text) :-
    pretty_type2(Type,N,noparen,TText),
    ptf_seperator(FRest,Sep),
    pretty_type_fields(FRest,N,RestText),
    opt_scramble_id(Name,ScrName),
    ajoin([ScrName,':',TText,Sep,RestText],Text).
ptf_seperator(L,', ') :- var(L),!.
ptf_seperator([],'') :- !.
ptf_seperator(_,', ').

pretty_freetype(Id,A) :-
    extract_vartype_names(Id,N),
    pretty_freetype2(Id,N,A).
pretty_freetype2(Id,_,A) :- var(Id),!,A='_'.
pretty_freetype2(Id,_,A) :- atomic(Id),!,Id=A.
pretty_freetype2(Id,N,A) :-
    Id=..[Name|TypeArgs],
        pretty_type2_l(TypeArgs,N,PArgs),
        ajoin_with_sep(PArgs,',',P),
        ajoin([Name,'(',P,')'],A).

name_variables([],_,[]).
name_variables([_|VRest],Index,[Name|NRest]) :-
    ( nth1(Index,"ABCDEFGHIJKLMNOPQRSTUVWXYZ",C) -> SName = [C]
    ; otherwise ->  number_codes(Index,SName)),
    append("_",SName,CName),atom_codes(Name,CName),
    Next is Index+1,
    name_variables(VRest,Next,NRest).

ppatom(Var) --> {var(Var)},!, ppatom('$VARIABLE').
ppatom(Cmp) --> {compound(Cmp)},!, ppatom('$COMPOUND_TERM').
ppatom(Atom) --> {safe_atom_codes(Atom,Codes)}, ppcodes(Codes).

ppnumber(Number) --> {var(Number)},!,pp_clpfd_variable(Number).
ppnumber(inf) --> !,"inf".
ppnumber(minus_inf) --> !,"minus_inf".
ppnumber(Number) --> {number(Number),number_codes(Number,Codes)},!, ppcodes(Codes).
ppnumber(Number) --> {add_internal_error('Not a number: ',ppnumber(Number,_,_)),fail}.

pp_numberedvar(N) --> "_",ppnumber(N),"_".

pp_clpfd_variable(X) --> "?:",{fd_dom(X,Dom)},write_to_codes(Dom), pp_frozen_info(X).

pp_frozen_info(_X) --> {get_preference(translate_print_frozen_infos,false)},!,[].
pp_frozen_info(X) -->
    ":(",{frozen(X,Goal)},
   write_goal_with_max_depth(Goal),
   ")".

write_goal_with_max_depth((A,B)) --> !, "(",write_goal_with_max_depth(A),
   ", ", write_goal_with_max_depth(B), ")".
write_goal_with_max_depth(Term) --> write_with_max_depth(3,Term).

write_with_max_depth(Depth,Term,S1,S2) :- write_term_to_codes(Term,S1,S2,[max_depth(Depth)]).

ppterm(Term) --> write_to_codes(Term).
ppcodes([],S,S).
ppcodes([C|Rest],[C|In],Out) :- ppcodes(Rest,In,Out).

% for debugging:
:- public b_portray_hook/1.
b_portray_hook(X) :-
    ground(X),
    (is_texpr(X) -> print('{# '),print_bexpr_or_subst(X),print(' #}')).

install_b_portray_hook :- % mainly for the Prolog debugger
    assert(( user:portray(X) :- translate:b_portray_hook(X) )).
remove_b_portray_hook :-
    retractall( user:portray(_) ).


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Pretty-print of Event-B models as classical B

translate_eventb_to_classicalb(EBMachine,AddInfo,Rep) :-
    ( conversion_check(EBMachine) ->
        convert_eventb_classicalb(EBMachine,CBMachine),
        call_cleanup(( set_animation_mode(b), % clear minor mode "eventb"
                       translate_machine(CBMachine,Rep,AddInfo),!),
                       set_animation_minor_mode(eventb))
    ; \+ animation_minor_mode(eventb) -> add_error(translate,'Conversion only applicable to Event-B models')
    ; otherwise ->
        add_error_and_fail(translate,'Conversion not applicable, check if you limited the number of abstract level to 0')
    ).

convert_eventb_classicalb(EBMachine,CBMachine) :-
    select_section(operation_bodies,In,Out,EBMachine,CBMachine1),
    maplist(convert_eventop,In,Out),
    select_section(initialisation,IIn,IOut,CBMachine1,CBMachine),
    convert_event(IIn,[],IOut).
convert_eventop(EBOp,CBOp) :-
    get_texpr_expr(EBOp,operation(Id,[],Args,EBBody)),
    get_texpr_info(EBOp,Info),
    convert_event(EBBody,Args,CBBody),
    % Remove the arguments
    create_texpr(operation(Id,[],[],CBBody),op([],[]),Info,CBOp).
convert_event(TEvent,Parameters,TSubstitution) :-
    get_texpr_expr(TEvent,rlevent(_Id,_Section,_Status,_Parameters,Guard,_Theorems,Actions,_VariableWitnesses,_ParameterWitnesses,_Ums,_Refined)),
    in_parallel(Actions,PAction),
    convert_event2(Parameters,Guard,PAction,TSubstitution).
convert_event2([],Guard,Action,Action) :-
    is_truth(Guard),!.
convert_event2([],Guard,Action,Select) :-
    !,create_texpr(select([When]),subst,[],Select),
    create_texpr(select_when(Guard,Action),subst,[],When).
convert_event2(Parameters,Guard,Action,Any) :-
    create_texpr(any(Parameters,Guard,Action),subst,[],Any).
in_parallel([],Skip) :- !,create_texpr(skip,subst,[],Skip).
in_parallel([A],A) :- !.
in_parallel(Actions,Parallel) :- create_texpr(parallel(Actions),subst,[],Parallel).

conversion_check(Machine) :-
    animation_mode(b),
    animation_minor_mode(eventb),
    get_section(initialisation,Machine,Init),
    get_texpr_expr(Init,rlevent(_Id,_Sec,_St,_Par,_Grd,_Thms,_Act,_VW,_PW,_Ums,[])).

% ------------------------------------------------------------

% divide a B typed expression into columns for CSV export or Table viewing of its values
get_bexpression_column_template(b(couple(A,B),_,_),(AVal,BVal),ColHeaders,Columns) :- !,
    get_bexpression_column_template(A,AVal,AHeaders,AColumns),
    get_bexpression_column_template(B,BVal,BHeaders,BColumns),
    append(AHeaders,BHeaders,ColHeaders),
    append(AColumns,BColumns,Columns).
get_bexpression_column_template(TypedExpr,Value,[ColHeader],[Value]) :-
    translate:translate_bexpression_with_limit(TypedExpr,100,ColHeader).


:- use_module(gensym,[gensym/2]).
gen_fresh_id_if_necessary(Default,Expr,FreshID) :-
    occurs_in_expr(Default,Expr),!,
    gensym('__FRESH_ID__',FreshID). % assumes _Fresh_XXX not used
gen_fresh_id_if_necessary(Default,_,Default).


% a version of member that creates an error when info list not instantiated
member_in_info(X,T) :- var(T),!, add_internal_error('Illegal info field:', member_in_info(X,T)),fail.
member_in_info(X,[X|_]).
member_in_info(X,[_|T]) :- member_in_info(X,T).

memberchk_in_info(X,T) :- var(T),!, add_internal_error('Illegal info field:', memberchk_in_info(X,T)),fail.
memberchk_in_info(X,[X|_]) :- !.
memberchk_in_info(X,[_|T]) :- memberchk_in_info(X,T).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

:- use_module(library(clpfd)).

% print also partially instantiated variables with CLP(FD) Info
print_value_variable(X) :- var(X), !, print(X).
print_value_variable(int(X)) :- print('int('), print_clpfd_variable(X), print(')').
print_value_variable(fd(X,T)) :- print('fd('), print_clpfd_variable(X), print(','), print(T),print(')').
print_value_variable(X) :- print(X).

print_clpfd_variable(X) :- var(X),!,print(X), print(':'), fd_dom(X,Dom), print(Dom), print_frozen_info(X).
print_clpfd_variable(X) :- print(X).

%print_clpfd_variables([]).
%print_clpfd_variables([H|T]) :- print('CLPFD: '),print_clpfd_variable(H), nl, print_clpfd_variables(T).


:- public l_print_frozen_info/1.
l_print_frozen_info([]).
l_print_frozen_info([H|T]) :- print(H), print(' '),
  (var(H) -> print_frozen_info(H) ;
    H=fd_var(V,_) -> print_frozen_info(V) ; true), l_print_frozen_info(T).

print_frozen_info(X) :- frozen(X,Goal), print_frozen_goal(Goal).
print_frozen_goal((A,B)) :- !, print_frozen_goal(A), print(','), print_frozen_goal(B).
print_frozen_goal(prolog:trig_nondif(_A,_B,R,_S)) :- !, frozen(R,G2), print_frozen_goal2(G2).
print_frozen_goal(G) :- print_frozen_goal2(G).
print_frozen_goal2(V) :- var(V),!, print(V).
print_frozen_goal2(true) :- !.
print_frozen_goal2((A,B)) :- !, print_frozen_goal2(A), print(','), print_frozen_goal2(B).
print_frozen_goal2(G) :- print(' :: '), tools_printing:print_term_summary(G).


/* Event-B operators */
translate_eventb_operators([]) --> !.
translate_eventb_operators([Name-Call|Rest]) -->
    translate_eventb_operator(Call,Name),
    translate_eventb_operators(Rest).

translate_eventb_operator(Module:Call,Name) -->
    insertcodes("\n  "),
    indention_codes(In,Out),
    {Call =.. [Functor|Args],atom_concat('pp_',Functor,PPFunctor),
     PPCall =.. [PPFunctor|Args],
     catch( (call(Module:PPCall,Name,In,Out) -> R=ok; R=fail),
            error(_,_),
            R=error),
     ( R = ok -> true
     ; otherwise ->
         print_operator_failure(Name,Module,Functor,In,Out))}.
print_operator_failure(Name,Module,Functor) -->
         ppterm(Name),
         ppcodes(": Operator implemented by "),
         ppatom(Module),ppcodes(":"),ppatom(Functor).

% ---------------------------------------

% translate a predicate into B machine for manipulation
translate_predicate_into_machine(Pred,ResultAtom) :-
   bsyntaxtree:find_typed_identifier_uses(Pred, TUsedIds),
   get_texpr_ids(TUsedIds,UsedIds), print(used(UsedIds)),nl,
   add_typing_predicates(TUsedIds,Pred,TPred),
   set_print_type_infos(all),
   specfile:set_animation_mode(b), % clear eventb minor mode; to do: set back
   translate_bexpression(TPred,PredAtom), %print(res(UsedIds,ResultAtom)),nl,
   set_print_type_infos(none),
   ajoin_with_sep(UsedIds,', ',AllIds),
   bmachine:get_full_b_machine(_Name,BMachine),
   include(relevant_section,BMachine,RelevantSections),
   % TO DO: we could filter out enumerate/deferred sets not occuring in Pred
   translate_section_list(RelevantSections,SetsParas),
   atom_codes(ASP,SetsParas),
   ajoin(['MACHINE UnsatCore',ASP,'CONSTANTS ',AllIds,'\nPROPERTIES\n ',PredAtom,'\nEND\n'],ResultAtom).

relevant_section(deferred_sets/_).
relevant_section(enumerated_elements/_).
relevant_section(parameters/_).

% ---------------------------------------
